import java.util.*;
import java.awt.*;

import sv.kernel.*;

/** 
 * CPlane.java
 * Configuration for 1d
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public final class CPlane extends Configs {

  public boolean connect_point = false;
  public boolean amr = false;
  public boolean color_by_info = false;
  public boolean color_by_set = false;
  public boolean tickmark = false;
  public boolean display_label = false;
  public boolean display_legend = false;
  public String colormap = null;

  public float zoomStartX = 0.0f;
  public float zoomStartY = 0.0f;
  public float zoomEndX = 0.0f;
  public float zoomEndY = 0.0f;
  public float zoomFactorX = 1.0f;
  public float zoomFactorY = 1.0f;
  public int zoomMode = 3;

  public Rectangle legendRect = new Rectangle();

  public String xLabel = "X";
  public String yLabel = "Y"; 
  public Font xLabelFont; 
  public Font yLabelFont; 
  public boolean isXLabelRotate = false; 
  public boolean isYLabelRotate = false; 
  public Point xLabelPoint = new Point();
  public Point yLabelPoint = new Point();

  // default constructor
  public CPlane() {
    String[] fontLists = Toolkit.getDefaultToolkit().getFontList();
    xLabelFont = new Font(fontLists[0], Font.PLAIN, 12);
    yLabelFont = new Font(fontLists[0], Font.PLAIN, 12);
  }


  public CPlane(String[] args) {
     this();
     doConfig(args);
  }

  public CPlane(String arg) {
     this();
     StringTokenizer st = new StringTokenizer(arg, ";");
     xLabel = st.nextToken();
     yLabel = st.nextToken();
  }


  /** 
   * return color map file name
   */
  public String getColormap() {
     return colormap;
  }

  /**
   * set color map file 
   */
  public void setColorMap(String str) {
     colormap = str;
  }




  /**
   * return 1d setting values
   */
  public String[] getSettings() {
     String[] settings = {
        (new Boolean(global_mode)).toString(),
        (new Boolean(overlay_mode)).toString(),
        (new Boolean(discard_mode)).toString(),
        (new Boolean(display_time)).toString(),
        (new Boolean(connect_point)).toString(),
        (new Boolean(amr)).toString(),
        (new Boolean(color_by_info)).toString(),
        (new Boolean(color_by_set)).toString(),
        (new Boolean(tickmark)).toString(),
        (new Boolean(display_label)).toString(),
        (new Boolean(display_legend)).toString(),
        String.valueOf(zoomStartX),
        String.valueOf(zoomStartY),
        String.valueOf(zoomEndX),
        String.valueOf(zoomEndY),
        String.valueOf(zoomFactorX),
        String.valueOf(zoomFactorY),
	String.valueOf(zoomMode),
        String.valueOf(legendRect.x),
        String.valueOf(legendRect.y),
        String.valueOf(legendRect.width),
        String.valueOf(legendRect.height),
        xLabelFont.getName(),
        String.valueOf(xLabelFont.getStyle()),
        String.valueOf(xLabelFont.getSize()),
        yLabelFont.getName(),
        String.valueOf(yLabelFont.getStyle()),
        String.valueOf(yLabelFont.getSize()),
        (new Boolean(isXLabelRotate)).toString(),
        (new Boolean(isYLabelRotate)).toString(),
        String.valueOf(xLabelPoint.x),
        String.valueOf(xLabelPoint.y),
        String.valueOf(yLabelPoint.x),
        String.valueOf(yLabelPoint.y),
        xLabel,
        yLabel  };

     return settings;
  }

  /**
   * return 1d setting values which is need for file writing
   */
  public String getFileSettings() {
     String settings = xLabel + ";" + yLabel;
     return settings;
  }

   public void doConfig(String[] args) {
        global_mode = Boolean.valueOf(args[0]).booleanValue();
        overlay_mode = Boolean.valueOf(args[1]).booleanValue();
        discard_mode = Boolean.valueOf(args[2]).booleanValue();
        display_time = Boolean.valueOf(args[3]).booleanValue();
        connect_point = Boolean.valueOf(args[4]).booleanValue();
        amr = Boolean.valueOf(args[5]).booleanValue();
        color_by_info = Boolean.valueOf(args[6]).booleanValue();
        color_by_set = Boolean.valueOf(args[7]).booleanValue();
        tickmark = Boolean.valueOf(args[8]).booleanValue();
        display_label = Boolean.valueOf(args[9]).booleanValue();
        display_legend = Boolean.valueOf(args[10]).booleanValue();
        zoomStartX = Float.valueOf(args[11]).floatValue();
        zoomStartY = Float.valueOf(args[12]).floatValue();
        zoomEndX = Float.valueOf(args[13]).floatValue();
        zoomEndY = Float.valueOf(args[14]).floatValue();
        zoomFactorX = Float.valueOf(args[15]).floatValue();
        zoomFactorY = Float.valueOf(args[16]).floatValue();
	zoomMode = Integer.valueOf(args[17]).intValue();
        legendRect.x = Integer.valueOf(args[18]).intValue();
        legendRect.y = Integer.valueOf(args[19]).intValue();
        legendRect.width = Integer.valueOf(args[20]).intValue();
        legendRect.height = Integer.valueOf(args[21]).intValue();
        
        try { 
          Font f = new Font(args[22], Integer.valueOf(args[23]).intValue(),
                            Integer.valueOf(args[24]).intValue());
          if (f != null) xLabelFont = f;
        } catch (Exception e) {
          System.out.println("Not defined fonts ...");
        }

        try { 
          Font f = new Font(args[25], Integer.valueOf(args[26]).intValue(),
                            Integer.valueOf(args[27]).intValue());
          if (f != null) yLabelFont = f;
        } catch (Exception e) {
          System.out.println("Not defined fonts ...");
        }

        isXLabelRotate = Boolean.valueOf(args[28]).booleanValue();
        isYLabelRotate = Boolean.valueOf(args[29]).booleanValue();
        xLabelPoint.x = Integer.valueOf(args[30]).intValue();
        xLabelPoint.y = Integer.valueOf(args[31]).intValue();
        yLabelPoint.x = Integer.valueOf(args[32]).intValue();
        yLabelPoint.y = Integer.valueOf(args[33]).intValue();

        xLabel = args[34];
        yLabel = args[35];

   }
  
} // end of class




