import java.awt.event.*;
import java.util.*;

import sv.kernel.CommandSet;

/** 
 * CSPlane.java
 * Event handler for GFPlane
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class CSPlane extends CommandSet 
                         implements ActionListener, ItemListener {

  GFPlane this_frame;
 
  public static final int CONNECTPOINT   = 1;
  public static final int DISPLAYLBL     = 2;
  public static final int TICKMARK       = 3;
  public static final int ADDTIME        = 4;
  public static final int AMR            = 5;
  public static final int XYZOOM         = 6;
  public static final int XZOOM          = 7;
  public static final int YZOOM          = 8;
  public static final int NOZOOM         = 9;
  public static final int ZOOMOUT        = 10;
  public static final int RESET          = 11;
  public static final int ZOOMING        = 12;
  public static final int ASPEEDCHANGE   = 13;
  public static final int DISPLAYLGD     = 14;
  public static final int LEGENDMOVE     = 15;
  public static final int LEGENDCHANGE   = 16;
  public static final int TICKERCHANGE   = 17;
  public static final int RESIZEPDA      = 18;
  public static final int LABELMOVE      = 19;
  public static final int COLORMAP       = 20;
  public static final int COLORBYINFO    = 21;
  public static final int COLORBYSET     = 22;

  int id;

  public CSPlane(GFPlane this_frame, int id) {
     this.this_frame = this_frame;
     this.id = id;

  }

  /**
   * overrided function for action event
   * when event occurs, 
   *     1. check first global flag mode
   *     2. collaboration
   */
  public void actionPerformed(ActionEvent e) {

     if (id >= 1000) {
	this_frame.doFilterAction(e.getActionCommand());
        return;
     }

     switch(id) {
       case SAVEAS: 
       case PRINT:
       case GIFSEQUENCE:
       case CUT:
       case COPY:
       case PASTE:
       case ADDTIME:
       case GOTO:
       case XYZOOM:
       case XZOOM:
       case YZOOM:
       case NOZOOM:
       case ASPEEDCHANGE:
       case LEGENDMOVE:
       case LEGENDCHANGE:
       case RENAMETITLE:
       case LABELCHANGE:
       case TICKERCHANGE:
       case RESIZEPDA:
       case LABELMOVE:
       case COLORMAP:
       case BUFFERSIZE:
         this_frame.doWork(id, null);
         break;

       case QUIT: 
       case ANIMATE:
       case NEXT:
       case PREVIOUS:
       case SPLIT: 
       case ZOOMOUT:
       case RESET:
         this_frame.svServer.fmanager.action1d(this_frame, id, null, true);
         break;
 
       case STOP:
         String[] arg = new String[1];
         arg[0] = String.valueOf(this_frame.currentTime);
         this_frame.svServer.fmanager.action1d(this_frame, id, arg, true);
         break;

       case WCUT:
       case WCOPY: 
       case WPASTE: 
       case WFILL: 
       case WLINECOLOR: 
       case WFILLCOLOR: 
       case WTEXTCOLOR: 
       case WTEXTFONT: 
       case WPEN: 
         this_frame.canvas.doWhiteboardWork(id);     
         break;
     }
  }

  /**
   * overrided function for item state
   */
  public void itemStateChanged(ItemEvent e) {

     String[] args = new String[1];
     switch(id) {
       case CONNECTPOINT: 
          args[0] = String.valueOf(this_frame.lineMI.getState());
          break;

       case DISPLAYLBL:
          args[0] = String.valueOf(this_frame.labelMI.getState());
          break;

       case TICKMARK: 
          args[0] = String.valueOf(this_frame.tickerMI.getState());
          break;

       case GMINMAX:
          args[0] = String.valueOf(this_frame.checkMI.getState());
          break;

       case AMR:
          args[0] = String.valueOf(this_frame.amrMI.getState());
          break;

       case COLORBYINFO:
          args[0] = String.valueOf(this_frame.cinfoMI.getState());
          break;

       case COLORBYSET:
          args[0] = String.valueOf(this_frame.csetMI.getState());
          break;

       case DISPLAYTIME:
          args[0] = String.valueOf(this_frame.timeMI.getState());
          break;

       case DISPLAYLGD:
          args[0] = String.valueOf(this_frame.legendMI.getState());
          break;

       case FILTEROVERLAY:
          args[0] = String.valueOf(this_frame.overlayMI.getState());
          break;

       case FILTERDISCARD:
          args[0] = String.valueOf(this_frame.discardMI.getState());
          break;

       case STEALTH:
          this_frame.doWork(id, null);
          break;
     }
     this_frame.svServer.fmanager.action1d(this_frame, id, args, true);
  }

} // end of class

