import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

import sv.kernel.*;

/** 
 * CollSender.java
 * Sending Thread to collaborator
 * @version 1.0   10/20/97
 * @author Byeongseob Ki
 */

class CollSender extends Thread {
  CollabManager cmanager;
  svserver svServer; 

  Vector conNameVec;
  Vector conPortVec;


  String server;
  int port;

  // for todo 1
  int mode;

  // for todo 2
  DataSet dataSet;
  long id;

  // for todo 11
  GFPlane gf1d;

  // for todo 12
  GFrame2D gf2d;

  // for todo 13
  GFrame3D gf3d;


  int todo;   // what to do

  public CollSender(CollabManager cmanager,
                                String server, int port, int mode) {
     this.cmanager = cmanager;
     svServer = cmanager.svServer;

     this.server = server;
     this.port = port;
     this.mode = mode;

     this.todo = 1;

     conNameVec = cmanager.getNameVec();
     conPortVec = cmanager.getPortVec();
  }


  public CollSender(CollabManager cmanager, DataSet dataSet, long id) {
     this.cmanager = cmanager;
     svServer = cmanager.svServer;

     this.dataSet = dataSet;
     this.id = id;

     this.todo = 2;

     conNameVec = cmanager.getNameVec();
     conPortVec = cmanager.getPortVec();
  }


  public CollSender(CollabManager cmanager, GFPlane gf1d) {
     this.cmanager = cmanager;
     svServer = cmanager.svServer;

     this.gf1d = gf1d;

     this.todo = 11;

     conNameVec = cmanager.getNameVec();
     conPortVec = cmanager.getPortVec();
  }


  public CollSender(CollabManager cmanager, GFrame2D gf2d) {
     this.cmanager = cmanager;
     svServer = cmanager.svServer;

     this.gf2d = gf2d;

     this.todo = 12;

     conNameVec = cmanager.getNameVec();
     conPortVec = cmanager.getPortVec();
  }

  public CollSender(CollabManager cmanager, GFrame3D gf3d) {
     this.cmanager = cmanager;
     svServer = cmanager.svServer;

     this.gf3d = gf3d;

     this.todo = 13;

     conNameVec = cmanager.getNameVec();
     conPortVec = cmanager.getPortVec();
  }

  public void run() {
    switch (todo) {
       case 1:
         InitSending();
         break;
       case 2:
         broadcastDataSet();
         break;
       case 11:
         broadcastGFPlane();
         break;
       case 12:
         broadcastGFrame2D();
         break;
       case 13:
//         broadcastGFrame3D();
         break;
    }
  } 


  private void InitSending() {

     try {
        Socket sock = new Socket(server, port);
        DataOutputStream output = 
                new DataOutputStream(sock.getOutputStream());

        output.writeInt(4);         // send acception message
        output.writeInt(mode);      // send mode(master(1) or slave mode)

         // *** (2-1) send collaborator list ***
        if (mode == 1) {
           int connectionSz = conNameVec.size();
           output.writeInt(connectionSz);
           for (int i=0; i<connectionSz; i++) {
              String sname = (String)conNameVec.elementAt(i);
              int sport = ((Integer)conPortVec.elementAt(i)).intValue();

              output.writeUTF(sname);     // send server name
              output.writeInt(sport);     // send port number
           }
           output.writeInt(svServer.this_port);    
        }

        // *** (2-2) send system configuration ***
        String[] sysconfig = svServer.configHandler.config1d.getSettings();
        output.writeInt(sysconfig.length);
        for (int j=0; j<sysconfig.length; j++)
           output.writeUTF(sysconfig[j]);

        sysconfig = svServer.configHandler.config2d.getSettings();
        output.writeInt(sysconfig.length);
        for (int j=0; j<sysconfig.length; j++)
           output.writeUTF(sysconfig[j]);

        sysconfig = svServer.configHandler.config3d.getSettings();
        output.writeInt(sysconfig.length);
        for (int j=0; j<sysconfig.length; j++)
           output.writeUTF(sysconfig[j]);

        // *** (3) send data set ***
        Vector vec = svServer.fmanager.get1dFrames();
        int noOfFrames = vec.size();
        output.writeInt(noOfFrames);   // send number of frames

        for (int i=0; i<noOfFrames; i++) {
           GFPlane gf1d = (GFPlane)vec.elementAt(i);
           if (gf1d.stealthMode) {
              output.writeBoolean(true);
              continue;
           } else
              output.writeBoolean(false);

           output.writeInt(gf1d.getDataType());
           output.writeLong(gf1d.getID());

           String[] configs = gf1d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);

	      ////////////////////////////////////////////////////////
           // sends whiteboard objects
           String drawingobjects = gf1d.getDHVString();
           if (drawingobjects != null) {
              byte[] sentArr = drawingobjects.getBytes();

              // send size of dataset array
              output.writeInt(sentArr.length); 
              output.write(sentArr, 0, sentArr.length);
           } else 
              output.writeInt(0); 
           
	      ////////////////////////////////////////////////////////

           int noOfDataSet = gf1d.dataSetVector.size();
           output.writeInt(noOfDataSet);       // send number of DataSet

           for (int j=0; j<noOfDataSet; j++) {
              DSPlane dataSet = 
                     (DSPlane)gf1d.dataSetVector.elementAt(j);

              output.writeUTF(dataSet.getTitle());

              int noOfTimeData = dataSet.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 dataSet.timeData[k].sendingData(output);
           }
        }

        vec = svServer.fmanager.get2dFrames();            
        noOfFrames = vec.size();
        output.writeInt(noOfFrames);   // send number of frames for 2D

        for (int i=0; i<noOfFrames; i++) {
           GFrame2D gf2d = (GFrame2D)vec.elementAt(i);
           if (gf2d.stealthMode) {
              output.writeBoolean(true);
              continue;
           } else
              output.writeBoolean(false);

           output.writeLong(gf2d.getID());

           String[] configs = gf2d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);

           configs = gf2d.getColorInfo();
           output.writeUTF(configs[0]);
           output.writeUTF(configs[1]);

	   //           output.writeUTF("NONE");   // whiteboard objects

           int noOfDataSet = gf2d.dataSetVector.size();
           output.writeInt(noOfDataSet);   // send number of DataSet

           for (int j=0; j<noOfDataSet; j++) {
              DataSet2D dataSet = 
                   (DataSet2D)gf2d.dataSetVector.elementAt(j);

              output.writeUTF(dataSet.getTitle());

              int noOfTimeData = dataSet.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 dataSet.timeData[k].sendingData(output);
           }
        }

 output.writeInt(0);  // ADD 3D COLLABORATION IN THE NEXT VERSION
/*
        vec = svServer.fmanager.get3dFrames();            
        noOfFrames = vec.size();
        output.writeInt(noOfFrames);   // send number of frames for 3D

        for (int i=0; i<noOfFrames; i++) {
           GFrame3D gf3d = (GFrame3D)vec.elementAt(i);
           if (gf3d.stealthMode) {
              output.writeBoolean(true);
              continue;
           } else
              output.writeBoolean(false);

           output.writeLong(gf3d.getID());

           String[] configs = gf3d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);

	   //           output.writeUTF("NONE");   // whiteboard objects

	   //int noOfDataSet = gf3d.dataSetVector.size();
           //output.writeInt(noOfDataSet);   // send number of DataSet

           //for (int j=0; j<noOfDataSet; j++) {
           //   DataSet3D dataSet = 
           //        (DataSet3D)gf3d.dataSetVector.elementAt(j);

              output.writeUTF(gf3d.iso.getTitle());
              String hn = gf3d.iso.getSourceHostname();
              int pt = gf3d.iso.getSourcePort();
              if (hn != null && pt > 0) {
                 output.writeInt(1);
                 output.writeUTF(hn);
                 output.writeInt(pt);
                 output.writeInt(gf3d.iso.getVersion());
              } else
                  output.writeInt(0);

              int noOfTimeData = gf3d.iso.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 gf3d.iso.timeData[k].sendingData(output);
           //}
        }
*/  // 3D stuff
     } catch (Exception e) {
        svServer.message("Connection failed ....", true);
     }

     // *** (4) send  new collaborator's name to all other collaborators.
    int size = conNameVec.size();
    for (int i=0; i<size; i++) {
       String sname = (String)conNameVec.elementAt(i);
       int sport = ((Integer)conPortVec.elementAt(i)).intValue();

       try {
          Socket sock = new Socket(sname, sport);
          DataOutputStream output = 
                          new DataOutputStream(sock.getOutputStream());

          output.writeInt(6);   // send new collaborator adding
          output.writeUTF(server);
          output.writeInt(port);

       } catch (Exception e) {
          svServer.message("Connection failed ....", true);
       }
     }

     // *** (5) finally add new collaborator new connection list 
     cmanager.setCollaborationMode(true);
     if (cmanager.conHandler == null) {
        cmanager.conHandler = new ConnectionHandler(svServer);
        cmanager.conHandler.setSize(480, 350);
        cmanager.conHandler.show();
     }  
     cmanager.addNewCollaborator(server, new Integer(port), server);
        
  }

 
  private void broadcastDataSet() {
     int dim = dataSet.getRank();
     int size = conNameVec.size();
     int noOfTimeData = 0;
     byte[] sentArr = null;

if (dim == DataSet.THREED)
return;

     for (int i=0; i<size; i++) {
        String sname = (String)conNameVec.elementAt(i);
        int sport = ((Integer)conPortVec.elementAt(i)).intValue();
	//System.out.println("broadcasting: "  + sname + " " + sport);
        try {
           Socket sock = new Socket(sname, sport);
           DataOutputStream output = 
                 new DataOutputStream(sock.getOutputStream());

           output.writeInt(7);            // send command for broadcasting
           output.writeInt(dim);          // send dimension
           output.writeLong(id); 
           output.writeUTF(dataSet.getTitle());

           if (dim == DataSet.ONED) {
               output.writeInt(((DSPlane)dataSet).getDataType());  
               noOfTimeData = ((DSPlane)dataSet).timeData.length;

           } else if (dim == DataSet.TWOD)  
               noOfTimeData = ((DataSet2D)dataSet).timeData.length;
           else if (dim == DataSet.THREED) {  

              String hn = ((DataSet3D)dataSet).getSourceHostname();
              int pt = ((DataSet3D)dataSet).getSourcePort();
              if (hn != null && pt > 0) {
                 output.writeInt(1);
                 output.writeUTF(hn);
                 output.writeInt(pt);
                 output.writeInt(((DataSet3D)dataSet).getVersion());
              } else
                  output.writeInt(0);

               noOfTimeData = ((DataSet3D)dataSet).timeData.length;
           }

           output.writeInt(noOfTimeData);   // send number of TimeData

           for (int k=0; k<noOfTimeData; k++) {
              if (dim == DataSet.ONED)  
                 ((DSPlane)dataSet).timeData[k].sendingData(output);
              else if (dim == DataSet.TWOD)  
                 ((DataSet2D)dataSet).timeData[k].sendingData(output);
              else if (dim == DataSet.THREED)  
                 ((DataSet3D)dataSet).timeData[k].sendingData(output);
           }

        } catch (Exception e) {
           svServer.message("Connection failed ....", true);
e.printStackTrace();
        }
     }
  }

  private void broadcastGFPlane() {

     int size = conNameVec.size();

     for (int i=0; i<size; i++) {
        String sname = (String)conNameVec.elementAt(i);
        int sport = ((Integer)conPortVec.elementAt(i)).intValue();
	//System.out.println("broadcasting: "  + sname + " " + sport);
        try {
           Socket sock = new Socket(sname, sport);
           DataOutputStream output = 
                 new DataOutputStream(sock.getOutputStream());

           output.writeInt(8);            // send command for broadcasting
           output.writeInt(DataSet.ONED); 

           if (gf1d.stealthMode) {
              output.writeBoolean(true);
              return;
           } else
              output.writeBoolean(false);

           output.writeInt(gf1d.getDataType());
           output.writeLong(gf1d.getID());

           String[] configs = gf1d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);

	      ////////////////////////////////////////////////////////
           // sends whiteboard objects
           String drawingobjects = gf1d.getDHVString();
           if (drawingobjects != null) {
              byte[] sentArr = drawingobjects.getBytes();

              // send size of dataset array
              output.writeInt(sentArr.length); 
              output.write(sentArr, 0, sentArr.length);
           } else 
              output.writeInt(0); 
           
	      ////////////////////////////////////////////////////////

           int noOfDataSet = gf1d.dataSetVector.size();
           output.writeInt(noOfDataSet);       // send number of DataSet

           for (int j=0; j<noOfDataSet; j++) {
              DSPlane dataSet = 
                     (DSPlane)gf1d.dataSetVector.elementAt(j);

              output.writeUTF(dataSet.getTitle());

              int noOfTimeData = dataSet.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 dataSet.timeData[k].sendingData(output);
           }
	} catch (Exception e) {
           svServer.message("Connection failed ....", true);
        }
     }
  }
  

  private void broadcastGFrame2D() {

     int size = conNameVec.size();

     for (int i=0; i<size; i++) {
        String sname = (String)conNameVec.elementAt(i);
        int sport = ((Integer)conPortVec.elementAt(i)).intValue();
	//System.out.println("broadcasting: "  + sname + " " + sport);
        try {
           Socket sock = new Socket(sname, sport);
           DataOutputStream output = 
                 new DataOutputStream(sock.getOutputStream());

           output.writeInt(8);            // send command for broadcasting
           output.writeInt(DataSet.TWOD);

           if (gf2d.stealthMode) {
              output.writeBoolean(true);
              return;
           } else
              output.writeBoolean(false);

           output.writeLong(gf2d.getID());

           String[] configs = gf2d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);

           configs = gf2d.getColorInfo();
           output.writeUTF(configs[0]);
           output.writeUTF(configs[1]);
           
	   //           output.writeUTF("NONE");   // whiteboard objects

           int noOfDataSet = gf2d.dataSetVector.size();
           output.writeInt(noOfDataSet);       // send number of DataSet

           for (int j=0; j<noOfDataSet; j++) {
              DataSet2D dataSet = 
                     (DataSet2D)gf2d.dataSetVector.elementAt(j);

              output.writeUTF(dataSet.getTitle());

              int noOfTimeData = dataSet.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 dataSet.timeData[k].sendingData(output);
           }
	} catch (Exception e) {
           svServer.message("Connection failed ....", true);
        }
     }
  }


  private void broadcastGFrame3D() {
/*
     int size = conNameVec.size();

     for (int i=0; i<size; i++) {
        String sname = (String)conNameVec.elementAt(i);
        int sport = ((Integer)conPortVec.elementAt(i)).intValue();
	//System.out.println("broadcasting: "  + sname + " " + sport);
        try {
           Socket sock = new Socket(sname, sport);
           DataOutputStream output = 
                 new DataOutputStream(sock.getOutputStream());

           output.writeInt(8);            // send command for broadcasting
           output.writeInt(DataSet.THREED);

           if (gf3d.stealthMode) {
              output.writeBoolean(true);
              return;
           } else
              output.writeBoolean(false);

           output.writeLong(gf3d.getID());

           String[] configs = gf3d.config.getSettings();
           output.writeInt(configs.length);
           for (int j=0; j<configs.length; j++)
              output.writeUTF(configs[j]);
           
	   //           output.writeUTF("NONE");   // whiteboard objects

           //int noOfDataSet = gf3d.dataSetVector.size();
           //output.writeInt(noOfDataSet);       // send number of DataSet

           //for (int j=0; j<noOfDataSet; j++) {
           //   DataSet3D dataSet = 
           //          (DataSet3D)gf3d.dataSetVector.elementAt(j);

              output.writeUTF(gf3d.iso.getTitle());
              String hn = gf3d.iso.getSourceHostname();
              int pt = gf3d.iso.getSourcePort();
              if (hn != null && pt > 0) {
                 output.writeInt(1);
                 output.writeUTF(hn);
                 output.writeInt(pt);
                 output.writeInt(gf3d.iso.getVersion());
              } else
                  output.writeInt(0);

              int noOfTimeData = gf3d.iso.timeData.length;
              output.writeInt(noOfTimeData);   // send number of TimeData
              for (int k=0; k<noOfTimeData; k++) 
                 gf3d.iso.timeData[k].sendingData(output);
            //}
	} catch (Exception e) {
           svServer.message("Connection failed ....", true);
        }
     }
*/
  }

} // end of class







