import java.awt.event.*;
import java.util.*;

import sv.kernel.CommandSet;

/** 
 * CommandSet2D.java
 * Event handler for GFrame2D
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class CommandSet2D extends CommandSet 
                             implements ActionListener, ItemListener {
  GFrame2D this_frame;

  public static final int BOUNDBOX       = 1;
  public static final int XYMESH         = 2;
  public static final int FACEGRID       = 3;
  public static final int VECTOR         = 4;
  public static final int XYTICK         = 5;
  public static final int ZTICK          = 6;
  public static final int WIRE           = 7;
  public static final int HIDDEN         = 8;
  public static final int COLOR          = 9;
  public static final int GRAY           = 10;
  public static final int POINTPLOT      = 11;
  public static final int SURFACE        = 12;
  public static final int CONTOUR        = 13;
  public static final int REGENERATE     = 14;
  public static final int COLORMAP       = 15;
  public static final int CONTOURSETTING = 16;
  public static final int RESET          = 17;
  public static final int CONTOURMODIFY  = 18;
  public static final int POSITIONCHANGE = 19;
  public static final int WIRECOLOR      = 20;
   
  int id;

  public CommandSet2D(GFrame2D this_frame, int id) {
     this.this_frame = this_frame;
     this.id = id;

  }

  /**
   * overrided function for action event
   */
  public void actionPerformed(ActionEvent e) {

     if (id >= 1000) {
        this_frame.doFilterAction(e.getActionCommand());
        return;
     }

     switch(id) {
       case SAVEAS: 
       case PRINT:
       case GIFSEQUENCE:
       case CUT:
       case COPY:
       case PASTE:
       case GOTO:
       case COLORMAP:
       case CONTOURSETTING:
       case RENAMETITLE:
       case LABELCHANGE:
       case BUFFERSIZE:
         this_frame.doWork(id, null);
         break;

       case QUIT: 
       case ANIMATE:
       case NEXT:
       case PREVIOUS:
       case SPLIT: 
       case RESET:
         this_frame.svServer.fmanager.action2d(this_frame, id, null, true);
         break;

       case STOP:
         String[] arg = new String[1];
         arg[0] = String.valueOf(this_frame.currentTime);
         this_frame.svServer.fmanager.action2d(this_frame, id, arg, true);
         break;
     }
  }

  /**
   * overrided function for item state
   */
  public void itemStateChanged(ItemEvent e) {
     String[] args = new String[1];
     switch(id) {
       case BOUNDBOX: 
          args[0] = String.valueOf(this_frame.boundMI.getState());
          break;

       case XYMESH: 
          args[0] = String.valueOf(this_frame.meshMI.getState());
          break;

       case FACEGRID: 
          args[0] = String.valueOf(this_frame.faceMI.getState());
          break;

       case VECTOR: 
          args[0] = String.valueOf(this_frame.vectorMI.getState());
          break;

       case XYTICK: 
          args[0] = String.valueOf(this_frame.xytickMI.getState());
          break;

       case ZTICK: 
          args[0] = String.valueOf(this_frame.ztickMI.getState());
          break;

       case WIRE: 
          args[0] = String.valueOf(this_frame.wireMI.getState());
          break;

       case WIRECOLOR: 
          args[0] = String.valueOf(this_frame.cwireMI.getState());
          break;

       case HIDDEN: 
          args[0] = String.valueOf(this_frame.hiddenMI.getState());
          break;

       case COLOR: 
          args[0] = String.valueOf(this_frame.spectrumMI.getState());
          break;

       case GRAY: 
          args[0] = String.valueOf(this_frame.grayMI.getState());
          break;

       case POINTPLOT: 
          args[0] = String.valueOf(this_frame.pointMI.getState());
          break;

       case SURFACE: 
          args[0] = String.valueOf(this_frame.surfaceMI.getState());
          break;

       case CONTOUR:
          args[0] = String.valueOf(this_frame.contourMI.getState());
          break;

       case REGENERATE:
          args[0] = String.valueOf(this_frame.delayMI.getState());
          break;

       case GMINMAX:
          args[0] = String.valueOf(this_frame.globalMI.getState());
          break;

       case FILTEROVERLAY:
          args[0] = String.valueOf(this_frame.overlayMI.getState());
          break;

       case FILTERDISCARD:
          args[0] = String.valueOf(this_frame.discardMI.getState());
          break;

       case STEALTH:
          this_frame.doWork(id, null);
          break;

       case DISPLAYTIME:
          args[0] = String.valueOf(this_frame.timeMI.getState());
          break;
     }
     this_frame.svServer.fmanager.action2d(this_frame, id, args, true);
  }

} // end of class

