
import java.util.*;
import sv.kernel.*;

/** 
 * Config2D.java
 * Configuration for 2d
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public final class Config2D extends Configs {

  public int plotmode_2d = 7;
  public boolean delay_regenerating_2d = false;
  public boolean bounding_box_2d = true;
  public boolean xymesh_2d = false;
  public boolean xyticks_2d = false;
  public boolean zticks_2d = false;
  public boolean display_grid_2d = false;
  public boolean vector_plot_2d = false;
  public float distance_2d = 70f;
  public float scale_2d = 10f;
  public float theta_2d = 125f;
  public float phi_2d = 10f;
  public int xtrans_2d = 0;
  public int ytrans_2d = 0;
  public String colormap_2d = null;

  public String xLabel = "X";
  public String yLabel = "Y";
  public String zLabel = "Z";     // use for 2d and 3d


  public Config2D() {
    // nothing to do
  }


  public Config2D(String[] args) {
    doConfig(args);
  }

  public Config2D(String arg) {
     StringTokenizer st = new StringTokenizer(arg, ";");
     xLabel = st.nextToken();
     yLabel = st.nextToken();
     zLabel = st.nextToken();
  }


  /** 
   * return color map file name
   */
  public String getColormap() {
     return colormap_2d;
  }

  /**
   * set color map file 
   */
  public void setColorMap(String str) {
     colormap_2d = str;
  }


  /**
   * return 2d setting values
   */
  public String[] getSettings() {
     String[] settings = {
        (new Boolean(global_mode)).toString(),
        (new Boolean(overlay_mode)).toString(),
        (new Boolean(discard_mode)).toString(),
        (new Boolean(display_time)).toString(),
        String.valueOf(plotmode_2d),
        (new Boolean(delay_regenerating_2d)).toString(),
        (new Boolean(bounding_box_2d)).toString(),
        (new Boolean(xymesh_2d)).toString(),
        (new Boolean(xyticks_2d)).toString(),
        (new Boolean(zticks_2d)).toString(),
        (new Boolean(display_grid_2d)).toString(),
        (new Boolean(vector_plot_2d)).toString(),
        String.valueOf(distance_2d),
        String.valueOf(scale_2d),
        String.valueOf(theta_2d),
        String.valueOf(phi_2d),
        String.valueOf(xtrans_2d),
        String.valueOf(ytrans_2d),
        xLabel, yLabel, zLabel };
  
       return settings;
  }

  /**
   * return 1d setting values which is need for file writing
   */
  public String getFileSettings() {
     String settings = xLabel + ";" + yLabel + ";" + zLabel;
     return settings;
  }


   public void doConfig(String[] args) {
        global_mode = Boolean.valueOf(args[0]).booleanValue();
        overlay_mode = Boolean.valueOf(args[1]).booleanValue();
        discard_mode = Boolean.valueOf(args[2]).booleanValue();
        display_time = Boolean.valueOf(args[3]).booleanValue();
        plotmode_2d = Integer.valueOf(args[4]).intValue();
        delay_regenerating_2d = Boolean.valueOf(args[5]).booleanValue();
        bounding_box_2d = Boolean.valueOf(args[6]).booleanValue();
        xymesh_2d = Boolean.valueOf(args[7]).booleanValue();
        xyticks_2d = Boolean.valueOf(args[8]).booleanValue();
        zticks_2d = Boolean.valueOf(args[9]).booleanValue();
        display_grid_2d = Boolean.valueOf(args[10]).booleanValue();
        vector_plot_2d = Boolean.valueOf(args[11]).booleanValue();
        distance_2d = Float.valueOf(args[12]).floatValue();
        scale_2d = Float.valueOf(args[13]).floatValue();
        theta_2d = Float.valueOf(args[14]).floatValue();
        phi_2d = Float.valueOf(args[15]).floatValue();
        xtrans_2d = Integer.valueOf(args[16]).intValue();
        ytrans_2d = Integer.valueOf(args[17]).intValue();
        xLabel = args[18];
        yLabel = args[19];
        zLabel = args[20];
   }
  
} // end of class




