import java.awt.*;
import java.io.*;
import java.util.*;


/** 
 * Configuration.java
 * Configuration handler
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class Configuration {

  /** 
   * parsed config values
   */
  public CPlane config1d;
  public Config2D config2d;
  public Config3D config3d;

  private String sv_menu = null;

  public Configuration(String setting) {

    if (setting != null) {
       if (!setting.endsWith(File.separator))
         setting = setting + File.separator;
 
       sv_menu = setting + "sv_menu";
    }

    config1d = new CPlane();
    config2d = new Config2D();
    config3d = new Config3D();

    if (setting != null) 
       parseConfigs(setting, "sv_setting");
  }

  /**
   * return menu file name
   */
  public String getMenuFile() {
     return sv_menu;
  }


  // parse configuration file
  private void parseConfigs(String dir, String fname) {
     fname = dir + fname;

     String s;

     try {
       FileInputStream propFile = new FileInputStream(fname);
       Properties p = new Properties(System.getProperties());
       p.load(propFile);

       // set the system properties
       System.setProperties(p);


       //////////////////////////////////////////////////////////
       // 1d
       //////////////////////////////////////////////////////////
       s = System.getProperty("connect_points_1d");
       if (s.equals("yes"))  config1d.connect_point = true;

       s = System.getProperty("color_by_amr_1d");
       if (s.equals("yes"))  config1d.amr = true;

       s = System.getProperty("color_by_info_1d");
       if (s.equals("yes"))  config1d.color_by_info = true;

       s = System.getProperty("color_by_set_1d");
       if (s.equals("yes"))  config1d.color_by_set = true;

       s = System.getProperty("tickmark_1d");
       if (s.equals("yes"))  config1d.tickmark = true;

       s = System.getProperty("display_label_1d");
       if (s.equals("yes"))  config1d.display_label = true;

       s = System.getProperty("display_legend_1d");
       if (s.equals("yes"))  config1d.display_legend = true;

       s = System.getProperty("colormap_1d");
       if (s != null) config1d.colormap = dir + s.trim();

       s = System.getProperty("x_label_1d");
       if (s != null)  config1d.xLabel = s.trim();

       s = System.getProperty("y_label_1d");
       if (s != null)  config1d.yLabel = s.trim();

       s = System.getProperty("x_label_rotate_1d");
       if (s.equals("yes")) config1d.isXLabelRotate = true;

       s = System.getProperty("y_label_rotate_1d");
       if (s.equals("yes")) config1d.isYLabelRotate = true;

       s = System.getProperty("global_mode_1d");
       if (s.equals("yes"))  config1d.global_mode = true;

       s = System.getProperty("overlay_mode_1d");
       if (s.equals("yes"))  config1d.overlay_mode = true;

       s = System.getProperty("discard_mode_1d");
       if (s.equals("yes"))  config1d.discard_mode = true;

       s = System.getProperty("display_time_1d");
       if (s.equals("yes"))  config1d.display_time = true;

       String sr = System.getProperty("bgColor_1d_red");
       String sg = System.getProperty("bgColor_1d_green");
       String sb = System.getProperty("bgColor_1d_blue");

       if (sr != null && sg != null && sb != null) {
          try {
             Color bg = new Color(Integer.valueOf(sr).intValue(),
                                  Integer.valueOf(sr).intValue(),
                                  Integer.valueOf(sr).intValue());
             if (bg != null) config1d.bgColor = bg;
          } catch (Exception e) {
             System.out.println("Color is not valid.");
          }
       }


       //////////////////////////////////////////////////////////
       // 2d
       //////////////////////////////////////////////////////////
       s = System.getProperty("plot_mode_2d");
       if (s != null) config2d.plotmode_2d = Integer.valueOf(s).intValue();

       s = System.getProperty("delay_regenerating_2d");
       if (s.equals("yes"))  config2d.delay_regenerating_2d = true;

       s = System.getProperty("bounding_box_2d");
       if (s.equals("yes"))  config2d.bounding_box_2d = true;

       s = System.getProperty("xymesh_2d");
       if (s.equals("yes"))  config2d.xymesh_2d = true;

       s = System.getProperty("xyticks_2d");
       if (s.equals("yes"))  config2d.xyticks_2d = true;

       s = System.getProperty("zticks_2d");
       if (s.equals("yes"))  config2d.zticks_2d = true;

       s = System.getProperty("display_grid_2d");
       if (s.equals("yes"))  config2d.display_grid_2d = true;

       s = System.getProperty("vector_plot_2d");
       if (s.equals("yes"))  config2d.vector_plot_2d = true;

       s = System.getProperty("x_label_2d");
       if (s != null)  config2d.xLabel = s.trim();

       s = System.getProperty("y_label_2d");
       if (s != null)  config2d.yLabel = s.trim();

       s = System.getProperty("z_label_2d");
       if (s != null)  config2d.zLabel = s.trim();
    
       s = System.getProperty("colormap_2d");
       if (s != null)  config2d.colormap_2d = dir + s.trim();

       s = System.getProperty("global_mode_2d");
       if (s.equals("yes"))  config2d.global_mode = true;

       s = System.getProperty("overlay_mode_2d");
       if (s.equals("yes"))  config2d.overlay_mode = true;

       s = System.getProperty("discard_mode_2d");
       if (s.equals("yes"))  config2d.discard_mode = true;

       s = System.getProperty("display_time_2d");
       if (s.equals("yes"))  config2d.display_time = true;

       sr = System.getProperty("bgColor_2d_red");
       sg = System.getProperty("bgColor_2d_green");
       sb = System.getProperty("bgColor_2d_blue");

       if (sr != null && sg != null && sb != null) {
          try {
             Color bg = new Color(Integer.valueOf(sr).intValue(),
                                  Integer.valueOf(sr).intValue(),
                                  Integer.valueOf(sr).intValue());
             if (bg != null) config2d.bgColor = bg;
          } catch (Exception e) {
             System.out.println("Color is not valid.");
          }
       }
 
     } catch (Exception e) {
e.printStackTrace();
       System.err.println("I/O failed while reading the configuration file...");
     }
  }


}  // end of class

