import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.ui.*;
import sv.kernel.*;

/** 
 * DesktopControl.java
 * Virtual desktop controll
 * @version 1.0     10/15/97
 * @author Byeongseob Ki
 */

public class DesktopControl extends JPanel 
                         implements ActionListener {

  svserver svServer;

  private DesktopCanvas canvas;

  // popup menu
  PopupMenu popup, ipop;
  Menu iconMenu;

  DesktopInfo targetInfo;

  // tile resizing menu
  public boolean tileResize = false;

  // virtual desktop
  public Rectangle screenSize;

  /**
   * constructor
   */
  DesktopControl(svserver svServer) {
     this.svServer = svServer;

     Dimension sz = getToolkit().getScreenSize();
     screenSize = new Rectangle(20, 20, sz.width-20, sz.height-20);

     makeGUI();
  }

  //////////////////////////////////////////////////////
  // Desktop methods
  //////////////////////////////////////////////////////
  public void desktopRepaint() {
     canvas.repaint();
  }

  // to clear focused window(remove magenta window)
  public void desktopRepaint(boolean b) {
     canvas.focusedInfo = null;
     canvas.repaint();
  }

  private Vector iconizedInfos = new Vector();

  // clear iconized window vector
  public void clearIconvec() {
    iconMenu.removeAll();
    iconizedInfos.removeAllElements();
  }

  /**
   * add item to ipop(iconized window)
   * @param item DestopInfo item to be iconized
   */
  public void addItem(DesktopInfo item) {
     iconizedInfos.addElement(item);
     MenuItem mi = new MenuItem(item.title);
     mi.addActionListener(this);
     iconMenu.add(mi);
  }

  /** 
   * set target window for the popup menu
   */
  public void setTarget(DesktopInfo info) {
    targetInfo = info;
  }


  /**
   * overrided method (for buttons)
   */
  public void actionPerformed(ActionEvent e) {
     String command = e.getActionCommand();
     int size, i, index = 0;

     ////////////////////////////////////////////////////
     // STACK
     if (command.equals("Stack")) {
       int xstep = screenSize.x;
       int ystep = screenSize.y;
       int xgap, ygap;

       int maxstack = (int)((screenSize.height - 360)/50);
       if (maxstack < 2) {
          System.out.println("Too small screen size to stack ...");
          return;
       }

       int size1 = svServer.fmanager.get1dvecSize();
       int size2 = svServer.fmanager.get2dvecSize();
       int size3 = svServer.fmanager.get3dvecSize();

       size = size1 + size2 + size3;

       for (i=0; i<size1; i++) {
          GFPlane gf1d = svServer.fmanager.get1dFrame(i);
          if (!gf1d.isIconized()) {
             xgap = (int)(index / maxstack);
             ygap = index % maxstack;
             gf1d.setLocation(xstep+xgap*200+ygap*30, ystep+ygap*50);
             gf1d.validate();
             gf1d.pack();
             index++;
          }
       }

       for (i=0; i<size2; i++) {
          GFrame2D gf2d = svServer.fmanager.get2dFrame(i);
          if (!gf2d.isIconized()) {
             xgap = (int)(index / maxstack);
             ygap = index % maxstack;
             gf2d.setLocation(xstep+xgap*200+ygap*30, ystep+ygap*50);
             gf2d.validate();
             gf2d.pack();
             index++;
          }
       }

       for (i=0; i<size3; i++) {
          GFrame3D gf3d = svServer.fmanager.get3dFrame(i);
          if (!gf3d.isIconized()) {
             xgap = (int)(index / maxstack);
             ygap = index % maxstack;
             gf3d.setLocation(xstep+xgap*200+ygap*30, ystep+ygap*50);
             gf3d.validate();
             gf3d.pack();
             index++;
          }
       }

       canvas.repaint();

     ////////////////////////////////////////////////////
     // TILE
     } else if (command.equals("Tile")) {      
       int[] formation;
       int wid, hgt, row, col;

       int size1 = svServer.fmanager.get1dvecSize();
       int size2 = svServer.fmanager.get2dvecSize();
       int size3 = svServer.fmanager.get3dvecSize();

       size = 0;
       for (i=0; i<size1; i++) {
          GFPlane gf1d = svServer.fmanager.get1dFrame(i);
          if (!gf1d.isIconized())
             size++;
       }
       for (i=0; i<size2; i++) {
          GFrame2D gf2d = svServer.fmanager.get2dFrame(i);
          if (!gf2d.isIconized())
             size++;
       }
       for (i=0; i<size3; i++) {
          GFrame3D gf3d = svServer.fmanager.get3dFrame(i);
          if (!gf3d.isIconized())
             size++;
       }
             
       formation = tileFormation(size);

       wid = (int)(screenSize.width / formation[0]);
       hgt = (int)(screenSize.height / formation[1]);

       index = 0;
       for (i=0; i<size1; i++) {
          GFPlane gf1d = svServer.fmanager.get1dFrame(i);
          if (!gf1d.isIconized()) {
             row = (int)(index / formation[0]);
             col = index - row * formation[0];
             if (tileResize) 
                gf1d.setSize(wid-10, hgt-10);
             gf1d.setLocation(col*wid+screenSize.x, row*hgt+screenSize.y);
             gf1d.validate();
             gf1d.pack();
             index++;
          }
       }
       for (i=0; i<size2; i++) {
          GFrame2D gf2d = svServer.fmanager.get2dFrame(i);
          if (!gf2d.isIconized()) {
             row = (int)(index / formation[0]);
             col = index - row * formation[0];
             if (tileResize)
                gf2d.setSize(wid-10, hgt-10);
             gf2d.setLocation(col*wid+screenSize.x, row*hgt+screenSize.y);
             gf2d.validate();
             gf2d.pack();
             index++;
          }
       }
       for (i=0; i<size3; i++) {
          GFrame3D gf3d = svServer.fmanager.get3dFrame(i);
          if (!gf3d.isIconized()) {
             row = (int)(index / formation[0]);
             col = index - row * formation[0];
             if (tileResize) 
                gf3d.setSize(wid-10, hgt-10);
             gf3d.setLocation(col*wid+screenSize.x, row*hgt+screenSize.y);
             gf3d.validate();
             gf3d.pack();
             index++;
          }
       }

       canvas.repaint();

     } else if (canvas.focusedInfo != null) {
       DesktopInfo focusedInfo = canvas.focusedInfo;

       if (focusedInfo.dim == 1 && focusedInfo.gf1d != null) {
          if (command.equals("Animate")) 
            svServer.fmanager.action1d(focusedInfo.gf1d, 
                         CommandSet.ANIMATE, null, true);    

          else if (command.equals("Stop")) {
            focusedInfo.gf1d.doStop();
            String[] arg = new String[1];
            arg[0] = String.valueOf(focusedInfo.gf1d.currentTime);
            svServer.fmanager.action1d(focusedInfo.gf1d, CommandSet.STOP, 
                                                              arg, true);  
          } else if (command.equals("Next")) 
            svServer.fmanager.action1d(focusedInfo.gf1d, CommandSet.NEXT, 
                                                              null, true);
          else if (command.equals("Previous")) 
            svServer.fmanager.action1d(focusedInfo.gf1d, CommandSet.PREVIOUS, 
                                                              null, true);
          else if (command.equals("Hide")) 
            focusedInfo.gf1d.setVisible(false);
          else if (command.equals("Kill")) 
            svServer.fmanager.action1d(focusedInfo.gf1d, CommandSet.QUIT, 
                                                              null, true);
       } else if (focusedInfo.dim == 2 && focusedInfo.gf2d != null) {
          if (command.equals("Animate")) 
            svServer.fmanager.action2d(focusedInfo.gf2d, 
                         CommandSet2D.ANIMATE, null, true);    

          else if (command.equals("Stop")) {
            focusedInfo.gf2d.doStop();
            String[] arg = new String[1];
            arg[0] = String.valueOf(focusedInfo.gf2d.currentTime);
            svServer.fmanager.action2d(focusedInfo.gf2d, CommandSet.STOP, 
                                                              arg, true);  
          } else if (command.equals("Next")) 
            svServer.fmanager.action2d(focusedInfo.gf2d, CommandSet.NEXT, 
                                                              null, true);
          else if (command.equals("Previous")) 
            svServer.fmanager.action2d(focusedInfo.gf2d, CommandSet.PREVIOUS, 
                                                              null, true);
          else if (command.equals("Hide")) 
            focusedInfo.gf2d.setVisible(false);
          else if (command.equals("Kill")) 
            svServer.fmanager.action2d(focusedInfo.gf2d, CommandSet.QUIT, 
                                                              null, true);

       } else if (focusedInfo.dim == 3 && focusedInfo.gf3d != null) {
          if (command.equals("Animate")) 
            svServer.fmanager.action3d(focusedInfo.gf3d, 
                         CommandSet.ANIMATE, null, true);    

          else if (command.equals("Stop")) {
            focusedInfo.gf3d.doStop();
            String[] arg = new String[1];
            arg[0] = String.valueOf(focusedInfo.gf3d.currentTime);
            svServer.fmanager.action3d(focusedInfo.gf3d, CommandSet.STOP, 
                                                              arg, true);  
          } else if (command.equals("Next")) 
            svServer.fmanager.action3d(focusedInfo.gf3d, CommandSet.NEXT, 
                                                              null, true);
          else if (command.equals("Previous")) 
            svServer.fmanager.action3d(focusedInfo.gf3d, CommandSet.PREVIOUS, 
                                                              null, true);
          else if (command.equals("Hide")) 
            focusedInfo.gf3d.setVisible(false);
          else if (command.equals("Kill")) 
            svServer.fmanager.action3d(focusedInfo.gf3d, CommandSet.QUIT, 
                                                              null, true);
       }

     } else {   // iconized popup menu action
       
       int sz = iconMenu.getItemCount();
       for (int j=0; j<sz; j++) {
	 if (iconMenu.getItem(j).getLabel().equals(command)) {
            for (int k=0; k<iconizedInfos.size(); k++) {
               DesktopInfo item = (DesktopInfo)iconizedInfos.elementAt(k);
               if (item.title.equals(command)) {
	          if (item.dim == 1) 
                     item.gf1d.setVisible(true);               
                  else if (item.dim == 2) 
                     item.gf2d.setVisible(true);      
                  else if (item.dim == 3) 
                     item.gf3d.setVisible(true); 
                  iconizedInfos.removeElementAt(k);
                  break;
               }
            }
            canvas.repaint();
            break;
         }
       }
     }
  }

  // make GUI of this class
  private void makeGUI() {
     setBorder(SVUIC.systemBorder);
     setLayout(new BorderLayout());
     add("Center", canvas = new DesktopCanvas(this, svServer));

     /////////////////////////////////////////////////////////
     // make popup menu
     MenuItem mi;

     popup = new PopupMenu("Activated:");

     mi = new MenuItem("Animate");
     mi.addActionListener(this);
     popup.add(mi);

     mi = new MenuItem("Stop");
     mi.addActionListener(this);
     popup.add(mi);

     mi = new MenuItem("Next");
     mi.addActionListener(this);
     popup.add(mi);

     mi = new MenuItem("Previous");
     mi.addActionListener(this);
     popup.add(mi);

     popup.addSeparator();

     mi = new MenuItem("Hide");
     mi.addActionListener(this);
     popup.add(mi);

     mi = new MenuItem("Kill");
     mi.addActionListener(this);
     popup.add(mi);

     ipop = new PopupMenu("Desktop:");

     mi = new MenuItem("Stack");
     mi.addActionListener(this);
     ipop.add(mi);

     mi = new MenuItem("Tile");
     mi.addActionListener(this);
     ipop.add(mi);

     ipop.addSeparator();

     iconMenu = new Menu("Show");
     ipop.add(iconMenu);

     add(popup); // add popup menu
     add(ipop);
     /////////////////////////////////////////////////////////

     validate();
  }


  // calc tile formation
  private int[] tileFormation(int t) {
     int ret[] = {1, 1};
     int incr = 2;
     int incc = 1;

     while(true) {
       if (t <= (incr * incc)) {
          ret[0] = incr;
          ret[1] = incc;
          return ret;
       }     
       if (incr != incc)
          incc++;
       else
          incr++;
     }
  }

} // end of class


/////////////////////////////////////////////////////////////////////////
class DesktopCanvas extends Canvas implements MouseMotionListener {
  private DesktopControl controller;
  private svserver svServer;

  private Graphics offScrGc;
  private Image offScrImage;

  // size of the canvas
  private Dimension canvasSize;

  // dragging and moving
  Vector deskVec;
  DesktopInfo selectedInfo, focusedInfo;

  DesktopCanvas(DesktopControl controller, svserver svServer) {
     this.controller = controller;
     this.svServer = svServer;

     deskVec = new Vector();

     addMouseMotionListener(this);
     enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

  /**
   * overrided update method (we need this function for remove flicker)
   */
  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {

    Dimension d = getSize();

    if ((offScrImage == null) || 
          (d.width != canvasSize.width) || (d.height != canvasSize.height)) {
       offScrImage = createImage(d.width, d.height);
       offScrGc = offScrImage.getGraphics();
       canvasSize = d;
    }

     Rectangle screenSize = controller.screenSize;

    // erase whole canvas
    offScrGc.setColor(getBackground());
    offScrGc.fill3DRect(0, 0, d.width, d.height, true);
    offScrGc.setFont(new Font("Helvetica", Font.BOLD, 10));
    FontMetrics fm = offScrGc.getFontMetrics(offScrGc.getFont());

    /////////////////////////////////////////////////////////

    int x, y, w, h;   // each virtual window x, y, w, and h
    int focusDim = 0;
    String focusTitle = null;
    Rectangle focusRect = null;
  
    // clear up rectangle vector
    deskVec.removeAllElements();
    controller.clearIconvec();

    for (int i=0; i<svServer.fmanager.get1dvecSize(); i++) {
       GFPlane gf1d = svServer.fmanager.get1dFrame(i);

       Rectangle tmp = gf1d.getBounds();
       Rectangle rect1d = new Rectangle(tmp.x-screenSize.x, tmp.y-screenSize.y,
                      tmp.width, tmp.height);

       x = (int)((rect1d.x * d.width) / screenSize.width);
       y = (int)((rect1d.y * d.height) / screenSize.height);
       w = (int)((rect1d.width * d.width) / screenSize.width);
       h = (int)((rect1d.height * d.height) / screenSize.height);

       Rectangle draw1d = new Rectangle(x, y, w, h);
       DesktopInfo info = new DesktopInfo(draw1d, gf1d);

       if (gf1d.isIconized()) { 
	  controller.addItem(info);
          continue;
       } else
          deskVec.addElement(info);

       offScrGc.setColor(getBackground().darker());
       if (gf1d.isActivated()) {
          focusDim = 1;
          focusRect = draw1d;
          focusTitle = gf1d.getTitle();

          offScrGc.setColor(Color.blue);
          drawVFrame(offScrGc, fm, gf1d.getTitle(), x, y, w, h); 

       } else if (focusedInfo != null && focusedInfo.gf1d != null &&
                     focusedInfo.gf1d.equals(gf1d)) {
          offScrGc.setColor(Color.magenta);
          drawVFrame(offScrGc, fm, gf1d.getTitle(), x, y, w, h); 
       } else {
          // draw virtrual frame
          drawVFrame(offScrGc, fm, gf1d.getTitle(), x, y, w, h);  
       }
    }

    for (int i=0; i<svServer.fmanager.get2dvecSize(); i++) {
       GFrame2D gf2d = svServer.fmanager.get2dFrame(i);

       Rectangle tmp = gf2d.getBounds();
       Rectangle rect2d = new Rectangle(tmp.x-screenSize.x, tmp.y-screenSize.y,
                      tmp.width, tmp.height);

       x = (int)((rect2d.x * d.width) / screenSize.width);
       y = (int)((rect2d.y * d.height) / screenSize.height);
       w = (int)((rect2d.width * d.width) / screenSize.width);
       h = (int)((rect2d.height * d.height) / screenSize.height);

       Rectangle draw2d = new Rectangle(x, y, w, h);
       DesktopInfo info = new DesktopInfo(draw2d, gf2d);

       if (gf2d.isIconized()) { 
	  controller.addItem(info);
          continue;
       } else
          deskVec.addElement(info);

       offScrGc.setColor(getBackground().darker());
       
       if (gf2d.isActivated()) {
          focusDim = 2;
          focusRect = draw2d;
          focusTitle = gf2d.getTitle();

          offScrGc.setColor(Color.blue);
          drawVFrame(offScrGc, fm, gf2d.getTitle(), x, y, w, h); 

       } else if (focusedInfo != null && focusedInfo.gf2d != null &&
                     focusedInfo.gf2d.equals(gf2d)) {
          offScrGc.setColor(Color.magenta);
          drawVFrame(offScrGc, fm, gf2d.getTitle(), x, y, w, h); 

       } else {
          // draw virtrual frame
          drawVFrame(offScrGc, fm, gf2d.getTitle(), x, y, w, h);  
       }
    }

    for (int i=0; i<svServer.fmanager.get3dvecSize(); i++) {
       GFrame3D gf3d = svServer.fmanager.get3dFrame(i);

       Rectangle tmp = gf3d.getBounds();
       Rectangle rect3d = new Rectangle(tmp.x-screenSize.x, tmp.y-screenSize.y,
                      tmp.width, tmp.height);

       x = (int)((rect3d.x * d.width) / screenSize.width);
       y = (int)((rect3d.y * d.height) / screenSize.height);
       w = (int)((rect3d.width * d.width) / screenSize.width);
       h = (int)((rect3d.height * d.height) / screenSize.height);

       Rectangle draw3d = new Rectangle(x, y, w, h);
       DesktopInfo info = new DesktopInfo(draw3d, gf3d);

       if (gf3d.isIconized()) { 
	  controller.addItem(info);
          continue;
       } else
          deskVec.addElement(info);

       offScrGc.setColor(getBackground().darker());
       
       if (gf3d.isActivated()) {
          focusDim = 3;
          focusRect = draw3d;
          focusTitle = gf3d.getTitle();

          offScrGc.setColor(Color.blue);
          drawVFrame(offScrGc, fm, gf3d.getTitle(), x, y, w, h); 

       } else if (focusedInfo != null && focusedInfo.gf3d != null &&
                     focusedInfo.gf3d.equals(gf3d)) {
          offScrGc.setColor(Color.magenta);
          drawVFrame(offScrGc, fm, gf3d.getTitle(), x, y, w, h); 
       } else {
          // draw virtrual frame
          drawVFrame(offScrGc, fm, gf3d.getTitle(), x, y, w, h);  
       }
    }

    ///////////////////////////////////////////////////////////////

    g.drawImage(offScrImage, 0, 0, this);
  }  

  // draw the title string
  private void drawVFrame(Graphics g, FontMetrics fm, String title, 
           int x, int y, int w, int h) {
     int len = title.length();
     int charwid = 0, pos = 0;

     char[] tarr = new char[len];
     title.getChars(0, len, tarr, 0);

     g.fill3DRect(x, y, w, h, true);
     
     for (int i=0; i<len; i++) {
        if (charwid > w-10) {
           pos = i;
           break;
        } else 
           charwid += fm.charWidth(tarr[i]);
        pos = len;
     }

     title = new String(tarr, 0, pos);
     int gap = w - fm.stringWidth(title);

     g.setColor(Color.white);
     g.drawString(title, x+(int)gap/2, y+(int)h/2);
  }


  public void mouseDragged(MouseEvent e)  {
     if (selectedInfo == null) return;
     float stepx = (float)e.getX() / (float)canvasSize.width;
     float stepy = (float)e.getY() / (float)canvasSize.height;

     Rectangle screenSize = controller.screenSize;

     if (selectedInfo.dim == 1) {
       if (selectedInfo.gf1d != null) {
          int xx = (int)((float)screenSize.width*stepx);
	  int yy = (int)((float)screenSize.height*stepy);

          selectedInfo.gf1d.setLocation(xx, yy);
          selectedInfo.gf1d.validate();
          selectedInfo.gf1d.pack();
       }
     
     } else if (selectedInfo.dim == 2) {  
          int xx = (int)((float)screenSize.width*stepx);
          int yy = (int)((float)screenSize.height*stepy);
          selectedInfo.gf2d.setLocation(xx, yy);
          selectedInfo.gf2d.validate();
          selectedInfo.gf2d.pack();

     } else if (selectedInfo.dim == 3) {  
          int xx = (int)((float)screenSize.width*stepx);
          int yy = (int)((float)screenSize.height*stepy);
          selectedInfo.gf3d.setLocation(xx, yy);
          selectedInfo.gf3d.validate();
          selectedInfo.gf3d.pack();

     }

     repaint();
  }

  public void mouseMoved(MouseEvent evt) {
     int x = evt.getX();
     int y = evt.getY();

     int size = deskVec.size();
     for (int i=size-1; i>=0; i--) {
        DesktopInfo info = (DesktopInfo)deskVec.elementAt(i);
           
        if (info.rect.contains(x, y)) {
            focusedInfo = info;
            repaint();
            return;
        }         
     }
     if (focusedInfo != null) {
        focusedInfo = null;
        repaint();
     }
  }


  // overrided function
  public void processMouseEvent(MouseEvent evt) {
     int x = evt.getX();
     int y = evt.getY();
     int size = deskVec.size();

     if (evt.getID() == MouseEvent.MOUSE_PRESSED) { 

        for (int i=size-1; i>=0; i--) {
           DesktopInfo info = (DesktopInfo)deskVec.elementAt(i);
           
           if (info.rect.contains(x, y)) {
              selectedInfo = info;
              break;
           }         
        }   

     } else if (evt.getID() == MouseEvent.MOUSE_RELEASED) { 
        selectedInfo = null;
     }

     if (evt.isPopupTrigger()) { 
        for (int i=size-1; i>=0; i--) {
           DesktopInfo info = (DesktopInfo)deskVec.elementAt(i);
           
           if (info.rect.contains(x, y)) {
              controller.setTarget(info);
              controller.popup.show(evt.getComponent(), 
                                              evt.getX(), evt.getY());
              super.processMouseEvent(evt);
              return;
           }         
        }   
        controller.setTarget(null);
        controller.ipop.show(evt.getComponent(), evt.getX(), evt.getY());
     }

     super.processMouseEvent(evt);
  }



} // end of class


/////////////////////////////////////////////////////////////////////
class DesktopInfo {

   Rectangle rect;
   int dim;
   GFPlane gf1d = null;
   GFrame2D gf2d = null;
   GFrame3D gf3d = null;
   String title;
   boolean iconized = false;

  DesktopInfo(Rectangle rect, GFPlane gf1d) {
    this.rect = rect;
    this.gf1d = gf1d;
    title = gf1d.getTitle();
    dim = 1;
  }
  
  DesktopInfo(Rectangle rect, GFrame2D gf2d) {
    this.rect = rect;
    this.gf2d = gf2d;
    title = gf2d.getTitle();
    dim = 2;
  }

  DesktopInfo(Rectangle rect, GFrame3D gf3d) {
    this.rect = rect;
    this.gf3d = gf3d;
    title = gf3d.getTitle();
    dim = 3;
  }

} // end of class






