import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.ui.*;
import sv.kernel.*;

/** 
 * FilterEditor.java
 * filter editor
 * @version 1.0
 * @author Byeongseob Ki
 */
public class FilterEditor extends Frame
                 implements ActionListener, ItemListener {

   svserver svServer;

   // main panel with card layout
   Panel panels; 
   Vector pvec;
   int currentpos = 0;

   public FilterEditor(svserver svServer) {
      super("Scivis Filter Creation Widget");
      this.svServer = svServer;

      setLayout(new BorderLayout());
      add("Center", panels = new Panel());
      panels.setLayout(new CardLayout());

      pvec = new Vector();
      makePanels();
   }


   String filterName = "NewFilter.java";
   String packageName = "user";
   
  /**
   * overrided method
   */
  public void actionPerformed(ActionEvent evt) {
     String command = evt.getActionCommand();

     if (command.equals("Back")) {  
         currentpos--;
         ((CardLayout)panels.getLayout()).show(panels, 
                   ((GeneralPanel)pvec.elementAt(currentpos)).panelName);

     } else if (command.equals("Next")) {   
        currentpos++;
        ((CardLayout)panels.getLayout()).show(panels, 
                   ((GeneralPanel)pvec.elementAt(currentpos)).panelName);

        if (currentpos == 2) {
          FilterNameInputPanel p = (FilterNameInputPanel)pvec.elementAt(1);
          filterName = p.inputs.getText().trim();
          if (filterName == null || filterName.length() < 1)
             filterName = "newFilter.java";
          if (filterName.indexOf(".java") < 0)
             filterName = filterName + ".java"; 
        } else if (currentpos == 3) {
          PackageNameInputPanel p = (PackageNameInputPanel)pvec.elementAt(2);
          packageName = p.inputs.getText().trim();
        }

     } else if (command.equals("Finish")) { 
        OutputDimPanel p1 = (OutputDimPanel)pvec.elementAt(3);
        InputInfoPanel p2 = (InputInfoPanel)pvec.elementAt(4);
        ArgumentPanel p3 = (ArgumentPanel)pvec.elementAt(5);

        String outdim = "DataSet.ONED";
        int outputtype = 1;
        if (p1.twod.isSelected()) {
           outdim = "DataSet.TWOD";
           outputtype = 2;
        } else if (p1.threed.isSelected()) {
           outdim = "DataSet.THREED";
           outputtype = 3;
        }

        int inputtype = 1;
        if (p2.twod.isSelected()) inputtype = 2;
        else if (p2.threed.isSelected()) inputtype = 3;

        int args = 0;
        try {
          if (p3.inputs.getText().trim().length() > 0)
            args = Integer.valueOf(p3.inputs.getText().trim()).intValue();
        } catch (Exception E) {
          System.out.println("Number format error for the number of args.");
        }

        String constructorStr = "super(" + outdim;

        if (inputtype == 1) {
          if (args == 0) 
            constructorStr = constructorStr + ")";
          else
            constructorStr = constructorStr + ", " + String.valueOf(args) + ")";

        } else {
          if (inputtype == 2) 
              constructorStr = constructorStr + ", 0, 2)";
          else if (inputtype == 3) 
              constructorStr = constructorStr + ", 0, 999)";
        }
        

        try {
          PrintWriter out = new PrintWriter(new FileWriter(filterName));

          out.println("// Generated from Scivis filter creation widget.");
          out.println("// For further information, please refer other filter");
          out.println("// implementations included in Scivis release.\n");

          if (packageName.length() >= 1)
             out.println("package " + packageName + ";\n");

          out.println("import java.util.*;");
          out.println("import sv.kernel.*;\n");
          out.println("/**");
          out.println(" * " + filterName);
          out.println(" */\n");
          
          int pos = filterName.indexOf(".java");
          if (pos > 0)
             filterName = filterName.substring(0, pos);
 
          out.println("public class " + filterName + " extends Filter {\n");
          
          out.println("\tpublic " + filterName + "() {");
          out.println("\t\t" + constructorStr);
          out.println("\t}\n");

        
          out.println("\tpublic Object performFilter() {\n");
          out.println("\t\tString title = \"\";");
          out.println("\t\tVector timeDataVec = new Vector();\n");
          
          if (outputtype == 1 && inputtype == 1) { 
             out.println("\t\tint points;\n");
             out.println("\t\tTimeData1D onetimedata;");
             out.println("\t\tDataSet1D dataSet = (DataSet1D)sources.firstElement();");
             out.println("\n\t\tfor (int i=0; i<dataSet.getnoOfTime(); i++) {");
             out.println("\t\t  onetimedata = (TimeData1D)dataSet.timeData[i];");
             out.println("\t\t  points = onetimedata.getnoOfPoints();");
             out.println("\n\n\t\t  ///////////////////////////////////////////");
             out.println("\t\t  // ADD YOUR CODES HERE");
             out.println("\t\t  ///////////////////////////////////////////\n\n");
             out.println("\t\t  onetimedata = new TimeData1D(dataSet.getTimeValue(i), points, \n\t\t\t\tonetimedata.getxArr(), onetimedata.getyArr());");

             out.println("\t\t  timeDataVec.addElement(onetimedata);");
             out.println("\t\t}");

             out.println("\t\treturn new DataSet1D(title, timeDataVec);");

          } else if (outputtype == 2 && inputtype == 1) {
             out.println("\t\tint points;\n");
             out.println("\t\tTimeData2D onetimedata;");
             out.println("\t\tDataSet2D dataSet = (DataSet2D)sources.firstElement();");
             out.println("\n\t\tfor (int i=0; i<dataSet.getnoOfTime(); i++) {");
             out.println("\t\t  onetimedata = (TimeData2D)dataSet.timeData[i];");
             out.println("\t\t  points = onetimedata.getnoOfPoints();");
             out.println("\n\n\t\t  ///////////////////////////////////////////");
             out.println("\t\t  // ADD YOUR CODES HERE");
             out.println("\t\t  ///////////////////////////////////////////\n\n");
             out.println("\t\t  onetimedata = new TimeData2D(dataSet.getTimeValue(i), points, \n\t\t\t\tonetimedata.getxArr(), onetimedata.getyArr(), onetimedata.getzArr());");

             out.println("\t\t  timeDataVec.addElement(onetimedata);");
             out.println("\t\t}");
             out.println("\t\treturn new DataSet2D(title, timeDataVec);");

          } else if (outputtype == 3 && inputtype == 1) {
             out.println("\t\treturn new DataSet3D(title, timeDataVec);");
          }
          out.println("\t}\n");


          out.println("} // end of class");
         
          out.close();

        } catch (Exception e) {
          System.out.println("\n\nFile writing error ...\n\n");
        }        

        svServer.mainb.editBtn.setEnabled(true);
        pvec.removeAllElements();
        dispose();

     } else if (command.equals("Cancel")) { 
        svServer.mainb.editBtn.setEnabled(true);
        pvec.removeAllElements();
        dispose();
     }
  }

  /**
   * overrided method 
   */
  public void itemStateChanged(ItemEvent e) {
  }

  private void makePanels() {
     GeneralPanel p;
     Image img = 
         getToolkit().getImage(getClass().getResource("icons/request.gif"));
     pvec.addElement(p = new IntroductionPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new FilterNameInputPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new PackageNameInputPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new OutputDimPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new InputInfoPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new ArgumentPanel(this, img));
     panels.add(p.panelName, p);

     pvec.addElement(p = new FinalPanel(this, img));
     panels.add(p.panelName, p);
  }


  //////////////////////////////////////////////////////////////////
  // Super class of all card panels
  class GeneralPanel extends Panel {
    public String panelName;
    public int panelIndex;

    GeneralPanel() {

    }
  } // end of GeneralPanel


  //////////////////////////////////////////////////////////////////
  // Introduction Panel
  class IntroductionPanel extends GeneralPanel { 
    FilterEditor parent;

    IntroductionPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "INTRO";
       panelIndex = 0;

       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Introduction"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("Welcome to Scivis filter creator widget."));

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
       Button btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);

    }
  }

  //////////////////////////////////////////////////////////////////
  // Filter name  Panel
  class FilterNameInputPanel extends GeneralPanel { 
    FilterEditor parent;

    TextField inputs;

    FilterNameInputPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "FILENAME";
       panelIndex = 1;
       
       inputs = new TextField(20);

       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Filter Name"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("First, you have to input filter name."));
       p2.add(new Label("Enter the filter name: "));
       inputs = new TextField(30);
       inputs.setText("NewFilter.java");
       p2.add(inputs);

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);
    }
  }
  


  //////////////////////////////////////////////////////////////////
  // Package name  Panel
  class PackageNameInputPanel extends GeneralPanel { 
    FilterEditor parent;

    TextField inputs;

    PackageNameInputPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "PACKAGENAME";
       panelIndex = 2;
       
       inputs = new TextField(20);

       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Package Name"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("Package is a Java's mechanism for grouping"));
       p2.add(new Label("classes. Enter the package name: "));
       inputs = new TextField(30);
       inputs.setText("user");
       p2.add(inputs);
       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);
    }
  }


  //////////////////////////////////////////////////////////////////
  // Output dimention Panel
  class OutputDimPanel extends GeneralPanel { 
    FilterEditor parent;

    JRadioButton oned, twod, threed;

    OutputDimPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "OUTPUTDIM";
       panelIndex = 3;
       
       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Output Dimension"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("Choose output dimension:"));

       ButtonGroup group = new ButtonGroup();;
       oned = new JRadioButton("1D");
       oned.setToolTipText("1D DataSet output");
       group.add(oned);
       oned.addItemListener(parent);
       oned.setSelected(true);
       p2.add(oned);

       twod = new JRadioButton("2D");
       twod.setToolTipText("2D DataSet output");
       group.add(twod);
       twod.addItemListener(parent);
       p2.add(twod);

       threed = new JRadioButton("3D");
       threed.setToolTipText("3D DataSet output");
       group.add(threed);
       threed.addItemListener(parent);
       p2.add(threed);

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);
    }
  }

  //////////////////////////////////////////////////////////////////
  // input info Panel
  class InputInfoPanel extends GeneralPanel { 
    FilterEditor parent;

    JRadioButton oned, twod, threed;

    InputInfoPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "INPUTINFO";
       panelIndex = 4;
       
       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Data Set"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("How many data set(s) do you need?"));

       ButtonGroup group = new ButtonGroup();;
       oned = new JRadioButton("1");
       oned.setToolTipText("Only one data set window");
       group.add(oned);
       oned.addItemListener(parent);
       oned.setSelected(true);
       p2.add(oned);

       twod = new JRadioButton("2");
       twod.setToolTipText("Two data set windows");
       group.add(twod);
       twod.addItemListener(parent);
       p2.add(twod);

       threed = new JRadioButton("many");
       threed.setToolTipText("Many data set windows");
       group.add(threed);
       threed.addItemListener(parent);
       p2.add(threed);

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);
    }
  }

 //////////////////////////////////////////////////////////////////
  // number of arguments  Panel
  class ArgumentPanel extends GeneralPanel { 
    FilterEditor parent;

    TextField inputs;

    ArgumentPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "ARGUMENT";
       panelIndex = 5;
       
       inputs = new TextField(20);

       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Number of Argument(s)"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("How many arguments do you need?"));
       p2.add(new Label("Enter the number of arguments: "));
       inputs = new TextField(30);
       inputs.setText("0");
       p2.add(inputs);

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Next");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);
    }
  }
  


  //////////////////////////////////////////////////////////////////
  // Final Panel
  class FinalPanel extends GeneralPanel { 
    FilterEditor parent;

    FinalPanel(FilterEditor parent, Image img) {
       this.parent = parent;
       panelName = "Final";
       panelIndex = 6;

       setLayout(new BorderLayout());    

       // make center part(instruction panel)
       JPanel p = new JPanel();
       BorderLayout bl = new BorderLayout();
       p.setLayout(bl);
       bl.setHgap(10);
       p.setBorder(new TitledBorder("Finish"));
     
       SimpleCanvas sc = new SimpleCanvas(img);
       sc.setSize(40, 40);
       p.add("West", sc);

       Panel p2 = new Panel();
       Label name;
       p2.setLayout(new BoxLayout(p2, BoxLayout.Y_AXIS));
       p2.add(new Label("Scivis will generate the filter template:"));
       p2.add(name = new Label(parent.filterName));
       name.setForeground(Color.blue);
       p2.add(new Label("You can edit this template with your editor."));

       p.add("Center", p2);       

       // make lower part(two buttons)
       Panel lowerPanel = new Panel();
       lowerPanel.setLayout(new BorderLayout());
       lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
       Button btn = new Button("Back");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Finish");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       btn = new Button("Cancel");
       btn.addActionListener(parent);
       lowerPanel.add(btn);

       add("Center", p);
       add("South", lowerPanel);

    }
  }

} // end of class


