import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;

import com.sun.java.swing.*;

import sv.util.*;
import sv.ui.*;
import sv.drawer.*;
import sv.kernel.*;

/** 
 * GFPOneD.java
 * Define a frame to draw the graph
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

class GFPOneD extends GFPlane {

  // various MenuItems
  private MenuItem lmoveMI;

  /**
   * constructor
   * @param title title of this class
   * @param dataSet dataSet for this class
   * @param svServer svserver
   */
  public GFPOneD(String title, DataSet1D dataSet, 
                            CPlane config, svserver svServer) {
     super(title, dataSet, config, svServer);

     // layout others     
     setLayout(new BorderLayout());
     canvas = new GCPOneD(this);

     canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
     DHV.addElement(canvas.dh);

     boolean b = false;

     if (config.colormap != null) {
         b = readColorMap(config.colormap);

         if (b) {
           float value;

           for (int i=0; i<101; i++) {
              value = rspline.computePoint((float)i/100f); 
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              rcolors[i] = value;

              value = gspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              gcolors[i] = value;

              value = bspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              bcolors[i] = value;
           }
         }
     }

     if (config.colormap == null || b == false) {
         int nControls = 5;
         float[] xcontrols = { 0.0f, 0.25f, 0.5f, 0.75f, 1.0f };
         float[] ycontrols = { 0.0f, 0.5f, 1.0f, 0.5f, 0.0f };
         rspline = new NewtonMethod(xcontrols, ycontrols);
         hspline = new NewtonMethod(xcontrols, ycontrols);

         for (int i=0; i<5; i++)
           ycontrols[i] = 0.5f;
         gspline = new NewtonMethod(xcontrols, ycontrols);
         sspline = new NewtonMethod(xcontrols, ycontrols);

         ycontrols[0] = 1.0f;
         ycontrols[1] = 0.5f; 
         ycontrols[2] = 0.0f;
         ycontrols[3] = 0.5f;
         ycontrols[4] = 1.0f;
         bspline = new NewtonMethod(xcontrols, ycontrols);
         lspline = new NewtonMethod(xcontrols, ycontrols);
     }

     add("Center", canvas);
     add("South", monitor = new SimpleMonitor());
     monitor.display(dataSet.getTitle() + " created.", false);
  }   


  /**
   * do additional work after get the user input
   * @param command command to do work
   * @param input user input
   */
  public void doWork(int command, String input) {
     super.doWork(command, input);

     if (command == CSPlane.ADDTIME) {
        int index = 0, size = 0;
        float[] xxarr, yyarr;
        Vector vec = new Vector();     
        String title = null;

        try {   
           for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
              DataSet1D data = (DataSet1D)e.nextElement();
           
              for (int i=0; i<data.getnoOfTime(); i++) 
                 size += data.timeData[i].getxArr().length;

              float[] xArr = new float[size];
              float[] yArr = new float[size];
              float[] zArr = new float[size];

              for (int i=0; i<data.getnoOfTime(); i++) {
                xxarr = data.timeData[i].getxArr();
                yyarr = data.timeData[i].getyArr();
           
                System.arraycopy(xxarr, 0, xArr, index, xxarr.length);
                System.arraycopy(yyarr, 0, zArr, index, xxarr.length);
           
                for (int j=0; j<xxarr.length; j++)
                  yArr[j+index] = data.timeData[i].getTime();
                
                index += xxarr.length;
             }
             title = "Added: " + data.getTitle();
             TimeData2D td2d = new TimeData2D(0.0f, index, xArr, yArr, zArr);
             vec.addElement(td2d);
           }
        } catch(Exception e) {
           System.out.println("Out of memory to perform action ...");
           return;
        }
        DataSet2D dataset = new DataSet2D(title, vec);
        long frameID = svServer.fmanager.register(dataset, -1, null);
        GFrame2D gf2d = (GFrame2D)svServer.fmanager.getGFrame(frameID);
        gf2d.config.yLabel = "TIME";
        gf2d.config.zLabel = "Y";
        if (!stealthMode)
           svServer.cmanager.broadcastDataSet(dataset, frameID);

    }  else if (command == CSPlane.LEGENDCHANGE) {
        int size = dataSetVector.size();

        String names[] = new String[size];
        Color colors[] = new Color[size];
        int markers[] = new int[size];

        int idx = 0;
        for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
           DataSet1D data = (DataSet1D)e.nextElement();
           names[idx] = data.getTitle();
           colors[idx] = data.getLineColor();
           markers[idx] = data.getMarkerType();
           idx++;
        }
        LegendChange chooser = new LegendChange(this, names, colors, markers);
        chooser.show();

    } else if (command == CSPlane.LEGENDMOVE) {
        ((GCPOneD)canvas).isLegendMove = true;
        canvas.repaint();

    } else if (command == CSPlane.LABELMOVE) {
        canvas.isLabelMove = 3;
        canvas.repaint();

    } 

  } 


  /**
   * action for boolean arguments
   */
  public void doAction_1(int id, String arg) {
     boolean b;
     if (arg.equals("true"))
        b = true;
     else
        b = false;

     switch (id) {
      case CSPlane.CONNECTPOINT:
          lineMI.setState(b);
          config.connect_point = lineMI.getState();
          break;

      case CSPlane.DISPLAYLBL:
          labelMI.setState(b);
          config.display_label = labelMI.getState();

          if (config.display_label) {
             clabelMI.setEnabled(true);
             labelmMI.setEnabled(true);
          } else {
             clabelMI.setEnabled(false);
             labelmMI.setEnabled(false);
          }
            
          canvas.changeNotify();
          break;

      case CSPlane.TICKMARK:
          tickerMI.setState(b);
          config.tickmark = tickerMI.getState();

          if (config.tickmark) 
             ctickerMI.setEnabled(true);
          else
             ctickerMI.setEnabled(false);

          canvas.changeNotify();
          break;

      case CSPlane.DISPLAYLGD:
          legendMI.setState(b);
          config.display_legend = legendMI.getState();
          lmoveMI.setEnabled(b);

          if (config.display_legend) 
             clegendMI.setEnabled(true);
          else
             clegendMI.setEnabled(false);

          canvas.changeNotify();
          break;

      case CSPlane.GMINMAX:
          checkMI.setState(b);
          config.global_mode = b;
          globalFlag = checkMI.getState();
          if (globalFlag)
              calcGlobalMinMax();
          canvas.minmaxArr = getMinMax();
          break;

      case CSPlane.AMR:
          amrMI.setState(b);
          if (amrMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               for (int j=0; j<dataSet1D.getnoOfTime(); j++)
                  ((TimeData1D)(dataSet1D.timeData[j])).calcAMRColor(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);
            }
          }
          config.amr = amrMI.getState();

          if (config.amr) {
             cinfoMI.setState(false);
             config.color_by_info = cinfoMI.getState();
             csetMI.setState(false);
             config.color_by_set = csetMI.getState();
          }

          break;

      case CSPlane.COLORBYINFO:
          cinfoMI.setState(b);
          if (cinfoMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               for (int j=0; j<dataSet1D.getnoOfTime(); j++)
                  ((TimeData1D)(dataSet1D.timeData[j])).calcColorByInfo(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);
            }
          }
          config.color_by_info = cinfoMI.getState();

          if (config.color_by_info) {
             amrMI.setState(false);
             config.amr = amrMI.getState();
             csetMI.setState(false);
             config.color_by_set = csetMI.getState();
          }
          break;

      case CSPlane.COLORBYSET:
          csetMI.setState(b);
 
          Color[] colorIndex = {
             Color.red, Color.blue, Color.magenta, Color.orange,
             Color.green, Color.pink, Color.yellow, Color.cyan
          };

          Color[] carrs= new Color[dataSetVector.size()];
          for (int i=0; i<carrs.length; i++) {
             if (i < colorIndex.length) 
                carrs[i] = colorIndex[i];
             else
                carrs[i] = Color.black;
          }

          int cidx = 0; 
          if (csetMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSet1D dataSet1D = (DataSet1D)e.nextElement();
               dataSet1D.setLineColor(carrs[cidx++]);
            }
          }
          carrs = null;
          config.color_by_set = csetMI.getState();

          if (config.color_by_set) {
             amrMI.setState(false);
             config.amr = amrMI.getState();
             cinfoMI.setState(false);
             config.color_by_info = cinfoMI.getState();
          }
          break;

      case CSPlane.DISPLAYTIME:
          timeMI.setState(b);
          config.display_time = timeMI.getState();
          break;

      case CSPlane.FILTEROVERLAY:
          overlayMI.setState(b);
          config.overlay_mode = overlayMI.getState();
 
          if (config.overlay_mode) {
             monitor.display("Filter overlay mode is on.", true);
             discardMI.setState(false);
          } else {
             monitor.display("Filter overlay mode is off.", true);
             discardMI.setState(true);
          }
          config.discard_mode = discardMI.getState(); 
          return;

      case CSPlane.FILTERDISCARD:
          discardMI.setState(b);
          config.discard_mode = discardMI.getState();
 
          if (config.discard_mode) {
             monitor.display("Filter discard mode is on.", true);
             overlayMI.setState(false);
          } else {
             monitor.display("Filter discard mode is off.", true);
             overlayMI.setState(true);
          }
          config.overlay_mode = overlayMI.getState();
          return;


     }

     canvas.repaint();
  }

  public void doAction_2(int id, String args[]) {
     int x, y, w, h;

     switch (id) {
      case CSPlane.BACKCOLOR:
          int r = Integer.valueOf(args[0]).intValue();
          int g = Integer.valueOf(args[1]).intValue();
          int b = Integer.valueOf(args[2]).intValue();
          config.bgColor = new Color(r, g, b);
          break;

      case CSPlane.RESIZEPDA:
          x = Integer.valueOf(args[0]).intValue();
          y = Integer.valueOf(args[1]).intValue();
          w = Integer.valueOf(args[2]).intValue();
          h = Integer.valueOf(args[3]).intValue();
          canvas.setPDA(new Rectangle(x, y, w, h));
          break;

      case CSPlane.LEGENDMOVE:
          x = Integer.valueOf(args[0]).intValue();
          y = Integer.valueOf(args[1]).intValue();
          w = Integer.valueOf(args[2]).intValue();
          h = Integer.valueOf(args[3]).intValue();
          config.legendRect = new Rectangle(x, y, w, h);
          break;

      case CSPlane.LABELMOVE:
          config.xLabelPoint.x = Integer.valueOf(args[0]).intValue();
          config.xLabelPoint.y = Integer.valueOf(args[1]).intValue();
          config.yLabelPoint.x = Integer.valueOf(args[2]).intValue();
          config.yLabelPoint.y = Integer.valueOf(args[3]).intValue();
          break;

      case CSPlane.TICKERCHANGE:
          int ints[] = { 
              Integer.valueOf(args[0]).intValue(), 
              Integer.valueOf(args[1]).intValue(),  
              Integer.valueOf(args[2]).intValue(), 
              Integer.valueOf(args[3]).intValue(), 
              Integer.valueOf(args[4]).intValue(),
              Integer.valueOf(args[5]).intValue(), 
              Integer.valueOf(args[6]).intValue(), 
              Integer.valueOf(args[7]).intValue(), 
              Integer.valueOf(args[8]).intValue(), 
              Integer.valueOf(args[9]).intValue() };

          boolean bools[] = new boolean[2];
          if (args[10].equals("true"))
              bools[0] = true;
          else
              bools[0] = false;

          if (args[11].equals("true"))
              bools[1] = true;
          else
              bools[1] = false;

          canvas.axis.setSetting(ints, bools);
          break;
      }

      canvas.repaint();
  }

  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);

     for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
        DataSet1D dataSet1D = (DataSet1D)e.nextElement();
        for (int j=0; j<dataSet1D.getnoOfTime(); j++) {
           if (config.amr)         
              ((TimeData1D)(dataSet1D.timeData[j])).calcAMRColor(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);   
           else if (config.color_by_info) 
              ((TimeData1D)(dataSet1D.timeData[j])).calcColorByInfo(colorModel, rcolors, gcolors, bcolors, hcolors, scolors, lcolors);

        }
     }

     canvas.repaint();
   }       

  /**
   * legend change action
   */
  public void doLegendChange(String args[]) {
    if (args.length < dataSetVector.size() * 3)
        return;

    int index = 0;
    for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
        DSPlane data = (DSPlane)e.nextElement();
        data.setTitle(args[index]);  index++;
        int rgb = Integer.valueOf(args[index]).intValue(); index++;

        ((DataSet1D)data).setLineColor(new Color(rgb));
        ((DataSet1D)data).setMarkerType(Integer.valueOf(args[index]).intValue()); index++;
    }

    canvas.repaint();
  }

 
  // write all objects of this frame to a file
  public void writeToFile(String fname) {
     DataOutputStream os = null;
     byte[] byteArr;
     int sz = dataSetVector.size();
   
     try {
        os = new DataOutputStream(
               new BufferedOutputStream(new FileOutputStream(fname)));
     } catch (Exception e) {
        System.out.println("File writing error(1d NORMAL) ...");
        return;
     }

     try {
        // write dimension
        os.writeInt(DataSet.ONED);    

        // write frame title
        byteArr = frameTitle.getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);
              
        // write configuration stream
        byteArr = config.getFileSettings().getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);

        // write drawing objects stream
        String drawingobjects = getDHVString();
        if (drawingobjects != null) {
           byteArr = getDHVString().getBytes();
           os.writeInt(byteArr.length); 
           os.write(byteArr, 0, byteArr.length);
        } else 
           os.writeInt(0); 

        // write the number of data sets
        os.writeInt(sz);  

        // now write the data sets
        for (int i=0; i<sz; i++) {
           DataSet1D dataSet = (DataSet1D)dataSetVector.elementAt(i);

           byteArr = dataSet.getTitle().getBytes();
           os.writeInt(byteArr.length); 
           os.write(byteArr, 0, byteArr.length);

           int noOfTimeData = dataSet.timeData.length;
           os.writeInt(noOfTimeData);   // send number of TimeData
           for (int k=0; k<noOfTimeData; k++) 
              dataSet.timeData[k].sendingData(os);
        } 

        os.close();
     } catch (Exception e) {
        System.out.println("Can't write the 1d data to a file ...");
     }     

  }


  // make menu system
  protected void makeMenu() {
     MenuBar menubar = new MenuBar();
     CSPlane command;
     MenuItem mi;
     int filter_command = 1000;

     // File manager
     Menu fileMenu = new Menu("File");

     addMI("Save as ...", fileMenu, new CSPlane(this, CSPlane.SAVEAS));
     addMI("Print", fileMenu, new CSPlane(this, CSPlane.PRINT));
     addMI("GIF Sequences", fileMenu, 
                              new CSPlane(this, CSPlane.GIFSEQUENCE));
     fileMenu.addSeparator();
     addMI("Quit C-q", fileMenu, new CSPlane(this, CSPlane.QUIT));

     menubar.add(fileMenu);

     // Cut, Copy and Paste
     Menu editMenu = new Menu("Edit");
     addMI("Cut", editMenu, new CSPlane(this, CSPlane.CUT));
     addMI("Copy", editMenu, new CSPlane(this, CSPlane.COPY));
     addMI("Paste", editMenu, new CSPlane(this, CSPlane.PASTE));

     menubar.add(editMenu);

     // Configuration
     Menu configMenu = new Menu("View");

     Menu colorMenu = new Menu("Colors");

     amrMI = new CheckboxMenuItem("By AMR");
     command = new CSPlane(this, CSPlane.AMR);
     amrMI.addItemListener(command);
     amrMI.setState(config.amr);
     colorMenu.add(amrMI);

     cinfoMI = new CheckboxMenuItem("By Color Info");
     command = new CSPlane(this, CSPlane.COLORBYINFO);
     cinfoMI.addItemListener(command);
     cinfoMI.setState(config.color_by_info);
     colorMenu.add(cinfoMI);

     csetMI = new CheckboxMenuItem("By Data Set");
     command = new CSPlane(this, CSPlane.COLORBYSET);
     csetMI.addItemListener(command);
     csetMI.setState(config.color_by_set);
     colorMenu.add(csetMI);

     addMI("Colormap Editor", colorMenu, new CSPlane(this, CSPlane.COLORMAP));

     mi = new MenuItem("Background Color");
     ActionListener bgColorChooser = new ActionListener() {
	public void actionPerformed(ActionEvent e) {
           Color c = 
              JColorChooser.showDialog(GFPOneD.this, 
                                      "Background Color", config.bgColor);
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRed()), 
                             String.valueOf(c.getGreen()), 
                             String.valueOf(c.getBlue()) };
           svServer.fmanager.action1d(GFPOneD.this, 
                                    CSPlane.BACKCOLOR, args, true);
	}
     };
     mi.addActionListener(bgColorChooser);
     colorMenu.add(mi);

     configMenu.add(colorMenu);

     lineMI = new CheckboxMenuItem("Connect Points");
     command = new CSPlane(this, CSPlane.CONNECTPOINT);
     lineMI.addItemListener(command);
     lineMI.setState(config.connect_point); 
     configMenu.add(lineMI);

     Menu labelMenu = new Menu("Labels");
     labelMI = new CheckboxMenuItem("Display Labels");
     command = new CSPlane(this, CSPlane.DISPLAYLBL);
     labelMI.addItemListener(command);
     labelMI.setState(config.display_label);
     labelMenu.add(labelMI);

     clabelMI = new MenuItem("Change Labels");
     command = new CSPlane(this, CSPlane.LABELCHANGE);
     clabelMI.addActionListener(command);
     labelMenu.add(clabelMI);

     labelmMI = new MenuItem("Move Labels");
     command = new CSPlane(this, CSPlane.LABELMOVE);
     labelmMI.addActionListener(command);
     labelMenu.add(labelmMI);


     if (config.display_label) {
        clabelMI.setEnabled(true);
        labelmMI.setEnabled(true);
     } else {
        clabelMI.setEnabled(false);
        labelmMI.setEnabled(false);
     }

     configMenu.add(labelMenu);

     timeMI = new CheckboxMenuItem("Display Time");
     command = new CSPlane(this, CSPlane.DISPLAYTIME);
     timeMI.addItemListener(command);
     timeMI.setState(config.display_time);
     configMenu.add(timeMI);

     Menu legendMenu = new Menu("Legend");
     legendMI = new CheckboxMenuItem("Display Legend");
     command = new CSPlane(this, CSPlane.DISPLAYLGD);
     legendMI.addItemListener(command);
     legendMI.setState(config.display_legend);
     legendMenu.add(legendMI);

     clegendMI = new MenuItem("Change Legends");
     command = new CSPlane(this, CSPlane.LEGENDCHANGE);
     clegendMI.addActionListener(command);
     legendMenu.add(clegendMI);
     if (config.display_legend)
        clegendMI.setEnabled(true);
     else
        clegendMI.setEnabled(false);


     lmoveMI = new MenuItem("Move Legends");
     command = new CSPlane(this, CSPlane.LEGENDMOVE);
     lmoveMI.addActionListener(command);
     legendMenu.add(lmoveMI);
     lmoveMI.setEnabled(legendMI.getState());

     configMenu.add(legendMenu);

     addMI("Rename Title", configMenu, 
                           new CSPlane(this, CSPlane.RENAMETITLE));
     addMI("Resize Plot Area", configMenu,
                           new CSPlane(this, CSPlane.RESIZEPDA));

     Menu tickerMenu = new Menu("Tick Marks");
     tickerMI = new CheckboxMenuItem("Display Ticks");
     command = new CSPlane(this, CSPlane.TICKMARK);
     tickerMI.addItemListener(command);
     tickerMI.setState(config.tickmark);
     tickerMenu.add(tickerMI);

     ctickerMI = new MenuItem("Change Ticks");
     command = new CSPlane(this, CSPlane.TICKERCHANGE);
     ctickerMI.addActionListener(command);
     tickerMenu.add(ctickerMI);
     if (config.tickmark)
        ctickerMI.setEnabled(true);
     else
        ctickerMI.setEnabled(false);

     configMenu.add(tickerMenu);

     checkMI = new CheckboxMenuItem("Use global minmax");
     command = new CSPlane(this, CSPlane.GMINMAX);
     checkMI.addItemListener(command);
     checkMI.setState(config.global_mode);
     configMenu.add(checkMI);

     menubar.add(configMenu);


     ////////////////////////////////////////////////////
     // FILTERS
     ////////////////////////////////////////////////////
     Menu filterMenu = new Menu("Filter");

     overlayMI = new CheckboxMenuItem("Overlay Mode");
     command = new CSPlane(this, CSPlane.FILTEROVERLAY);
     overlayMI.addItemListener(command);
     overlayMI.setState(config.overlay_mode);
     filterMenu.add(overlayMI);

     discardMI = new CheckboxMenuItem("Discard Mode");
     command = new CSPlane(this, CSPlane.FILTERDISCARD);
     discardMI.addItemListener(command);
     discardMI.setState(config.discard_mode);
     filterMenu.add(discardMI);

     filterMenu.addSeparator();

     Vector level0Vec = new Vector();
     Vector level1Vec = new Vector();
     Vector level2Vec = new Vector();
     Vector level3Vec = new Vector();

     for (Enumeration e=svServer.menuHandler.get1dElements(); 
                                               e.hasMoreElements();) {
        MenuInfo info = (MenuInfo)e.nextElement();

        if (info.level == 0) 
	  level0Vec.addElement(info);
        if (info.level == 1) 
	  level1Vec.addElement(info);
        if (info.level == 2) 
	  level2Vec.addElement(info);
        if (info.level == 3) 
	  level3Vec.addElement(info);
     } 

     for (Enumeration e=level0Vec.elements(); e.hasMoreElements();) {
        MenuInfo lv0item = (MenuInfo)e.nextElement();

        if (!lv0item.isItem) {
           Menu lv0Menu = new Menu(lv0item.name);

           for (Enumeration e1=level1Vec.elements(); e1.hasMoreElements();) {
              MenuInfo lv1item = (MenuInfo)e1.nextElement();
              
              if (lv1item.parent == lv0item.id) {
                 if (!lv1item.isItem) {
                    Menu lv1Menu = new Menu(lv1item.name);
                    
                    for (Enumeration e2=level2Vec.elements(); 
                                               e2.hasMoreElements();) {
                       MenuInfo lv2item = (MenuInfo)e2.nextElement();

                       if (lv2item.parent == lv1item.id) {
                          if (!lv2item.isItem) {
                             Menu lv2Menu = new Menu(lv2item.name);

                             for (Enumeration e3=level3Vec.elements(); 
                                               e3.hasMoreElements();) {
                                MenuInfo lv3item = (MenuInfo)e3.nextElement();
                                if (lv3item.parent == lv2item.id) {
                                   if (lv3item.isItem)  
                                      addMI(lv3item.name, lv2Menu, new CSPlane(this, ++filter_command));
                                }
			     }

                             lv1Menu.add(lv2Menu);
                          } else {
                             addMI(lv2item.name, lv1Menu, new CSPlane(this, ++filter_command));
                          }
                       }
                    }

                    lv0Menu.add(lv1Menu);
                 } else {
                    addMI(lv1item.name, lv0Menu, new CSPlane(this, ++filter_command));
                 }
              }
           }
           filterMenu.add(lv0Menu);
        } else {
           addMI(lv0item.name, filterMenu, new CSPlane(this, ++filter_command));
        }
     }
     menubar.add(filterMenu);


     // Misc menu
     Menu miscMenu = new Menu("Misc");

     addMI("Add Time Data", miscMenu, new CSPlane(this, CSPlane.ADDTIME));

     // animation menus
     Menu animationMenu = new Menu("Animation");

     animateMI = new MenuItem("Animate C-a");
     command = new CSPlane(this, CSPlane.ANIMATE);
     animateMI.addActionListener(command);
     animationMenu.add(animateMI);

     stopMI = new MenuItem("Stop C-s");
     command = new CSPlane(this, CSPlane.STOP);
     stopMI.addActionListener(command);
     animationMenu.add(stopMI);
     stopMI.setEnabled(false);

     nextMI = new MenuItem("Next C-n");
     command = new CSPlane(this, CSPlane.NEXT);
     nextMI.addActionListener(command);
     animationMenu.add(nextMI);

     prevMI = new MenuItem("Previous C-p");
     command = new CSPlane(this, CSPlane.PREVIOUS);
     prevMI.addActionListener(command);
     animationMenu.add(prevMI);
    
     addMI("Animation Speed", animationMenu, 
                    new CSPlane(this, CSPlane.ASPEEDCHANGE));

     miscMenu.add(animationMenu);

     addMI("Buffer Size", miscMenu, 
                    new CSPlane(this, CommandSet.BUFFERSIZE));

     mi = new MenuItem("Goto C-g");
     command = new CSPlane(this, CSPlane.GOTO);
     mi.addActionListener(command);
     miscMenu.add(mi);

     // Split the DataSet
     splitMI = new MenuItem("Split DataSets");
     command = new CSPlane(this, CSPlane.SPLIT);
     splitMI.addActionListener(command);
     miscMenu.add(splitMI);
     if (dataSetVector.size() < 2) 
        splitMI.setEnabled(false);

     // Stealth mode
     stealthMI = new CheckboxMenuItem("Stealth Mode");
     command = new CSPlane(this, CSPlane.STEALTH);
     stealthMI.addItemListener(command);
     stealthMI.setState(stealthMode);
     miscMenu.add(stealthMI);


     // zooming
     Menu zoomMenu = new Menu("Zoom");

     Menu zoomModeMenu = new Menu("Mode");
     addMI("xy Zoom", zoomModeMenu, new CSPlane(this, CSPlane.XYZOOM));
     addMI("x Zoom", zoomModeMenu, new CSPlane(this, CSPlane.XZOOM));
     addMI("y Zoom", zoomModeMenu, new CSPlane(this, CSPlane.YZOOM));
     addMI("no Zoom", zoomModeMenu, new CSPlane(this, CSPlane.NOZOOM));
     zoomMenu.add(zoomModeMenu);

     zoomoutMI = new MenuItem("Zoomout C-o");
     command = new CSPlane(this, CSPlane.ZOOMOUT);
     zoomoutMI.addActionListener(command);
     zoomMenu.add(zoomoutMI);
     zoomoutMI.setEnabled(false);

     mi = new MenuItem("Reset");
     command = new CSPlane(this, CSPlane.RESET);
     mi.addActionListener(command);
     zoomMenu.add(mi);
     miscMenu.add(zoomMenu);
    
     menubar.add(miscMenu);

     setMenuBar(menubar);
  }

}  // end of class

