import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;

import com.sun.java.swing.*;

import sv.util.*;
import sv.ui.*;
import sv.drawer.*;
import sv.kernel.*;

/** 
 * GFPlane.java
 * Define a frame to draw the graph
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

public abstract class GFPlane extends GFrame implements WindowListener {

  // to access parent
  public svserver svServer;

  // canvas to draw the graph
  public GCPlane canvas;


  // global min and max value
  public float globalxmin = Float.MAX_VALUE, globalxmax = Float.MIN_VALUE, 
               globalymin = Float.MAX_VALUE, globalymax = Float.MIN_VALUE;


  // animation class
  protected Animator animator;

  // Thread to creat GIF image
  protected GIFer gifer;

  // CutCopy manager dialog
  protected CutCopyManager editManager;

  // animation speed
  protected int animationSpeed = 10;

  // for merging
  int mergeIndex = 1;
  boolean mergeSelected = false;

  // various MenuItems
  protected MenuItem animateMI, stopMI, nextMI, prevMI, splitMI, labelmMI;
  public CheckboxMenuItem checkMI, tickerMI, labelMI, amrMI, lineMI, legendMI,
                    timeMI, stealthMI, overlayMI, discardMI, cinfoMI, csetMI;
  public MenuItem zoomoutMI, ctickerMI, clabelMI, clegendMI;

  // configuration
  public CPlane config;

  // data type of this window frame
  protected int dataType;

  /**
   * constructor
   * @param title title of this class
   * @param dataSet dataSet for this class
   * @param svServer svserver
   */
  public GFPlane(String title, DSPlane dataSet, 
                            CPlane config, svserver svServer) {
     super(title);

     this.svServer = svServer;
     dimension_ = DataSet.ONED;
     this.config = config;
     dataType = dataSet.getDataType();

     // copy the DataSet
     dataSetVector.addElement(dataSet);

     makeMenu();

     // initialize timeArr
     noOfTime = dataSet.getnoOfTime();
     timeArr = dataSet.getTimeArr();
     calcGlobalMinMax();
  
     // drawing handle vector
     DHV = new Vector();

     addWindowListener(this); 
  }   

  /** 
   * return data type
   */
  public int getDataType() {
     return dataType;
  }


  /**
   * calculate global min and max values for x and y
   * and update each time data
   */
  public void calcGlobalMinMax() {
    Enumeration e;
 
    globalxmin = Float.MAX_VALUE;
    globalxmax = Float.MIN_VALUE;
    globalymin = Float.MAX_VALUE;
    globalymax = Float.MIN_VALUE;

    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       for (int i=0; i<data.getnoOfTime(); i++) {
          if (globalxmin > data.timeData[i].getxmin())
             globalxmin = data.timeData[i].getxmin();
          if (globalymin > data.timeData[i].getymin())
             globalymin = data.timeData[i].getymin();
          if (globalxmax < data.timeData[i].getxmax())
             globalxmax = data.timeData[i].getxmax();
          if (globalymax < data.timeData[i].getymax())
             globalymax = data.timeData[i].getymax();
       }
    }
    
    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       for (int i=0; i<data.getnoOfTime(); i++) 
           data.timeData[i].setGlobalValues(globalxmin, globalxmax,
                                            globalymin, globalymax);
    }

    if (canvas != null)
       canvas.minmaxArr = getMinMax();
  }


  /**
   * add another dataSet to this frame
   * @param dataSet DSPlane to be added
   */
  public void addDataSet(DSPlane dataSet) {
     // reset all the currentTime value to 0
     currentTime = 0;

     dataSetVector.addElement(dataSet);
     updateTimeArr();
     calcGlobalMinMax();
     splitMI.setEnabled(true);

     monitor.display("Added new DSPlane " + dataSet.getTitle(), false);
     setFrameTitle(dataSet.getTitle() + "...");
     canvas.repaint();
  }


  /**
   * update dataSet to this frame
   * @param pos position of DataSet to be replaced
   * @param dataSet DSPlane to be updated
   */
  public void updateDataSet(int pos, DSPlane dataSet) {

     dataSetVector.removeElementAt(pos);
     dataSetVector.addElement(dataSet);
     updateTimeArr();

     calcGlobalMinMax();

     if (currentTime != noOfTime-1) {
        int saved = currentTime;
        currentTime = 0;

        // send to last time
        for (int i=0; i<saved; i++)
           next(false);
        next(true);   // one more time
     }

     monitor.display("Updated DSPlane " + dataSet.getTitle(), false);
     canvas.repaint();
  }

  /**
   * update dataSets to this frame
   */
  public void updateDataSet() {

     calcGlobalMinMax();
     canvas.repaint();
  }


  /**
   * remove d ataSet from this frame
   * @param str DSPlane title to be removed
   */
  public void removeDataSet(String str) {
     
    for (Enumeration e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();

       if (str.equals(data.getTitle())) {
          removeDataSet(data);
          break;
       }
    } 
  }

  /**
   * remove a dataSet from this frame
   * @param dataSet DSPlane to be removed
   */
  public void removeDataSet(DSPlane dataSet) {
     dataSetVector.removeElement(dataSet);

     // reset all the currentTime value to 0
     currentTime = 0;

     if (dataSetVector.size() <= 0) {
        svServer.fmanager.removeNotify(id, DataSet.ONED);
        this.dispose();
     } else {
         DSPlane data = (DSPlane)dataSetVector.firstElement();
         if (dataSetVector.size() == 1) {
            setFrameTitle(data.getTitle());
            splitMI.setEnabled(false);
         } else
            setFrameTitle(data.getTitle() + "...");
     }

     updateTimeArr();
     calcGlobalMinMax();

     monitor.display("Removed DSPlane " + dataSet.getTitle(), false);
     canvas.repaint();
  }



  /** 
   * calculate xmin, xmax, ymin, and xmax for each time slot
   * @return minmax array of min and max
   */
  public float[] getMinMax() {

    Enumeration e;
    float minmaxArr[] = new float[4];   

    minmaxArr[0] = 1.0e10f;
    minmaxArr[1] = -1.0e10f;
    minmaxArr[2] = 1.0e10f;
    minmaxArr[3] = -1.0e10f;

    if (globalFlag) {
      minmaxArr[0] = globalxmin;
      minmaxArr[1] = globalxmax;
      minmaxArr[2] = globalymin;
      minmaxArr[3] = globalymax;
      return minmaxArr;
    }

    for (e=dataSetVector.elements(); e.hasMoreElements();) {
       DSPlane data = (DSPlane)e.nextElement();
       
       for (int i=0; i<data.getTimeArr().length; i++) {
          if (timeArr[currentTime] == data.getTimeValue(i)) {
             if (minmaxArr[0] > data.timeData[i].getxmin()) 
                minmaxArr[0] =  data.timeData[i].getxmin();
             if (minmaxArr[1] < data.timeData[i].getxmax()) 
                minmaxArr[1] =  data.timeData[i].getxmax();
             if (minmaxArr[2] > data.timeData[i].getymin()) 
                minmaxArr[2] =  data.timeData[i].getymin();
             if (minmaxArr[3] < data.timeData[i].getymax()) 
                minmaxArr[3] =  data.timeData[i].getymax();
             break;
          }
       }
    }
    return minmaxArr;    
  }



  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);
  }



  /**
   * type one calulation. (Filter.NORMAL)
   */
  public void doTypeOne(Filter filter, String input, boolean broad) {
     boolean firstTime = true;
     GFPlane otherFrame = null;
     long newFrameID = -1;
     int data_type;
     Vector newDatasets = new Vector();

     int noOfInputs = filter.getNoOfArgs();

     int dsize = dataSetVector.size();

     for (int elt=0; elt<dsize; elt++) {
        DSPlane data = (DSPlane)dataSetVector.elementAt(elt);
        filter.clear();

        if (noOfInputs == 1) {
           float val = 0.0f;
           try {
              val = Float.valueOf(input).floatValue();
           } catch (Exception ee) {
              monitor.display("Input should be number...", false);
              return;
           }

           filter.assignDataSet(data);
           filter.assignArgs(val);

        } else if (noOfInputs == 2) {
           float start = 0, end = 0;
           int where = input.indexOf("-");
           if (where < 0) {
              monitor.display("Usage: start_time - end_time", false);
              return;
           } else {
              try {
                start = 
                 Float.valueOf(input.substring(0, where).trim()).floatValue();
                end = 
                 Float.valueOf(input.substring(where+1).trim()).floatValue();
              } catch (Exception ee) {
                  monitor.display("Usage: start - end", false);
                  return;
              }

              if (end < start) {
                  monitor.display("Invalid range ...", true);
                  return;
              }
           }
           filter.assignDataSet(data);
           filter.assignArgs(start, end);

        } else         // if it doesn't need an argument
           filter.assignDataSet(data);

        DSPlane newdata;
        if (filter.getOutDimension() == 1) {
	  try { 
             newdata = (DSPlane)filter.performFilter();
	  } catch (Exception ee) {
             monitor.display("Filter can't perform. Check filter codes.", true);
             return;
          }
        } else {
           monitor.display("Not supported yet...", true);
           System.out.println("This kind of filter is not support yet.");
           System.out.println("Please let me know your filter's description");
           System.out.println("to bki@top.cis.syr.edu");
           return;
        }


        if (config.overlay_mode) {
           addDataSet(newdata);
           if (broad)
	       svServer.cmanager.broadcastDataSet(newdata, id);

        } else if (config.discard_mode) {
           newDatasets.addElement(newdata);
           if (broad)
	       svServer.cmanager.broadcastDataSet(newdata, id);

        } else {
           if (firstTime) {
              firstTime = false;
              newFrameID = svServer.fmanager.register(newdata, -1, null);
              otherFrame = (GFPlane)svServer.fmanager.getGFrame(newFrameID);
              if (broad) 
                 svServer.cmanager.broadcastDataSet(newdata, newFrameID);

           } else {
               otherFrame.addDataSet(newdata);
               if (broad)
	          svServer.cmanager.broadcastDataSet(newdata, newFrameID);
           }
        }
      }

      if (config.discard_mode) {
         dataSetVector = newDatasets;
         updateTimeArr();
         calcGlobalMinMax();
         canvas.repaint();
      }
  }


  /**
   * do filter action
   * @param command action command
   */
  public void doFilterAction(String command) {
    Vector menuVec = new Vector();

    if (dataType == DSPlane.NORMAL) 
       menuVec = svServer.menuHandler.menuVec;
    else if (dataType == DSPlane.VECTOR) 
       menuVec = svServer.menuHandler.menuVec4D;

    for (Enumeration e=menuVec.elements(); e.hasMoreElements();) {
       MenuInfo info = (MenuInfo)e.nextElement();

       if (command.equals(info.name)) {
          Filter filter = null;
          String dir = "";

          try {
             if (info.directory != null)
                dir = info.directory;

             filter=(Filter)Class.forName(dir+"."+info.className).newInstance();
          } catch (Exception exception) {
             System.out.println("Can't find " + info.className + "...");
             return;
          }

          if (filter.getStyle() == Filter.NORMAL) {

             if (filter.getNoOfArgs() == 0) 
                svServer.fmanager.actionFilter(id, DataSet.ONED, null,
                                             filter, null, true);
             else {
               String result = 
                    JOptionPane.showInputDialog(null, filter.getLabel());
               if (result == null) return;
               svServer.fmanager.actionFilter(id, DataSet.ONED, null,
                                             filter, result, true);
             }
             break;
          } else if (filter.getStyle() == Filter.MULTIPLE) {
             svServer.mergeHandler.mergeMode = true;
             filter.clear();
             svServer.mergeHandler.mergeFilter = filter;
             mergeIndex = svServer.mergeHandler.mergeIndex = 1;
             mergeSelected = true;
             svServer.mergeHandler.mergeIndex++;
             canvas.repaint();

             filter.assignDataSet(new Long(id));

             if (filter.getExpectDataSets() < 2) {
               monitor.display("Check number of frames to be merged...", true);
               return;
             }
             monitor.display("Click the mouse on second GraphFrame ...", 
                                                           false);
	  }
       }
     }
  }


  /**
   * internal merge action
   * @param b if control key is pressed
   */
  public void mergeAction(boolean b) {

      int howmany = svServer.mergeHandler.mergeFilter.getDataVectorSize();
      mergeIndex = svServer.mergeHandler.mergeIndex;
      mergeSelected = true;
      svServer.mergeHandler.mergeIndex++;
      canvas.repaint();
      
      if (svServer.mergeHandler.mergeFilter.getExpectDataSets() > 100) {
	if (b) {
          svServer.mergeHandler.mergeMode = false;
          svServer.mergeHandler.mergeFilter.assignDataSet(new Long(id));
          svServer.mergeHandler.mergeFrames();
        } else {
          svServer.mergeHandler.mergeFilter.assignDataSet(new Long(id));
          monitor.display("merged ...", false);
        }
      } else if (howmany == 
             (svServer.mergeHandler.mergeFilter.getExpectDataSets() - 1)) {
         svServer.mergeHandler.mergeMode = false;
         svServer.mergeHandler.mergeFilter.assignDataSet(new Long(id));
         svServer.mergeHandler.mergeFrames();
      } else {
         svServer.mergeHandler.mergeFilter.assignDataSet(new Long(id));
         monitor.display("merged ...", false);
      }
  }


  /**
   * do additional work after get the user input
   * @param command command to do work
   * @param input user input
   */
  public void doWork(int command, String input) {
    if (command == CSPlane.SAVEAS) {
       FileDialog openFileDialog = new FileDialog(this,
                         "Save DataSet", FileDialog.SAVE);
       openFileDialog.show();
       input = openFileDialog.getFile();
       if (input == null) return;

       boolean b = input.indexOf(".") >=0 ? true : false; 
       if (!b)  input += ".svb";
       input = openFileDialog.getDirectory() + input;

       writeToFile(input);

    } else if (command == CSPlane.PRINT) {
        PrintJob printjob =  getToolkit().getPrintJob(this,
                                    "SciVis 1D Plot", printerprops);
                                                        
        if (printjob != null) {          
             Graphics pg = printjob.getGraphics();
             if (pg != null) {
                  canvas.printAll(pg);
                  pg.dispose();         // flush page
             }
             printjob.end();
        }
    
    } else if (command == CSPlane.GIFSEQUENCE) {
        FileDialog openFileDialog = new FileDialog(this,
                      "GIF Sequences save", FileDialog.SAVE);
        openFileDialog.show();

        String fname = openFileDialog.getFile();
        if (fname == null) return;
        fname = openFileDialog.getDirectory() + fname;

        if (animator != null) {
           monitor.display("Animation stoped ...", false);
           animator.stop();
           animator = null;
        }

        if (gifer != null) gifer.stop();

        gifer = new GIFer(this, fname);
        gifer.start();
    } else if (command == CSPlane.CUT) {
        doCutCopy(true);
    } else if (command == CSPlane.COPY) {
        doCutCopy(false);
    } else if (command == CSPlane.PASTE) {
        Vector clipboardVec = svServer.clipboard.getClipboard();

        int sz = clipboardVec.size();
        if (sz > 0) {
           for (int i=0; i<sz; i++) {
              DSPlane data = (DSPlane)clipboardVec.elementAt(i);
              addDataSet(data);
              if (!stealthMode)
                 svServer.cmanager.broadcastDataSet(data, id);
           }
        }

    } else if (command == CSPlane.COLORMAP) {
        ColorMapDialog colorDialog = new ColorMapDialog(this);
        colorDialog.show();

    } else if (command == CSPlane.XYZOOM) {
        canvas.zoomMode = 0;
        monitor.display("xy zoom mode setted", false);
    } else if (command == CSPlane.XZOOM) {
        canvas.zoomMode = 1;
        monitor.display("x zoom mode setted", false);
    } else if (command == CSPlane.YZOOM) {
        canvas.zoomMode = 2;
        monitor.display("y zoom mode setted", false);
    } else if (command == CSPlane.NOZOOM) {
        canvas.zoomMode = 3;
        monitor.display("no zoom mode setted", false);

    }  else if (command == CSPlane.GOTO) {
        String result = JOptionPane.showInputDialog(null, 
                             "Enter the time data index to go: ");
        if (result == null) return;
        String[] args = new String[1];
        args[0] = result.trim();
        svServer.fmanager.action1d(this, CSPlane.GOTO, args,true);

    } else if (command == CSPlane.RESIZEPDA) {
        canvas.isPlotAreaResize = true;
        canvas.repaint();

    } else if (command == CSPlane.RENAMETITLE) {
        RenameDialog chooser = new RenameDialog(this);
        chooser.show();
        String[] args = chooser.getArguments();
        if (args == null) return;
        svserver.fmanager.action1d(this, CSPlane.RENAMETITLE, args, true);

    } else if (command == CSPlane.ASPEEDCHANGE) {
        SlideInputDialog chooser = 
               new SlideInputDialog(this, "Animation Speed", 
              "Choose animation speed(msec)", 
              "Range will vary from 10 to 300 msec", 
              10, 300, animationSpeed);
        chooser.show();
        String args[] = new String[1];
        args[0] = String.valueOf(chooser.getChoosedValue());
        svserver.fmanager.action1d(this, CSPlane.ASPEEDCHANGE, args, true);
    } else if (command == CSPlane.LABELCHANGE) {
        String[] str = { config.xLabel, config.yLabel };
        Font[] f = { config.xLabelFont, config.yLabelFont };
        boolean[] b = { config.isXLabelRotate, config.isYLabelRotate};

        LabelInputDialog2 chooser = 
                  new LabelInputDialog2(this, str, f, b);

        chooser.show();
        String args[] = chooser.getArgs();
        if (args != null) 
           svserver.fmanager.action1d(this, CSPlane.LABELCHANGE, args, true);

    } else if (command == CSPlane.TICKERCHANGE) {
        TickerSetting chooser = 
          new TickerSetting(this, 
                 canvas.axis.getIntVars(), canvas.axis.getBoolVars());
        chooser.show();
        String args[] = chooser.getSettings();
        if (args != null)
          svserver.fmanager.action1d(this, CSPlane.TICKERCHANGE, args, true);
    } else if (command == CSPlane.STEALTH) {
        stealthMode = stealthMI.getState();
        if (stealthMode)
          monitor.display("Stealth mode is on.", true);
        else
          monitor.display("Stealth mode is off.", true);
    } else if (command == CSPlane.BUFFERSIZE) {
        String bsz = JOptionPane.showInputDialog(null, 
                             "Enter the maximum buffer size: ");
        if (bsz == null) return;
        String[] args = new String[1];
        args[0] = bsz.trim();
        svServer.fmanager.action1d(this, CSPlane.BUFFERSIZE, args,true);

    }
  }



  /**
   * kill this frame
   */
  public void killFrame() {

      if (animator != null) {
        animator.stop();
        animator = null;
      } 
      if (gifer != null) {
        gifer.stop();
        gifer = null;
      } 

      dataSetVector.removeAllElements();
      dataSetVector = null;

      svServer.fmanager.removeNotify(id, DataSet.ONED);
      this.dispose();
   }


  // Cut and copy command
  protected void doCutCopy(boolean cutting) {
     Vector clipboardVec = new Vector();

     if (dataSetVector.size() == 1) {
        DSPlane data = (DSPlane)dataSetVector.firstElement();
        clipboardVec.addElement(data);

        if (clipboardVec.size() > 0) {
           svServer.clipboard.setClipboard(clipboardVec);

           if (cutting) {
              removeDataSet(data);       
              svServer.cmanager.broadcastRemoveDataSet(id, data);
           }
	}
           
     } else {
       if (editManager == null) 
         editManager = 
              new CutCopyManager(this, "Cut & Copy Selection", cutting);
       editManager.show();
     }
  }

  /**
   * do multiple cut and copy
   */
  public void multiCutCopy(int[] selected, boolean cutting) {
     String str = null;
     DSPlane data;
     Vector clipboardVec = new Vector();

     if (selected.length == 1) {
        data = (DSPlane)(dataSetVector.elementAt(selected[0]));
        clipboardVec.addElement(data);
     } else if (selected.length > 1) {
        for (int i=0; i<selected.length; i++) {
           data = (DSPlane)(dataSetVector.elementAt(selected[i]));
           clipboardVec.addElement(data);
        }
     } 
         
     if (clipboardVec.size() > 0) 
        svServer.clipboard.setClipboard(clipboardVec);

     if (selected.length == 1) {
        data = (DSPlane)(dataSetVector.elementAt(selected[0]));
        if (cutting) {
           removeDataSet(data);       
           svServer.cmanager.broadcastRemoveDataSet(id, data);
        }

     } else if (selected.length > 1) {
        for (int i=selected.length-1; i>=0; i--) {
           data = (DSPlane)(dataSetVector.elementAt(selected[i]));
           if (cutting) {
              removeDataSet(data);       
              svServer.cmanager.broadcastRemoveDataSet(id, data);
           }
        }
     } 
  }

  /**
   * do animation command
   */
  public void doAnimate() {
     if (animator == null) {
        animator = new Animator(this, animationSpeed);
        animator.start();
     } else 
        System.out.println("Animation thread was corrupted ...");

     animateMI.setEnabled(false);
     nextMI.setEnabled(false);
     prevMI.setEnabled(false);
     stopMI.setEnabled(true);
  
  }

  /**
   * do animation  stop command
   */
  public void doStop() {
     if (animator != null) {
       animator.stop();
       animator = null;
     }
     animateMI.setEnabled(true);
     stopMI.setEnabled(false);
     nextMI.setEnabled(true);
     prevMI.setEnabled(true);
  }

  /** 
   * get next DataSet
   * @param done if true, repaint canvas
   */
  public void next(boolean done) {

    currentTime++;
    if (currentTime == noOfTime)
       currentTime = 0;

    if (done) {
       boolean endprocess = false;
       for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
          DrawHandle dh = (DrawHandle)e.nextElement();
          if (dh.getTime() == timeArr[currentTime]) {
             canvas.dh = dh;
             endprocess = true;
             break;
          }
       }
       if (!endprocess) {
          canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
          DHV.addElement(canvas.dh);
       }

       canvas.minmaxArr = getMinMax();
       canvas.repaint();
    }
  }
  
  /** 
   * get Previous DataSet (for Animation)
   */
  public void previous() {

    currentTime--;
    if (currentTime == -1)
       currentTime = noOfTime-1;

    boolean endprocess = false;
    for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       if (dh.getTime() == timeArr[currentTime]) {
          canvas.dh = dh;
          endprocess = true;
          break;
       }
    }
    if (!endprocess) {
       canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
       DHV.addElement(canvas.dh);
    } 
    canvas.minmaxArr = getMinMax();
    canvas.repaint();
  }


  /**
   * change color map info
   */
  public void changeColorMap(String[] args) {
     svServer.fmanager.action1d(this, CommandSet.COLORMAPCHANGE, args, true);
  }
  

  /**
   * action for boolean arguments
   */
  public abstract void doAction_1(int id, String arg);

  /**
   * action for multiple arguments
   */
  public abstract void doAction_2(int id, String args[]);

  /** 
   * split command
   */
  public void doSplit() {

     DSPlane data = (DSPlane)dataSetVector.lastElement();

     long frameID  = svServer.fmanager.register(data, -1, null);
     removeDataSet(data);

     canvas.repaint();
  }


  /**
   * label change action
   */
  public void doLabelChange(String args[]) {
      config.xLabel = args[0];
      config.yLabel = args[1];
      config.xLabelFont = new Font(args[2],
                       Integer.valueOf(args[3]).intValue(),
                       Integer.valueOf(args[4]).intValue());
      config.yLabelFont = new Font(args[5],
                       Integer.valueOf(args[6]).intValue(),
                       Integer.valueOf(args[7]).intValue());

      if (args[8].equals("true"))
         config.isXLabelRotate = true;
      else
         config.isXLabelRotate = false;

      if (args[9].equals("true"))
         config.isYLabelRotate = true;
      else
         config.isYLabelRotate = false;

      canvas.repaint();
  }

  /**
   * goto action
   */
  public void doGotoAction(String input) {
      int whereto = Integer.valueOf(input.trim()).intValue();

      if ((whereto > noOfTime) || (whereto <= 0))
         monitor.display("Invalid range ...", true);

      int index = 0;
      while (whereto != currentTime) {
         next(false);

         if (index > noOfTime) break;
         index++;
       }
      previous();
  }


  /**
   * buffer size change action
   */
  public void doBufferAction(String input) {
      int size = Integer.valueOf(input.trim()).intValue();

      if (size <= 0)
         monitor.display("Invalid range ...", true);

      bufferSize = size;

      for (Enumeration e = dataSetVector.elements(); e.hasMoreElements(); ) {
        DSPlane data = (DSPlane)e.nextElement();
        data.setBufferSize(size);
      }
 
  }
 
  /**
   * animation speed change action
   */
  public void doAnimationSpeedChange(int speed) {
     if (speed < 10) animationSpeed = 10;
     else if (speed > 500) animationSpeed = 500;
     else animationSpeed = speed;

     if (animator == null) return;
     if (animator.isAlive()) {
        animator.stop();
        animator = null;
        animator = new Animator(this, animationSpeed);
        animator.start();
     }
  }


  /**
   * send window size change notification to all whiteboard object
   */
  public void changeWinSize(Dimension d) {
     for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       dh.setWindowSize(d);
     }
  }



  /**
   * make drawing object to string
   */
  public String getDHVString() {
    if (DHV.size() < 1) return null;

    StringBuffer str = new StringBuffer();      // return value buffer
    
    for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
      DrawHandle dh = (DrawHandle)e.nextElement();

      if (dh.drawingVec.size() < 1) break;
      
      str.append(String.valueOf(dh.getTime() + "@"));
      Dimension sz = dh.getOrigWinSize();
      str.append(String.valueOf(sz.width + "@"));
      str.append(String.valueOf(sz.height + "@"));
      
      for (int i = 0; i < dh.drawingVec.size(); i++) {
         ShapeObject s = (ShapeObject)dh.drawingVec.elementAt(i);
         str.append(s.makeString() + "@");
      }
      str.append("%");       // deliminator among each time
    }

    return(new String(str));
  }


  /**
   * create drawing objects from received string
   * IT WILL BE CALLED FROM COLLABORATION MANAGER ONLY
   */
  public void createWhiteboard(String str) {
     if (str == null) return;

     StringTokenizer st = new StringTokenizer(str, "%");
     int count = st.countTokens();

     // replace current drawing handle vector
     DHV = new Vector();
     for (int i=0; i<count; i++) 
        DHV.addElement(new DrawHandle(st.nextToken()));

     for (Enumeration e=DHV.elements(); e.hasMoreElements();) {
       DrawHandle dh = (DrawHandle)e.nextElement();
       if (dh.getTime() == timeArr[currentTime]) {
          canvas.dh = dh;
          break;
       }
     }
  }


  // write all objects of this frame to a file
  public abstract void writeToFile(String fname);

  // make menu system
  protected abstract void makeMenu();

  // add menuItem to menu
  protected void addMI(String str, Menu parent, CSPlane command) {
     MenuItem mi = new MenuItem(str);
     mi.addActionListener(command);
     parent.add(mi);
  }


  // virtual desktop management
  protected boolean iconized = false;
  protected boolean activated = true;

  public boolean isIconized() {
     return iconized;
  }

  public boolean isActivated() {
     return activated;
  }

  public void windowOpened(WindowEvent e) {
  }
  public void windowClosing(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
    iconized = true;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowDeiconified(WindowEvent e) {
    iconized = false;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowActivated(WindowEvent e) {
    activated = true;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

  public void windowDeactivated(WindowEvent e) {
    activated = false;
    if (svServer.controller != null)
       svServer.controller.desktopRepaint();
  }

}  // end of class

