import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.ui.MarkerChooser;
import sv.ui.SVUIC;


/** 
 * LegendChange.java
 * To get the user input
 * @version 1.0
 * @author Byeongseob Ki
 */


public class LegendChange extends Frame 
      implements ActionListener, ItemListener {

    String names[];
    Color colors[];
    int markers[];
    int selected;

    JComboBox datasets;
    TextField namefield;
    GFPOneD gframe;

    /**
     * constructor
     */
    public LegendChange(GFPOneD gframe, 
                            String[] names, Color[] colors, int[] markers) {
        super("Legend Content Change");
        namefield = new TextField();

        this.gframe = gframe;
        this.names = names;
        this.colors = colors;
        this.markers = markers;

        makeGUI();
        pack();
    }

    public void actionPerformed(ActionEvent e) {

        String command = e.getActionCommand();

        if (e.getSource() instanceof TextField)
            names[selected] = namefield.getText().trim();

        if (command.equals("Ok")) {
            String args[] = new String[names.length * 3];
            int index = 0;
            names[selected] = namefield.getText().trim();

            for (int j = 0; j < names.length; j++) {
                args[index] = names[j]; index++;
                args[index] = String.valueOf(colors[j].getRGB());  index++;
                args[index] = String.valueOf(markers[j]); index++;
            }

            gframe.svServer.fmanager.action1d(gframe, CSPlane.LEGENDCHANGE, args, true);
            dispose();

        } else if (command.equals("Cancel")) {
            dispose();

        } else if (command.equals("Line Color")) {
            colors[selected] = 
               JColorChooser.showDialog(this, "Line Color", colors[selected]);

        } else if (command.equals("Marker Type")) {
            MarkerChooser markerChooser = 
                   new MarkerChooser(this, markers[selected]);
            markerChooser.show();
            markers[selected] = markerChooser.getType();
        }
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getSource() instanceof JComboBox && 
                     e.getStateChange() == ItemEvent.SELECTED) {
            selected = datasets.getSelectedIndex();
            namefield.setText(names[selected]);
        }
    }

    private void makeGUI() {
        Button btn;

        setLayout(new BorderLayout());

        JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
        p.setBorder(SVUIC.systemBorder);

        p.add(new Label("Current available data set(s)"));
        p.add(datasets = new JComboBox());
        datasets.addItemListener(this);
        for (int i=0; i<names.length; i++)
            datasets.addItem(names[i]);

        p.add(Box.createRigidArea(new Dimension(10, 1)));

        p.add(new Label("Data set name:"));
        p.add(namefield);
        namefield.addActionListener(this);

        Panel p2 = new Panel();
        p2.setLayout(new GridLayout(1, 2));
        p2.add(btn = new Button("Line Color"));
        btn.addActionListener(this);
        p2.add(btn = new Button("Marker Type"));
        btn.addActionListener(this);
        p.add(p2);

        Panel btnp = new Panel();
        btnp.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
        btn = new Button("Ok");
        btn.addActionListener(this);
        btnp.add(btn);
        btn = new Button("Cancel");
        btn.addActionListener(this);
        btnp.add(btn);

        add("Center", p);
        add("South", btnp);
    }
}
