import java.util.*;

import sv.kernel.*;

/** 
 * MergeFilterHandler.java
 * Merge Filter Handler
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class MergeFilterHandler {
  // if set true, merge mode (1d)
  public Filter mergeFilter;
  public boolean mergeMode = false;
  public int mergeIndex = 1;

  // if set true, merge mode (2d)
  public Filter mergeFilter2D;
  public boolean mergeMode2D = false;
  public int mergeIndex2D = 1;

  svserver svServer;

  public MergeFilterHandler(svserver svServer) {
     this.svServer = svServer;
  }

  /**
   * merge several GFPlane
   */
  public void mergeFrames() {

     long newFrameid = 0;

     // first check whether all frames have same structure
     GFPlane firstFrame = 
          (GFPlane)svServer.fmanager.getGFrame(((Long)mergeFilter.getSourcesAt(0)).longValue());

     if (firstFrame == null) {
        svServer.message("Merge two GraphFrames is failed ...", true);
        firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
        return;
     } else {
        firstFrame.mergeSelected = false;
        firstFrame.canvas.repaint();
     }

     for (int i=1; i<mergeIndex-1; i++) {
       GFPlane secondFrame = 
          (GFPlane)svServer.fmanager.getGFrame(((Long)mergeFilter.getSourcesAt(i)).longValue());
        
       if (secondFrame == null) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       } else {
         secondFrame.mergeSelected = false;
         secondFrame.canvas.repaint();
       }

       if (firstFrame.noOfTime != secondFrame.noOfTime) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       }

       if (firstFrame.dataSetVector.size() != 
                                    secondFrame.dataSetVector.size()) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       }
     }
  
     // perform merge action
     Vector dataVec = new Vector();
     int sz = firstFrame.dataSetVector.size();
     DSPlane newData = null;

     // save mergeFilter's data vector
     Vector savedV = new Vector();
     for (int j=0; j<mergeIndex-1; j++) {
        savedV.addElement(mergeFilter.getSourcesAt(j));
     }

     for (int i=0; i<sz; i++) {
        Vector vec = new Vector();

        for (int j=0; j<mergeIndex-1; j++) {
           GFPlane frame = 
              (GFPlane)svServer.fmanager.getGFrame(((Long)savedV.elementAt(j)).longValue());

           frame.mergeSelected = false;
           DSPlane dset = (DSPlane)frame.dataSetVector.elementAt(i);
           vec.addElement(dset);
        }
        
        mergeFilter.assignDataSets(vec);
        newData = (DSPlane)mergeFilter.performFilter();
       
        if (newData != null) 
           dataVec.addElement(newData);
        else {
           svServer.message("Merge failed ...", true);
          return;
        }
     }

     String title = newData.getTitle();
     for (int i=0; i<dataVec.size(); i++) {
       if (i == 0) {
          newFrameid = 
       svServer.fmanager.register((DSPlane)dataVec.elementAt(0), -1, null);
          svServer.cmanager.broadcastDataSet((DSPlane)dataVec.elementAt(0), newFrameid);
       } else {
          GFPlane otherFrame = 
               (GFPlane)svServer.fmanager.getGFrame(newFrameid);
          otherFrame.addDataSet((DSPlane)dataVec.elementAt(i));
          svServer.cmanager.broadcastDataSet((DSPlane)dataVec.elementAt(i), newFrameid);
       }
     }
     mergeIndex = 1;
  }



  /**
   * merge several GFrame2D
   */
  public void mergeFrames2D() {

     long newFrameid = 0;

     // first check whether all frames have same structure
     GFrame2D firstFrame = 
        (GFrame2D)svServer.fmanager.getGFrame(((Long)mergeFilter2D.getSourcesAt(0)).longValue());
     if (firstFrame == null) {
        svServer.message("Merge two GraphFrames is failed ...", true);
        firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
        return;
     } else {
        firstFrame.mergeSelected = false;
        firstFrame.canvas.destroyImage();
        firstFrame.canvas.repaint();
     }

     for (int i=1; i<mergeIndex2D-1; i++) {
        GFrame2D secondFrame = 
           (GFrame2D)svServer.fmanager.getGFrame(((Long)mergeFilter2D.getSourcesAt(i)).longValue());
        
       if (secondFrame == null) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       } else {
         secondFrame.mergeSelected = false;
         secondFrame.canvas.destroyImage();
         secondFrame.canvas.repaint();
       }

       if (firstFrame.noOfTime != secondFrame.noOfTime) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       }

       if (firstFrame.dataSetVector.size() != 
                                    secondFrame.dataSetVector.size()) {
         svServer.message("Merge two GraphFrames is failed ...", true);
         firstFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         secondFrame.monitor.display("Merge two GraphFrames is failed ...", true);
         return;
       }
     }
  
     // perform merge action
     Vector dataVec = new Vector();
     int sz = firstFrame.dataSetVector.size();
     DataSet2D newData = null;

     // save mergeFilter's data vector
     Vector savedV = new Vector();
     for (int j=0; j<mergeIndex2D-1; j++) {
        savedV.addElement(mergeFilter2D.getSourcesAt(j));
     }

     for (int i=0; i<sz; i++) {
        Vector vec = new Vector();

        for (int j=0; j<mergeIndex2D-1; j++) {
           GFrame2D frame = 
                     (GFrame2D)svServer.fmanager.getGFrame(((Long)savedV.elementAt(j)).longValue());

           frame.mergeSelected = false;
           DataSet2D dset = (DataSet2D)frame.dataSetVector.elementAt(i);
           vec.addElement(dset);
        }

        mergeFilter2D.assignDataSets(vec);
        newData = (DataSet2D)mergeFilter2D.performFilter();
       
        if (newData != null) 
           dataVec.addElement(newData);
        else {
           svServer.message("Merge failed ...", true);
          return;
        }
     }

     String title = newData.getTitle();
     for (int i=0; i<dataVec.size(); i++) {
       if (i == 0) {
          newFrameid = 
    svServer.fmanager.register((DataSet2D)dataVec.elementAt(0), -1, null);
          svServer.cmanager.broadcastDataSet((DataSet2D)dataVec.elementAt(0), newFrameid);
       } else {
          GFrame2D otherFrame = 
                 (GFrame2D)svServer.fmanager.getGFrame(newFrameid);
          otherFrame.addDataSet((DataSet2D)dataVec.elementAt(i));
          svServer.cmanager.broadcastDataSet((DataSet2D)dataVec.elementAt(i), newFrameid);
       }
     }
     mergeIndex2D = 1;

  }

} // end of class
