import java.net.*;
import java.io.*;
import java.util.*;

import sv.kernel.*;
import sv.util.*;

/** 
 * ServerManager.java
 * work after getting connection from client
 * @version 1.0    10/15/97
 * @author Byeongseob Ki
 */

class ServerManager extends Thread {
   private Socket sock;
   private DataOutputStream out;
   private DataInputStream in;

   private svserver parent;

   /**
    * constructor
    * @param s server socket
    */
   public ServerManager(Socket s, svserver parent) {
      sock = s;
      this.parent = parent;
   }

   /**
    * read commands from the client and do work
    */
   public void run() {
      try {
	out = new DataOutputStream(sock.getOutputStream());
	in  = new DataInputStream(new BufferedInputStream(sock.getInputStream()));

        String hostname = sock.getInetAddress().getHostName();
        parent.message("Connection from: " + hostname, false);
 
        int command = in.readInt();

        /////////////////////////////////////////////////////////////////
        // Now we have to decide what commands and data are received 
        // from the client. Normally, after receiving data, connection will
        // be disconnected. 

        doServerWork(hostname, command); // do the server work
        sock.close();                   // close the socket

        //////////////////////////////////////////////////////////////////

      } catch (Exception e) {
	e.printStackTrace();
      }
      stop();
   }


   //////////////////////////////////////////////////////////////
   // data reading from ".svb" file
   private void doCommand_1(String hostname) {
      SVFileReader.readFromInput(parent, in, 2);  
   }

   //////////////////////////////////////////////////////////////
   // collaboration request
   private void doCommand_2(String hostname) {
     int port;
     try {
       port = in.readInt();
     } catch (Exception e) {
         System.out.println("Data receiving error(2).");
         return;
     }
     parent.cmanager.peerCollaborationRequested(hostname, port);
   }

   //////////////////////////////////////////////////////////////
   // reqeusted exit command from other collaborator
   private void doCommand_3(String hostname) {
      
      parent.cmanager.receivedDisconnectRequest(hostname, getShortName(hostname));
   }

   //////////////////////////////////////////////////////////////
   // received a message that collaboration is accepted
   private void doCommand_4(String hostname) {
      parent.cmanager.connectionEstablished(in, hostname);
   }

   //////////////////////////////////////////////////////////////
   // received a message that collaboration is refused
   private void doCommand_5(String hostname) {
      parent.message("Connection refused from " + hostname + " ...", true);
   }

   //////////////////////////////////////////////////////////////
   // eqeusted adding new collaborator from other server
   private void doCommand_6(String hostname) {
     String server = null;
     Integer port = null;

     try {     
       server = in.readUTF();
       port = new Integer(in.readInt());
     } catch (Exception e) {
        System.out.println("Data receiving error(6).");
        return;
     }

     parent.cmanager.addNewCollaborator(server, port, getShortName(hostname));
   }

   //////////////////////////////////////////////////////////////
   // received broadcasted DataSet
   private void doCommand_7(String hostname) {
     int dim;
     try {   
        dim = in.readInt(); 
        long frameID = in.readLong();
        String title = in.readUTF();
        int noOfTimeData;

        if (dim == DataSet.ONED) {      
            DSPlane dataSet = null;
            int dataType = in.readInt();
            noOfTimeData = in.readInt();

            for (int k=0; k<noOfTimeData; k++) {
               TDPlane readed = null;

               if (dataType == DSPlane.NORMAL)
                  readed = SerialMgr.parse1dStream(in);             
               else if (dataType == DSPlane.VECTOR)
                  readed = SerialMgr.parsevecStream(in);     
               else if (dataType == DSPlane.CONNECTIVITY)
                  readed = SerialMgr.parseconStream(in);     

               if (k == 0) {
                   if (dataType == DSPlane.NORMAL)
                      dataSet = new DataSet1D(title, (TimeData1D)readed);
                   else if (dataType == DSPlane.VECTOR)
                      dataSet = new DataSetVec(title, (TimeDataVec)readed);
                    else if (dataType == DSPlane.CONNECTIVITY)
                      dataSet = new DataSetCon(title, (TimeDataCon)readed);
               } else 
                   dataSet.addTimeData(readed);

            }
            parent.fmanager.receivedDataSet(dataSet, null, frameID, false);

        } else if (dim == DataSet.TWOD) {      
            DataSet2D dataSet = null;
            noOfTimeData = in.readInt();

            for (int k=0; k<noOfTimeData; k++) {
                TimeData2D readed = SerialMgr.parse2dStream(in);
 
                if (k == 0) 
                  dataSet = new DataSet2D(title, readed);
                else 
                  dataSet.addTimeData(readed);
            }
            parent.fmanager.receivedDataSet(dataSet, null, frameID, false);

        } else if (dim == DataSet.THREED) {      
            DataSet3D dataSet = null;

            String t_hn = null;
            int t_port = -1;
            int t_ver = -1;

            if (in.readInt() == 1) {
              t_hn = in.readUTF();
              t_port = in.readInt();
              t_ver = in.readInt();
            }
            
            noOfTimeData = in.readInt();
            for (int k=0; k<noOfTimeData; k++) {
                TimeData3D readed = SerialMgr.parse3dStream(in);
                if (k == 0) 
                  dataSet = new DataSet3D(title, t_ver, t_hn, t_port, readed);
                else 
                  dataSet.addTimeData(readed);
            }
            parent.fmanager.receivedDataSet(dataSet, null, frameID, false);
        } 

        if (parent.cmanager.conHandler != null) 
           parent.cmanager.conHandler.addEventInfo("<" + getShortName(hostname) + "> Received data set " + title + ".");
             
     } catch (Exception e) {
        System.out.println("Data receiving error(7).");
e.printStackTrace();
        return;
     }
   }

   //////////////////////////////////////////////////////////////
   // received broadcased GFrame
   private void doCommand_8(String hostname) {
      parent.cmanager.receiveGFrame(in);  
   }

   //////////////////////////////////////////////////////////////
   // broadcasted chat string
   private void doCommand_9(String hostname) {
     try {     
       String str = in.readUTF();
       if (parent.cmanager.conHandler != null)
          parent.cmanager.conHandler.addEventInfo("<" + 
                               getShortName(hostname) + "> " + str);
       
      } catch (Exception e) {
         System.out.println("Data receiving error(9).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // for java_ser
   private void doCommand_10(String hostname) {
      parent.message("Data from java_ser codes...", false);
      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();

         float[] xArr = new float[nx];
         float[] yArr = new float[nx];

         for (int j=0; j<nx; j++) {
            xArr[j] = in.readFloat();
            yArr[j] = in.readFloat();
         }
         dataset = new DataSet1D(title, new TimeData1D(time, nx, xArr, yArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(12).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // reading Filter info
   private void doCommand_11(String hostname) {
      parent.message("Reading filter...", false);
      String input = null;

      try {
        long id = in.readLong();      
        int dim = in.readInt();       

        int dummy = in.readInt();  
        if (dummy == 1) 
           input = in.readUTF();   

        String cname = in.readUTF();

        dummy = in.readInt();  
        String[] idArr = new String[dummy];
        for (int j=0; j<dummy; j++)
           idArr[j] = in.readUTF();


        int size  = in.readInt();  
        byte[] result = new byte[size];
        in.read(result, 0, size);  

        if (result == null) {
          System.out.println("ClassNotFoundException ...");
          return;
        }
     
        try {
           FilterClassLoader loader = new FilterClassLoader(result);
           Class filter = loader.loadClass(cname, true);
           if (filter == null) {
              System.out.println("ClassFormatError ...");
              return;
           }
           System.out.println("Input for the filter: " + input);
           parent.fmanager.actionFilter(id, dim, idArr,
                               (Filter)filter.newInstance(), input, false); 

        } catch (Exception e) {
           System.out.println("Filter conversion error(11).");
        }

      } catch (Exception e) {
         System.out.println("Data receiving error(11).");
         return;
      }    

     if (parent.cmanager.conHandler != null) 
        parent.cmanager.conHandler.addEventInfo("<" + getShortName(hostname) + "> Received command, perform filter.");
      
   }

   //////////////////////////////////////////////////////////////
   // from java_mser
   private void doCommand_12(String hostname) {
      parent.message("Data from java_mser codes...", false);
      Vector vec = new Vector();

      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();
         for (int i=0; i<nt; i++) {
            float time = in.readFloat();
            int nx = in.readInt();

            float[] xArr = new float[nx];
            float[] yArr = new float[nx];

            for (int j=0; j<nx; j++) {
               xArr[j] = in.readFloat();
               yArr[j] = in.readFloat();
            }

            vec.addElement(new TimeData1D(time, nx, xArr, yArr));
         }

         dataset = new DataSet1D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(12).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // data is received from the isoserver
   private void doCommand_13(String hostname) {
      int size;

      try {
         int port = in.readInt();
         size = in.readInt();
         byte[] titlechs = new byte[size];
         in.read(titlechs, 0, size);
         String title = new String(titlechs);

         int version = in.readInt();   // data_version
         float time = in.readFloat();  // time value
	 //System.out.println("received time: " + time);
         int dimx = in.readInt();  
         int dimy = in.readInt();  
         int dimz = in.readInt();  

         float threshold = in.readFloat();
         float zmin = in.readFloat();
         float zmax = in.readFloat();

	 //System.out.println("dimension: " + dimx + " " + dimy + " " 
	 //     + dimz + "\n" + "min and max: " + zmin + " " + zmax);

         int vsize = in.readInt();     // read vertex size
         float[][] vertex = new float[vsize][3];

         for (int i=0; i<vsize; i++) 
            for (int j=0; j<3; j++) 
               vertex[i][j] = in.readFloat();

         int tsize = in.readInt();     // read triangle
         if ((tsize%3) != 0) {
            System.out.println("Error while receiving isosurface data..");
            return;
         }

         tsize = (int)(tsize / 3);
         int[][] triangle = new int[tsize][3];
         
         for (int i=0; i<tsize; i++) 
            for (int j=0; j<3; j++) 
               triangle[i][j] = in.readInt();


         DataSet3D iso = new DataSet3D(title, version, hostname, port,  
                      new TimeData3D(time, vsize, vertex, tsize, triangle,
                                    dimx, dimy, dimz, threshold, zmin, zmax));
         if (iso == null) 
              return;

         parent.fmanager.receivedDataSet(iso, null, -1, true);

     } catch (Exception e) {
       System.out.println("Isosurface data reading error ...");
       return;
     }
   }

   //////////////////////////////////////////////////////////////
   // remove data set command
   private void doCommand_14(String hostname) {
      try {
         int dim = in.readInt();
         long frameID = in.readLong();
         String title = in.readUTF();

         CollabManager manager = parent.cmanager;
         if (dim == 1) {
             GFPlane gframe = (GFPlane)parent.fmanager.getGFrame(frameID);
             if (gframe != null) {
                parent.message("Got remove data set " + title + " command...", 
                                                                     false);
                gframe.removeDataSet(title);
             }
             if (manager.conHandler != null) 
                manager.conHandler.addEventInfo("<" + getShortName(hostname) + "> Remove data set " + title);
         } else if (dim == 2) {
              GFrame2D gframe = (GFrame2D)parent.fmanager.getGFrame(frameID);
             if (gframe != null) {
                parent.message("Got remove data set " + title + " command...", 
                                                                     false);
                gframe.removeDataSet(title);
              }
              if (manager.conHandler != null) 
                manager.conHandler.addEventInfo("<" + getShortName(hostname) + "> Remove data set " + title);
	 }
      } catch (Exception e) {
         System.out.println("Data receiving error(14).");
         return;
      }
   }

   //////////////////////////////////////////////////////////////
   // from java_ser2d 
   private void doCommand_15(String hostname) {
       parent.message("Data from java_ser2d codes...", false);
       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();

         int nx = in.readInt();
         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] zArr = new float[nx];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             zArr[j] = in.readFloat();
          }
          dataset = 
             new DataSet2D(title, new TimeData2D(time, nx, xArr, yArr, zArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(15).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_mser2d 
   private void doCommand_16(String hostname) {
       parent.message("Data from java_mser2d codes...", false);
       Vector vec = new Vector();

       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             float[] xArr = new float[nx];
             float[] yArr = new float[nx];
             float[] zArr = new float[nx];

             for (int j=0; j<nx; j++) {
                xArr[j] = in.readFloat();
                yArr[j] = in.readFloat();
                zArr[j] = in.readFloat();
             }

             vec.addElement(new TimeData2D(time, nx, xArr, yArr, zArr));
          }

          dataset = new DataSet2D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(16).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_pser
   private void doCommand_17(String hostname) {
      
      parent.message("Data from java_pser codes...", false);
      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         int processor = in.readInt();

         float[] xArr = new float[nx];
         float[] yArr = new float[nx];

         for (int j=0; j<nx; j++) {
            xArr[j] = in.readFloat();
            yArr[j] = in.readFloat();
         }
         dataset = new DataSet1D(title, new TimeData1D(time, nx, xArr, yArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(17).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, 0, true);
   }
    
   //////////////////////////////////////////////////////////////
   // from java_pser2d 
   private void doCommand_18(String hostname) {
       parent.message("Data from java_pser2d codes...", false);
       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         int processors = in.readInt();

         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] zArr = new float[nx];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             zArr[j] = in.readFloat();
          }
          dataset = 
             new DataSet2D(title, new TimeData2D(time, nx, xArr, yArr, zArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(15).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_sercon
   private void doCommand_19(String hostname) {
       parent.message("Data from java_sercon codes...", false);
       DataSetCon dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();

         int nx = in.readInt();
         int nc = in.readInt();
         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] cArr = new float[nx];
         int[] con = new int[nc];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             cArr[j] = in.readFloat();
          }

         for (int j=0; j<nc; j++) 
             con[j] = in.readInt();

          dataset = 
          new DataSetCon(title, new TimeDataCon(time, xArr, yArr, cArr, con)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(19).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_cmser
   private void doCommand_20(String hostname) {
      parent.message("Data from java_cmser codes...", false);
      Vector vec = new Vector();

      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();
         for (int i=0; i<nt; i++) {
            float time = in.readFloat();
            int nx = in.readInt();

            float[] xArr = new float[nx];
            float[] yArr = new float[nx];
            float[] cArr = new float[nx];

            for (int j=0; j<nx; j++) {
               xArr[j] = in.readFloat();
               yArr[j] = in.readFloat();
               cArr[j] = in.readFloat();
            }

            vec.addElement(new TimeData1D(time, nx, xArr, yArr, cArr));
         }

         dataset = new DataSet1D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(20).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // received broadcasted command
   private void doCommand_21(String hostname) {
     try{
        String[] args = null;
        boolean null_args = true;
        int dim = in.readInt();
        int command = in.readInt();
        int size = in.readInt();
    
        if (size > 0)  {
           null_args = false;
           args = new String[size];
           for (int i=0; i<size; i++) 
              args[i] = in.readUTF();
        }

        long frameID = in.readLong();       

        FrameManager manager = parent.fmanager;
        Object gframe = manager.getGFrame(frameID);
        if (gframe == null) return;

        if (dim == DataSet.ONED) 
           manager.action1d((GFPlane)gframe, command, args, false);
        else if (dim == DataSet.TWOD) 
           manager.action2d((GFrame2D)gframe, command, args, false);
        else if (dim == DataSet.THREED) 
           manager.action3d((GFrame3D)gframe, command, args, false);
        else if (dim < 0)
           manager.actionGlobal(command, args, false); 

        if (parent.cmanager.conHandler != null) 
           parent.cmanager.conHandler.addEventInfo("<" + 
                 getShortName(hostname) + "> Received command: " + 
                 commandInfo(dim, command));

     } catch (Exception e) {
         System.out.println("Data receiving error(21).");
         return;
     }
   }

   //////////////////////////////////////////////////////////////
   // from java_vser
   private void doCommand_22(String hostname) {
       parent.message("Data from java_vser codes...", false);
       DataSetVec dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();

         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] rArr = new float[nx];
         float[] tArr = new float[nx];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             rArr[j] = in.readFloat();
             tArr[j] = in.readFloat();
          }
          dataset = 
        new DataSetVec(title, new TimeDataVec(time, xArr, yArr, rArr, tArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(22).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // fro java_sercon3d
   private void doCommand_23(String hostname) {

       parent.message("Data from java_sercon3d codes...", false);
       DataSet3D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();

         int nx = in.readInt();
         int nc = in.readInt();
         float[] xArr = new float[nx];
         float[] yArr = new float[nx];
         float[] zArr = new float[nx];
         float[] cArr = new float[nx];
         int[] con = new int[nc];

         for (int j=0; j<nx; j++) {
             xArr[j] = in.readFloat();
             yArr[j] = in.readFloat();
             zArr[j] = in.readFloat();
             cArr[j] = in.readFloat();
          }

         for (int j=0; j<nc; j++) 
             con[j] = in.readInt();

         dataset = 
          new DataSet3D(title, 
               new TimeData3D(time, nx, xArr, yArr, zArr, cArr, nc, con)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(23).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }
    
   //////////////////////////////////////////////////////////////
   // not used
   private void doCommand_24(String hostname) {
   }

   //////////////////////////////////////////////////////////////
   // from java_bbser2d
   private void doCommand_25(String hostname) {
       parent.message("Data from java_bbser2d codes...", false);
       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         int ny = in.readInt();

         float xmin = in.readFloat();
         float xmax = in.readFloat();
         float ymin = in.readFloat();
         float ymax = in.readFloat();

         size = nx * ny;
         float[] zArr = new float[size];

         for (int j=0; j<size; j++) 
             zArr[j] = in.readFloat();

         dataset = 
             new DataSet2D(title, new TimeData2D(time, xmin, xmax, nx, 
                                          ymin, ymax, ny, zArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(25).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_bbmser2d
   private void doCommand_26(String hostname) {
       parent.message("Data from java_bbmser2d codes...", false);
       Vector vec = new Vector();

       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             int ny = in.readInt();

             float xmin = in.readFloat();
             float xmax = in.readFloat();
             float ymin = in.readFloat();
             float ymax = in.readFloat();

             size = nx * ny;
             float[] zArr = new float[size];

             for (int j=0; j<size; j++) {
                zArr[j] = in.readFloat();
             }

             vec.addElement(new TimeData2D(time, 
                       xmin, xmax, nx, ymin, ymax, ny, zArr));
          }

          dataset = new DataSet2D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(26).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_vmser
   private void doCommand_27(String hostname) {
       parent.message("Data from java_vmser codes...", false);
       Vector vec = new Vector();

       DataSetVec dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             float[] xArr = new float[nx];
             float[] yArr = new float[nx];
             float[] rArr = new float[nx];
             float[] tArr = new float[nx];

             for (int j=0; j<nx; j++) {
                xArr[j] = in.readFloat();
                yArr[j] = in.readFloat();
                rArr[j] = in.readFloat();
                tArr[j] = in.readFloat();
             }

             vec.addElement(new TimeDataVec(time,
                                          xArr, yArr, rArr, tArr));
          }

          dataset = new DataSetVec(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(27).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_bbser
   private void doCommand_28(String hostname) {
      parent.message("Data from java_bbser codes...", false);
      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         
         float xmin = in.readFloat();
         float xmax = in.readFloat();

         float[] yArr = new float[nx];

         for (int j=0; j<nx; j++) 
            yArr[j] = in.readFloat();

         dataset = new DataSet1D(title, new TimeData1D(time, nx, xmin, xmax, yArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(28).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_bbmser
   private void doCommand_29(String hostname) {
      parent.message("Data from java_bbmser codes...", false);
      Vector vec = new Vector();

      DataSet1D dataset;      

      try {
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();
         for (int i=0; i<nt; i++) {
            float time = in.readFloat();
            size = in.readInt();

            float xmin = in.readFloat();
            float xmax = in.readFloat();

            float[] yArr = new float[size];

            for (int j=0; j<size; j++) 
               yArr[j] = in.readFloat();


            vec.addElement(new TimeData1D(time, size, xmin, xmax, yArr));
         }

         dataset = new DataSet1D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(29).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from from java_bser2d
   private void doCommand_30(String hostname) {
       parent.message("Data from java_bser2d codes...", false);
       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         float[] xArr = new float[nx];
         for (int i=0; i<nx; i++) 
            xArr[i] = in.readFloat();

         int ny = in.readInt();
         float[] yArr = new float[ny];
         for (int i=0; i<ny; i++) 
            yArr[i] = in.readFloat();

         int nz = in.readInt();
         float[] zArr = new float[nz];
         for (int j=0; j<nz; j++) 
             zArr[j] = in.readFloat();

          dataset = 
             new DataSet2D(title, new TimeData2D(time, xArr, nx, yArr, ny, zArr));
      } catch (Exception e) {
         System.out.println("Data receiving error(30).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from from java_bmser2d
   private void doCommand_31(String hostname) {
       parent.message("Data from java_bmser2d codes...", false);
       Vector vec = new Vector();

       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             float[] xArr = new float[nx];
             for (int j=0; j<nx; j++) 
                xArr[j] = in.readFloat();

             int ny = in.readInt();
             float[] yArr = new float[ny];
             for (int j=0; j<ny; j++) 
                yArr[j] = in.readFloat();

             int nz = nx * ny;
             float[] zArr = new float[nz];
             for (int j=0; j<nz; j++) 
                zArr[j] = in.readFloat();

             vec.addElement(new TimeData2D(time, xArr, nx, yArr, ny, zArr));
          }

          dataset = new DataSet2D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(31).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from java_pbbser2d
   private void doCommand_32(String hostname) {
       parent.message("Data from java_pbbser2d codes...", false);
       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         float time = in.readFloat();
         int nx = in.readInt();
         int ny = in.readInt();

         float xmin = in.readFloat();
         float xmax = in.readFloat();
         float ymin = in.readFloat();
         float ymax = in.readFloat();
         int processors = in.readInt();

         size = nx * ny;
         float[] zArr = new float[size];

         for (int j=0; j<size; j++) 
             zArr[j] = in.readFloat();

          dataset = 
             new DataSet2D(title, new TimeData2D(time, xmin, xmax, nx, 
                                          ymin, ymax, ny, zArr)); 
      } catch (Exception e) {
         System.out.println("Data receiving error(32).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

   //////////////////////////////////////////////////////////////
   // from // from java_cbbmser2d
   private void doCommand_33(String hostname) {
       parent.message("Data from java_cbbmser2d codes...", false);
       Vector vec = new Vector();

       DataSet2D dataset; 

       try {  
         int size = in.readInt();

         byte[] titlechars = new byte[size];
         in.read(titlechars, 0, size);
         String title = new String(titlechars);

         int nt = in.readInt();

         for (int i=0; i<nt; i++) {
             float time = in.readFloat();

             int nx = in.readInt();
             int ny = in.readInt();

             float xmin = in.readFloat();
             float xmax = in.readFloat();
             float ymin = in.readFloat();
             float ymax = in.readFloat();

             size = nx * ny;
             float[] zArr = new float[size];
             float[] cArr = new float[size];

             for (int j=0; j<size; j++) {
                zArr[j] = in.readFloat();
                cArr[j] = in.readFloat();
             }
          
             vec.addElement(new TimeData2D(time, 
                       xmin, xmax, nx, ymin, ymax, ny, zArr, cArr));
          }

          dataset = new DataSet2D(title, vec); 
      } catch (Exception e) {
         System.out.println("Data receiving error(33).");
         return;
      }
      parent.fmanager.receivedDataSet(dataset, null, -1, true);
   }

  // return short hostname 
  private String getShortName(String str) {
     String name = " ";
     int index = str.indexOf(".");
     if (index > 0)
         name = str.substring(0, index);
     else
        name = str;
     return name;
  }

  // return command information
  private String commandInfo(int dim, int command) {
     return "";
  }


   // real worker
   private synchronized void doServerWork(String hostname, int command) {
     switch(command) {
        case 1:  doCommand_1(hostname);   break;
        case 2:  doCommand_2(hostname);   break;
        case 3:  doCommand_3(hostname);   break;
        case 4:  doCommand_4(hostname);   break;
        case 5:  doCommand_5(hostname);   break;
        case 6:  doCommand_6(hostname);   break;
        case 7:  doCommand_7(hostname);   break;
        case 8:  doCommand_8(hostname);   break;
        case 9:  doCommand_9(hostname);   break;
        case 10: doCommand_10(hostname);  break;
        case 11: doCommand_11(hostname);  break;
        case 12: doCommand_12(hostname);  break;
        case 13: doCommand_13(hostname);  break;
        case 14: doCommand_14(hostname);  break;
        case 15: doCommand_15(hostname);  break;
        case 16: doCommand_16(hostname);  break;
        case 17: doCommand_17(hostname);  break;
        case 18: doCommand_18(hostname);  break;
        case 19: doCommand_19(hostname);  break;
        case 20: doCommand_20(hostname);  break;
        case 21: doCommand_21(hostname);  break;
        case 22: doCommand_22(hostname);  break;
        case 23: doCommand_23(hostname);  break;
        case 24: doCommand_24(hostname);  break;
        case 25: doCommand_25(hostname);  break;
        case 26: doCommand_26(hostname);  break;
        case 27: doCommand_27(hostname);  break;
        case 28: doCommand_28(hostname);  break;
        case 29: doCommand_29(hostname);  break;
        case 30: doCommand_30(hostname);  break;
        case 31: doCommand_31(hostname);  break;
        case 32: doCommand_32(hostname);  break;
        case 33: doCommand_33(hostname);  break;
        default:
          System.out.println("Got undefined command ... " + command);
     }
   }


} // end of class









