package sv.drawer;

/** 
 * BLineShape.java
 * Bezier line 
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

import sv.ui.SVUIC;
import sv.util.NewtonMethod;

public class BLineShape extends MLineShape implements Observer {
   NewtonMethod spline;

   public BLineShape(int type, int x1, int y1, Color c, int t, int w, int s) {
      super(type, x1, y1, c, t, w, s);
   }

   public void paint(Graphics g) {
     if (_ptsvec.size() < 2) return;
      
     if (_ptsvec.size()  ==  2) {
        super.paint(g);
        return;
     }
    
     ///////////////////////////////////////////////////////////
     // if the number of points are larger than 2
     ///////////////////////////////////////////////////////////
     int size =  _ptsvec.size();
     Point[] pts = new Point[size];
     Point[] arrow;

     _ptsvec.copyInto(pts);

     g.setColor(linec);

     int togo = size - 2;  
     int togo2 = size - 1; 

     float[] xcontrols = new float[size];
     float[] ycontrols = new float[size];

     for (int i=0; i<size; i++) {
        Point p = (Point)_ptsvec.elementAt(i);
        xcontrols[i] = (float)p.x;
        ycontrols[i] = (float)p.y;
     }

     spline = new NewtonMethod(xcontrols, ycontrols);

     switch (line_type) {
        case LineShape.NOARROW_LINE:
        case LineShape.NOARROW_DLINE:
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          break;

        case LineShape.RIGHTFILLARROW_LINE:
        case LineShape.RIGHTFILLARROW_DLINE:
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          break;

        case LineShape.RIGHTARROW_LINE:
        case LineShape.RIGHTARROW_DLINE:
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          break;

        case LineShape.LEFTFILLARROW_LINE:
        case LineShape.LEFTFILLARROW_DLINE:
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          break;

        case LineShape.LEFTARROW_LINE:
        case LineShape.LEFTARROW_DLINE:
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          break;

        case LineShape.LRFILLARROW_LINE:
        case LineShape.LRFILLARROW_DLINE:
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          break;

        case LineShape.LRARROW_LINE:
        case LineShape.LRARROW_DLINE:
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          for (int i=0; i<togo2; i++) 
             drawStep(g, pts[i], pts[i+1]);
          break;
     }
   }


   private void drawStep(Graphics g, Point p1, Point p2) {
      float[] xvals = new float[10];
      float[] yvals = new float[10];

      float dx = (float)(p2.x - p1.x);
      float gap = dx / 9.0f;

      xvals[0] = (float)p1.x;
      yvals[0] = (float)p1.y;
      xvals[9] = (float)p2.x;
      yvals[9] = (float)p2.y;

      float fval = xvals[0];
      for (int j=1; j<9; j++) {
         fval +=gap;
         xvals[j] = fval;
         yvals[j] = spline.computePoint(xvals[j]);               
      } 

      switch (line_type) {
         case LineShape.NOARROW_LINE:
         case LineShape.RIGHTFILLARROW_LINE:
         case LineShape.RIGHTARROW_LINE:
         case LineShape.LEFTFILLARROW_LINE:
         case LineShape.LEFTARROW_LINE:
         case LineShape.LRFILLARROW_LINE:
         case LineShape.LRARROW_LINE:

            for (int j=0; j<9; j++) 
              SVUIC.drawLine(g, (int)xvals[j], (int)yvals[j], 
                 (int)xvals[j+1], (int)yvals[j+1], line_width, linec);
            break;

         case LineShape.NOARROW_DLINE:
         case LineShape.RIGHTFILLARROW_DLINE:
         case LineShape.RIGHTARROW_DLINE:
         case LineShape.LEFTFILLARROW_DLINE:
         case LineShape.LEFTARROW_DLINE:
         case LineShape.LRFILLARROW_DLINE:
         case LineShape.LRARROW_DLINE:
             for (int j=0; j<9; j++) 
                SVUIC.drawDottedLine(g, (int)xvals[j], (int)yvals[j],
             (int)xvals[j+1], (int)yvals[j+1], line_width, dash_size, linec);
             break;
      }
   }


} // end of class





