package sv.drawer;

import java.awt.*;
import java.util.*;

/** 
 * DrawHandle.java
 * Drawing object handler
 * @version 1.0    11/15/97
 * @author Byeongseob Ki
 */


public class DrawHandle {

  // drawing objects in a given time
  public Vector drawingVec;          

  // hotspot manager       
  public HotSpotMgr hotspotManager;

  // time value
  private float time;

  // original window size
  private Dimension winSize;

  /**
   * constructor
   * @param time time value of this handler
   */
  public DrawHandle(float time, Dimension sz) {
     drawingVec = new Vector();
     hotspotManager = new HotSpotMgr();
     this.time = time;
      
     if (sz.width < 100 || sz.height < 100)
        this.winSize = new Dimension(100, 100);
     else
        this.winSize = sz;
  }


  /**
   * constructor
   * @param str stringized draw handle object
   */
  public DrawHandle(String str) {
     drawingVec = new Vector();
     hotspotManager = new HotSpotMgr();

     StringTokenizer st = new StringTokenizer(str, "@");
     int count = st.countTokens();
     time = Float.valueOf(st.nextToken()).floatValue();
     int w = Integer.valueOf(st.nextToken()).intValue();
     int h = Integer.valueOf(st.nextToken()).intValue();
     winSize = new Dimension(w, h);
     
     for (int i=3; i<count; i++)
        createDrawingObject(st.nextToken());
  }


  /**
   * add object and hotspot
   */
  public void manage(ShapeObject s) {
      drawingVec.addElement(s);
      Vector vecShapes = s.getHotSpots();

      for (int i = 0; i < vecShapes.size(); i++) 
         hotspotManager.manage((HotSpot)vecShapes.elementAt(i));
  }

  /**
   * received notification that window size is changed
   */
  public void setWindowSize(Dimension d) {
     for (int i = 0; i < drawingVec.size(); i++) {
        ShapeObject s = (ShapeObject)drawingVec.elementAt(i);
        s.changeCoordinates((double)d.width/(double)winSize.width, 
                             (double)d.height/(double)winSize.height);
     }
     winSize = d;
  }


  // parse drawing object from a string and create drawing object
  public void createDrawingObject(String str) {
     int size;
     Point[] pts;

     StringTokenizer st = new StringTokenizer(str, ";");

     // first object is a type
     int type = Integer.valueOf(st.nextToken()).intValue();

     switch (type) {
        case ShapeProperties.LINE:
           manage(new LineShape(type, 
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue()));
           break;

        case ShapeProperties.MULTILINE:
        case ShapeProperties.BEZIER:
           size = Integer.valueOf(st.nextToken()).intValue();
           if (size < 1) return;

           pts = new Point[size];

           for (int i=0; i<size; i++) { 
             pts[i] = new Point(Integer.valueOf(st.nextToken()).intValue(),
                                Integer.valueOf(st.nextToken()).intValue());
           }

           if (type == ShapeProperties.MULTILINE) {
              MLineShape mls = new MLineShape(type, pts[0].x, pts[0].y,
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue());
              manage(mls);

              for (int i=1; i<size; i++) 
                mls.addPoint(pts[i]);

	   } else if (type == ShapeProperties.BEZIER) {

              BLineShape bls = new BLineShape(type, pts[0].x, pts[0].y,
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue());
              manage(bls);

              for (int i=1; i<size; i++) 
                bls.addPoint(pts[i]);
           }

           break;

        case ShapeProperties.MOUSE:
           size = Integer.valueOf(st.nextToken()).intValue();

           if (size > 0) {
              pts = new Point[size];

              for (int i=0; i<size; i++) { 
                pts[i] = new Point(Integer.valueOf(st.nextToken()).intValue(),
                                   Integer.valueOf(st.nextToken()).intValue());
              }
           } else
              pts = null;

           RLineShape rls = new RLineShape(type, 
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue());
           manage(rls);

           rls.addPoints(pts);
           break;

        case ShapeProperties.RECTANGLE:
          manage(new RectangleShape(type,
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    st.nextToken()));
          break;
        case ShapeProperties.POLYGON:
           size = Integer.valueOf(st.nextToken()).intValue();

           if (size > 0) {
              pts = new Point[size];

              for (int i=0; i<size; i++) { 
                pts[i] = new Point(Integer.valueOf(st.nextToken()).intValue(),
                                   Integer.valueOf(st.nextToken()).intValue());
              }
           } else
              pts = null;

           PolygonShape poly = new PolygonShape(type, 
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    new Color(Integer.valueOf(st.nextToken()).intValue()),
                    st.nextToken());
           manage(poly);

           poly.addPoints(pts);

          break;
        case ShapeProperties.OVAL:
          manage(new OvalShape(type,
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    st.nextToken()));
          break;
        case ShapeProperties.TEXT:
              int x = Integer.valueOf(st.nextToken()).intValue();
              int y = Integer.valueOf(st.nextToken()).intValue();
              int c = Integer.valueOf(st.nextToken()).intValue();
              String name = st.nextToken();
              int style = Integer.valueOf(st.nextToken()).intValue();
              size = Integer.valueOf(st.nextToken()).intValue();
              Font f = new Font(name, style, size);
              manage(new TextShape(type, x, y, c, st.nextToken(), f));
              break;
        case ShapeProperties.IMAGE:
              break;
     }
  }


  // create drawing object(IMAGE type only)
  public void createDrawingObject(String str, Image img) {
     StringTokenizer st = new StringTokenizer(str, ";");

     ImageShape shape = 
         new ImageShape(Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue(),
                    Integer.valueOf(st.nextToken()).intValue());
     manage(shape);
     shape.setImage(img);
  }

  /**
   * return time
   */
  public float getTime() {
     return time;
  }


  /**
   * return original window size
   */
  public Dimension getOrigWinSize() {
     return winSize;
  }
 
}
