package sv.drawer;

/** 
 * ImageShape.java
 * Define image object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

public class ImageShape extends ShapeObject implements Observer {
   HotSpot _hsTL, _hsTR, _hsBL, _hsBR, _hsC;

   int _x, _y, _w, _h;

   Image img = null;

   public ImageShape(int type, int x, int y, int w, int h) {
      this.type = type;
      _x = x;
      _y = y;
      _w = w;
      _h = h;

      _hsTL = new HotSpot(new Point(x, y));
      _hsTR = new HotSpot(new Point(x + w, y));
      _hsBL = new HotSpot(new Point(x, y + h));
      _hsBR = new HotSpot(new Point(x + w, y + h));
      _hsC = new HotSpot(new Point(x + w/2, y + h/2), new Dimension(15, 15));

      _hsTL.addObserver(this);
      _hsTR.addObserver(this);
      _hsBL.addObserver(this);
      _hsBR.addObserver(this);
      _hsC.addObserver(this);
   }

   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      str.append(type + ";");
      str.append(_x + ";");
      str.append(_y + ";");
      str.append(_w + ";");
      str.append(_h + ";");

      return(new String(str));
   }

   public Vector getHotSpots() {
      Vector vec = new Vector();

      vec.addElement(_hsTL);
      vec.addElement(_hsTR);
      vec.addElement(_hsBL);
      vec.addElement(_hsBR);
      vec.addElement(_hsC);

      return vec;
   }

   public void setImage(Image img) {
     this.img = img;
   }

   public Image getImage() {
     return img;
   }

   public Rectangle getBounds() {
      if (img == null) return null;

      int x = _w < 0 ? _x + _w : _x;
      int y = _h < 0 ? _y + _h : _y;
      int w = _w < 0 ? -_w : _w;
      int h = _h < 0 ? -_h : _h;
 
      return new Rectangle(x, y, w, h);
   }

   public void paint(Graphics g) {

   }

   public void update(Observable obs, Object obj) {
      Point coordTL = _hsTL.getPoint();
      Point coordTR = _hsTR.getPoint();
      Point coordBL = _hsBL.getPoint();
      Point coordBR = _hsBR.getPoint();
      Point coordC = _hsC.getPoint();

      if (obs == _hsTL) {
         _x = coordTL.x;
         _y = coordTL.y;
         _w = coordBR.x - coordTL.x;
         _h = coordBR.y - coordTL.y;
      } else if (obs == _hsTR) {
         _y = coordTR.y;
         _w = coordTR.x - coordTL.x;
         _h = coordBR.y - coordTR.y;
      } else if (obs == _hsBL) {
         _x = coordBL.x;
         _w = coordBR.x - coordBL.x;
         _h = coordBL.y - coordTL.y;
      } else if (obs == _hsBR) {
         _w = coordBR.x - coordTL.x;
         _h = coordBR.y - coordTL.y;
      } else if (obs == _hsC) {
         _x = coordC.x - _w/2;
         _y = coordC.y - _h/2;
      }

      _hsTL.setPoint(new Point(_x, _y));
      _hsTR.setPoint(new Point(_x + _w, _y));
      _hsBL.setPoint(new Point(_x, _y + _h));
      _hsBR.setPoint(new Point(_x + _w, _y + _h));
      _hsC.setPoint(new Point(_x + _w/2, _y + _h/2));
   }

   public void changeCoordinates(double xfactor, double yfactor) {
      if (!isRelativeDrawing) return;

      Point tmp1 = getChangedValue(xfactor, yfactor, new Point(_x, _y));
      Point tmp2 = getChangedValue(xfactor, yfactor, new Point(_w, _h));

      _x = tmp1.x;
      _y = tmp1.y;
      _w = tmp2.x;
      _h = tmp2.y;

      _hsTL.setPoint(new Point(_x, _y));
      _hsTR.setPoint(new Point(_x + _w, _y));
      _hsBL.setPoint(new Point(_x, _y + _h));
      _hsBR.setPoint(new Point(_x + _w, _y + _h));
      _hsC.setPoint(new Point(_x + _w/2, _y + _h/2));
   }

   public boolean contains(int x, int y) {
     int xx = _w < 0 ? _x + _w : _x;
     int yy = _h < 0 ? _y + _h : _y;
     int ww = _w < 0 ? -_w : _w;
     int hh = _h < 0 ? -_h : _h;

     Rectangle rect = new Rectangle(xx, yy, ww, hh);
     return rect.contains(x, y);
   } 
}
