package sv.drawer;

/** 
 * LineShape.java
 * Define line object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

import sv.ui.SVUIC;

public class LineShape extends ShapeObject implements Observer {

   public static final int NOARROW_LINE          = 1;
   public static final int RIGHTFILLARROW_LINE   = 2;
   public static final int RIGHTARROW_LINE       = 3;
   public static final int LEFTFILLARROW_LINE    = 4;
   public static final int LEFTARROW_LINE        = 5;
   public static final int LRFILLARROW_LINE      = 6;
   public static final int LRARROW_LINE          = 7;

   public static final int NOARROW_DLINE         = 11;
   public static final int RIGHTFILLARROW_DLINE  = 12;
   public static final int RIGHTARROW_DLINE      = 13;
   public static final int LEFTFILLARROW_DLINE   = 14;
   public static final int LEFTARROW_DLINE       = 15;
   public static final int LRFILLARROW_DLINE     = 16;
   public static final int LRARROW_DLINE         = 17;

   HotSpot _hs1, _hs2;

   int _x1, _y1, _x2, _y2;

   Color linec;
   int line_type;
   int line_width;
   int dash_size;

   public LineShape(int type, int x1, int y1, int x2, int y2,
                                     Color c, int t, int w, int s) {
      this.type = type;
      _x1 = x1;
      _y1 = y1;
      _x2 = x2;
      _y2 = y2;

      linec = c;
      line_type = t;
      line_width = w;
      dash_size = s;

      _hs1 = new HotSpot(new Point(x1, y1));
      _hs2 = new HotSpot(new Point(x2, y2));

      _hs1.addObserver(this);
      _hs2.addObserver(this);
   }

   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      str.append(type + ";");
      str.append(_x1 + ";");
      str.append(_y1 + ";");
      str.append(_x2 + ";");
      str.append(_y2 + ";");
      str.append(linec.getRGB() + ";");
      str.append(line_type + ";");
      str.append(line_width + ";");
      str.append(dash_size + ";");
      return(new String(str));
   }

   public Vector getHotSpots() {
      Vector vec = new Vector();

      vec.addElement(_hs1);
      vec.addElement(_hs2);

      return vec;
   }

   public void setLineColor(Color c) {
      linec = c; 
   }

   public void setLineType(int val) {
      line_type = val;
   }

   public void setLineWidth(int val) {
      line_width = val;
   }

   public void setDashSize(int val) {
      dash_size = val;
   }


   public void paint(Graphics g) {
      Point[] pts;

      switch (line_type) {
        case NOARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          break;

        case RIGHTFILLARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, true, pts);

          break;

        case RIGHTARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, false, pts);
          break;

        case LEFTFILLARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, true, pts);
          break;

        case LEFTARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, false, pts);
          break;

        case LRFILLARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, true, pts);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, true, pts);
          break;

        case LRARROW_LINE:
          SVUIC.drawLine(g, _x1, _y1, _x2, _y2, line_width, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, false, pts);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, false, pts);
          break;

        case NOARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          break;

        case RIGHTFILLARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, true, pts);
          break;

        case RIGHTARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, false, pts);
          break;

        case LEFTFILLARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, true, pts);
          break;

        case LEFTARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, false, pts); 
          break;

        case LRFILLARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, true, pts);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, true, pts);
          break;

        case LRARROW_DLINE:
          SVUIC.drawDottedLine(g, _x1, _y1, _x2, _y2, 
                                  line_width, dash_size, linec);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x2, _y2), new Point(_x1, _y1));
          drawArrow(g, false, pts);
          pts = ShapeProperties.calcArrow(line_width, 
                       new Point(_x1, _y1), new Point(_x2, _y2));
          drawArrow(g, false, pts);
          break;

      }
   }

   public void update(Observable obs, Object obj) {
      Point coord1 = _hs1.getPoint();
      Point coord2 = _hs2.getPoint();

      if (obs == _hs1) {
         _x1 = coord1.x;
         _y1 = coord1.y;
      } else if (obs == _hs2) {
         _x2 = coord2.x;
         _y2 = coord2.y;
      }

      _hs1.setPoint(new Point(_x1, _y1));
      _hs2.setPoint(new Point(_x2, _y2));
   }

   public void changeCoordinates(double xfactor, double yfactor) {
      if (!isRelativeDrawing) return;

      Point tmp1 = getChangedValue(xfactor, yfactor, new Point(_x1, _y1));
      Point tmp2 = getChangedValue(xfactor, yfactor, new Point(_x2, _y2));

      _x1 = tmp1.x;
      _y1 = tmp1.y;
      _x2 = tmp2.x;
      _y2 = tmp2.y;

      _hs1.setPoint(tmp1);
      _hs2.setPoint(tmp2);
   }

   public boolean contains(int x, int y) {
     Rectangle rect = null;
     if (_x1 < _x2) {
       rect = _y1 < _y2 ? new Rectangle(_x1, _y1, _x2-_x1, _y2-_y1) :
                          new Rectangle(_x1, _y2, _x2-_x1, _y1-_y2);
     } else {
       rect = _y1 < _y2 ?  new Rectangle(_x2, _y1, _x1-_x2, _y2-_y1) :
                           new Rectangle(_x2, _y2, _x1-_x2, _y1-_y2);
     }
     
     return rect.contains(x, y);
   } 

   /**
    * draw arrow
    */
   private void drawArrow(Graphics g, boolean fill, Point[] pts) {
      int[] xpts = new int[3];
      int[] ypts = new int[3];

      xpts[0] = pts[0].x;
      xpts[1] = pts[1].x;
      xpts[2] = pts[2].x;
      ypts[0] = pts[0].y;
      ypts[1] = pts[1].y;
      ypts[2] = pts[2].y;

      if (fill)
         g.fillPolygon(xpts, ypts, 3);
      else {
         g.setColor(bgColor);
         g.fillPolygon(xpts, ypts, 3);
         g.setColor(linec);
         g.drawPolygon(xpts, ypts, 3);
      }
   }  

} // end of class





