package sv.drawer;

/** 
 * MLineShape.java
 * Define multi-line object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

import sv.ui.SVUIC;

public class MLineShape extends ShapeObject implements Observer {

   protected Vector _hsvec = new Vector();

   protected Vector _ptsvec = new Vector();

   protected Color linec;
   protected int line_type;
   protected int line_width;
   protected int dash_size;

   public MLineShape() {
     // nothing to do
   }

   public MLineShape(int type, int x1, int y1, Color c, int t, int w, int s) {
      this.type = type;
      _ptsvec.addElement(new Point(x1, y1));

      linec = c;
      line_type = t;
      line_width = w;
      dash_size = s;

      HotSpot hs = new HotSpot(new Point(x1, y1));
      hs.addObserver(this);
      _hsvec.addElement(hs);
   }

   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      int size = _ptsvec.size();

      str.append(type + ";");
      str.append(size + ";");

      for (int i=0; i<size; i++) {
        Point p = (Point)_ptsvec.elementAt(i);
        str.append(p.x + ";");
        str.append(p.y + ";");
      }

      str.append(linec.getRGB() + ";");
      str.append(line_type + ";");
      str.append(line_width + ";");
      str.append(dash_size + ";");
      return(new String(str));
   }

   public Vector getHotSpots() {
      return _hsvec;
   }

   public HotSpot addPoint(Point p) {
      _ptsvec.addElement(p);

      HotSpot hs = new HotSpot(p);
      hs.addObserver(this);
      _hsvec.addElement(hs);

      return hs;
   }

   public void setLineColor(Color c) {
      linec = c; 
   }

   public void setLineType(int val) {
      line_type = val;
   }

   public void setLineWidth(int val) {
      line_width = val;
   }

   public void setDashSize(int val) {
      dash_size = val;
   }


   public void paint(Graphics g) {
      if (_ptsvec.size() < 2) return;

      Point[] pts = new Point[_ptsvec.size()];
      Point[] arrow;

      _ptsvec.copyInto(pts);

      g.setColor(linec);
      int togo = pts.length - 2;  
      int togo2 = pts.length - 1; 

      switch (line_type) {
        case LineShape.NOARROW_LINE:
          for (int i=0; i<togo2; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          break;

        case LineShape.RIGHTFILLARROW_LINE:
          for (int i=0; i<togo; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          SVUIC.drawLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          break;

        case LineShape.RIGHTARROW_LINE:
          for (int i=0; i<togo; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          SVUIC.drawLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          break;

        case LineShape.LEFTFILLARROW_LINE:
          SVUIC.drawLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          for (int i=1; i<togo2; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          break;

        case LineShape.LEFTARROW_LINE:
          SVUIC.drawLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          for (int i=1; i<togo2; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          break;

        case LineShape.LRFILLARROW_LINE:
          SVUIC.drawLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          for (int i=1; i<togo; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          SVUIC.drawLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          break;

        case LineShape.LRARROW_LINE:
          SVUIC.drawLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          for (int i=1; i<togo; i++) 
             SVUIC.drawLine(g, pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                                                         line_width, linec);
          SVUIC.drawLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          break;

        case LineShape.NOARROW_DLINE:
          for (int i=0; i<togo2; i++) 
             SVUIC.drawDottedLine(g, 
                       pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                       line_width, dash_size, linec);

          break;

        case LineShape.RIGHTFILLARROW_DLINE:
          for (int i=0; i<togo; i++) 
             SVUIC.drawDottedLine(g, 
                          pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                          line_width, dash_size, linec);
          SVUIC.drawDottedLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          break;

        case LineShape.RIGHTARROW_DLINE:
          for (int i=0; i<togo; i++) 
             SVUIC.drawDottedLine(g, 
                          pts[i].x, pts[i].y, pts[i+1].x, pts[i+1].y, 
                          line_width, dash_size, linec);
          SVUIC.drawDottedLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          break;

        case LineShape.LEFTFILLARROW_DLINE:
          SVUIC.drawDottedLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          for (int i=1; i<togo2; i++) 
             SVUIC.drawDottedLine(g,pts[i].x, pts[i].y, 
                pts[i+1].x, pts[i+1].y, line_width, dash_size, linec);
          break;

        case LineShape.LEFTARROW_DLINE:
          SVUIC.drawDottedLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          for (int i=1; i<togo2; i++) 
             SVUIC.drawDottedLine(g,pts[i].x, pts[i].y, 
                pts[i+1].x, pts[i+1].y, line_width, dash_size, linec);
          break;

        case LineShape.LRFILLARROW_DLINE:
          SVUIC.drawDottedLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, true, arrow);
          for (int i=1; i<togo; i++) 
             SVUIC.drawDottedLine(g,pts[i].x, pts[i].y, 
                pts[i+1].x, pts[i+1].y, line_width, dash_size, linec);
          SVUIC.drawDottedLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, true, arrow);
          break;

        case LineShape.LRARROW_DLINE:
          SVUIC.drawDottedLine(g, pts[0].x, pts[0].y, pts[1].x, pts[1].y, 
                                              line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[1], pts[0]);
          drawArrow(g, false, arrow);
          for (int i=1; i<togo; i++) 
             SVUIC.drawDottedLine(g,pts[i].x, pts[i].y, 
                pts[i+1].x, pts[i+1].y, line_width, dash_size, linec);
          SVUIC.drawDottedLine(g, pts[togo].x, pts[togo].y,  
                pts[togo2].x, pts[togo2].y, line_width, dash_size, linec);
          arrow = ShapeProperties.calcArrow(line_width, pts[togo], pts[togo2]);
          drawArrow(g, false, arrow);
          break;

      }

   }

   public void update(Observable obs, Object obj) {
      for (int i=0; i<_hsvec.size(); i++) {
         HotSpot hs = (HotSpot)_hsvec.elementAt(i);
         Point p = (Point)_ptsvec.elementAt(i);

         Point coord = hs.getPoint();

         if (obs == hs) {
            p.x = coord.x;
            p.y = coord.y;
            hs.setPoint(p);
         }
      }
   }

   public void changeCoordinates(double xfactor, double yfactor) {
   }

   public boolean contains(int x, int y) {
     Rectangle rect = null;

     int xmin = 500;
     int xmax = -100;
     int ymin = 500;
     int ymax = -100;

     for (int i=0; i<_ptsvec.size(); i++) {
        Point p = (Point)_ptsvec.elementAt(i);

        if (xmin > p.x) xmin = p.x;
        if (xmax < p.x) xmax = p.x;
        if (ymin > p.y) ymin = p.y;
        if (ymax < p.y) ymax = p.y;
     }

     rect = new Rectangle(xmin, ymin, xmax-xmin, ymax-ymin);

     return rect.contains(x, y);
   } 


   /**
    * draw arrow
    */
   protected void drawArrow(Graphics g, boolean fill, Point[] pts) {
      int[] xpts = new int[3];
      int[] ypts = new int[3];

      xpts[0] = pts[0].x;
      xpts[1] = pts[1].x;
      xpts[2] = pts[2].x;
      ypts[0] = pts[0].y;
      ypts[1] = pts[1].y;
      ypts[2] = pts[2].y;

      if (fill)
         g.fillPolygon(xpts, ypts, 3);
      else {
         g.setColor(bgColor);
         g.fillPolygon(xpts, ypts, 3);
         g.setColor(linec);
         g.drawPolygon(xpts, ypts, 3);
      }
   }  


} // end of class

