package sv.drawer;

/** 
 * OvalShape.java
 * Define oval object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */


import java.awt.*;
import java.util.*;

public class OvalShape extends ShapeObject implements Observer {
   HotSpot _hsT, _hsB,_hsR, _hsL,_hsC;

   int _x, _y, _w, _h;

   Color linec = Color.black;
   Color fillc = Color.white;
   boolean fillmode = false;

   public OvalShape(int type, int x, int y, int w, int h) {
      this.type = type;
      _x = x;
      _y = y;
      _w = w;
      _h = h;

      _hsT = new HotSpot(new Point(x + w/2, y));
      _hsB = new HotSpot(new Point(x + w/2, y + h));
      _hsR = new HotSpot(new Point(x + w, y + h/2));
      _hsL = new HotSpot(new Point(x, y + h/2));
      _hsC = new HotSpot(new Point(x + w/2, y + h/2), new Dimension(15, 15));

      _hsT.addObserver(this);
      _hsB.addObserver(this);
      _hsR.addObserver(this);
      _hsL.addObserver(this);
      _hsC.addObserver(this);
   }

   public OvalShape(int type, int x1, int y1, int x2, int y2, 
                           int lc, int fc, String m) {
      this(type, x1, y1, x2, y2);
      linec = new Color(lc);
      fillc = new Color(fc);
      if (m.equals("yes")) fillmode = true;
   }

   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      str.append(type + ";");
      str.append(_x + ";");
      str.append(_y + ";");
      str.append(_w + ";");
      str.append(_h + ";");
      str.append(linec.getRGB() + ";");
      str.append(fillc.getRGB() + ";");
      if (fillmode)
         str.append("yes;");
      else
         str.append("no;");
      return(new String(str));
   }

   public Vector getHotSpots() {
      Vector vec = new Vector();

      vec.addElement(_hsT);
      vec.addElement(_hsB);
      vec.addElement(_hsR);
      vec.addElement(_hsL);
      vec.addElement(_hsC);

      return vec;
   }

  public void setLineColor(Color c) {
     linec = c;
  }

  public void setFillColor(Color c) {
     fillc = c;
  }


  public boolean isFillMode() {
     return fillmode;
  }

  public void changeFillMode() {
     if (fillmode) fillmode = false;
     else          fillmode = true;
  }


   public void paint(Graphics g) {
      int x = _w < 0 ? _x + _w : _x;
      int y = _h < 0 ? _y + _h : _y;
      int w = _w < 0 ? -_w : _w;
      int h = _h < 0 ? -_h : _h;

      if (fillmode) {
        g.setColor(fillc);
        g.fillOval(x, y, w, h);
        g.setColor(linec);
        g.drawOval(x, y, w, h);
      } else {
        g.setColor(linec);
        g.drawOval(x, y, w, h);
      }
   }

   public void update(Observable obs, Object obj) {
      Point coordT = _hsT.getPoint();
      Point coordB = _hsB.getPoint();
      Point coordR = _hsR.getPoint();
      Point coordL = _hsL.getPoint();
      Point coordC = _hsC.getPoint();

      if (obs == _hsT) {
         _y = coordT.y;
         _w = (coordT.x - coordL.x) * 2;
         _h = coordB.y - coordT.y;
      } else if (obs == _hsB) {
         _w = (coordB.x - coordL.x) * 2;
         _h = coordB.y - coordT.y;
      } else if (obs == _hsR) {
         _w = coordR.x - coordL.x;
         _h = (coordR.y - coordT.y) * 2;
      } else if (obs == _hsL) {
         _x = coordL.x;
         _w = coordR.x - coordL.x;
         _h = (coordL.y - coordT.y) * 2;
      } else if (obs == _hsC) {
         _x = coordC.x - _w/2;
         _y = coordC.y - _h/2;
      }

      _hsT.setPoint(new Point(_x + _w/2, _y));
      _hsB.setPoint(new Point(_x + _w/2, _y + _h));
      _hsR.setPoint(new Point(_x + _w, _y + _h/2));
      _hsL.setPoint(new Point(_x, _y + _h/2));
      _hsC.setPoint(new Point(_x + _w/2, _y + _h/2));
   }

   public void changeCoordinates(double xfactor, double yfactor) {
      if (!isRelativeDrawing) return;

      Point tmp1 = getChangedValue(xfactor, yfactor, new Point(_x, _y));
      Point tmp2 = getChangedValue(xfactor, yfactor, new Point(_w, _h));

      _x = tmp1.x;
      _y = tmp1.y;
      _w = tmp2.x;
      _h = tmp2.y;

      _hsT.setPoint(new Point(_x + _w/2, _y));
      _hsB.setPoint(new Point(_x + _w/2, _y + _h));
      _hsR.setPoint(new Point(_x + _w, _y + _h/2));
      _hsL.setPoint(new Point(_x, _y + _h/2));
      _hsC.setPoint(new Point(_x + _w/2, _y + _h/2));
   }

   public boolean contains(int x, int y) {
     int xx = _w < 0 ? _x + _w : _x;
     int yy = _h < 0 ? _y + _h : _y;
     int ww = _w < 0 ? -_w : _w;
     int hh = _h < 0 ? -_h : _h;

     Rectangle rect = new Rectangle(xx, yy, ww, hh);
     return rect.contains(x, y);
   } 
}




