package sv.drawer;


/** 
 * PolygonShape.java
 * Define multi-line object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

import sv.ui.SVUIC;

public class PolygonShape extends ShapeObject implements Observer {

   Vector _hsvec = new Vector();

   Vector _ptsvec = new Vector();

   Color linec = Color.black;
   Color fillc = Color.white;
   boolean fillmode = false;

   public PolygonShape(int type, Color lc, Color fc, String m) {
      this.type = type;

      linec = lc;
      fillc = fc;
      if (m.equals("yes")) fillmode = true;
   }

   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      int size = _ptsvec.size();

      str.append(type + ";");
      str.append(size + ";");

      for (int i=0; i<size; i++) {
        Point p = (Point)_ptsvec.elementAt(i);
        str.append(p.x + ";");
        str.append(p.y + ";");
      }

      str.append(linec.getRGB() + ";");
      str.append(fillc.getRGB() + ";");
      if (fillmode)
         str.append("yes;");
      else
         str.append("no;");
      return(new String(str));
   }

   public Vector getHotSpots() {
      return _hsvec;
   }

   public void setLineColor(Color c) {
     linec = c;
   }

   public void setFillColor(Color c) {
     fillc = c;
   }

   public boolean isFillMode() {
     return fillmode;
   }

  public void changeFillMode() {
     if (fillmode) fillmode = false;
     else          fillmode = true;
  }


   public HotSpot[] addPoints(Point[] pts) {
      if (pts == null) return null;

      for (int i=0; i<pts.length; i++) {
         _ptsvec.addElement(pts[i]);

         HotSpot hs = new HotSpot(pts[i]);
         hs.addObserver(this);
         _hsvec.addElement(hs);
      }

      HotSpot[] spots = new HotSpot[_hsvec.size()];
      _hsvec.copyInto(spots);
      return spots;
   }


   public void paint(Graphics g) {
      int size = _ptsvec.size();

      if (size < 2) return;
      if (size == 2) {
         Point p1 = (Point)_ptsvec.elementAt(0);
         Point p2 = (Point)_ptsvec.elementAt(1);
         g.setColor(linec);
         g.drawLine(p1.x, p1.y, p2.x, p2.y);

      } else {
         int xpts[] = new int[size];
         int ypts[] = new int[size];
       
	 for (int i=0; i<size; i++) {
            Point p = (Point)_ptsvec.elementAt(i);
            xpts[i] = p.x;
            ypts[i] = p.y;
         }         

         if (fillmode) {
            g.setColor(fillc);
            g.fillPolygon(xpts, ypts, size);
         }

         g.setColor(linec);
         g.drawPolygon(xpts, ypts, size);

      }
   }

   public void update(Observable obs, Object obj) {
      for (int i=0; i<_hsvec.size(); i++) {
         HotSpot hs = (HotSpot)_hsvec.elementAt(i);
         Point p = (Point)_ptsvec.elementAt(i);

         Point coord = hs.getPoint();

         if (obs == hs) {
            p.x = coord.x;
            p.y = coord.y;
            hs.setPoint(p);
         }
      }
   }

   public void changeCoordinates(double xfactor, double yfactor) {
   }

   public boolean contains(int x, int y) {
     Rectangle rect = null;

     int xmin = 500;
     int xmax = -100;
     int ymin = 500;
     int ymax = -100;

     for (int i=0; i<_ptsvec.size(); i++) {
        Point p = (Point)_ptsvec.elementAt(i);

        if (xmin > p.x) xmin = p.x;
        if (xmax < p.x) xmax = p.x;
        if (ymin > p.y) ymin = p.y;
        if (ymax < p.y) ymax = p.y;
     }

     rect = new Rectangle(xmin, ymin, xmax-xmin, ymax-ymin);

     return rect.contains(x, y);
   } 


} // end of class

