package sv.drawer;

import java.awt.*;

/** 
 * ShapeProperties.java
 * Define shape object properties
 * @version 1.0     10/15/97
 * @author Byeongseob Ki
 */

public class ShapeProperties {

   // objects
   public final static int NONE       = -1;
   public final static int LINE       = 0;
   public final static int MULTILINE  = 1;
   public final static int BEZIER     = 2;
   public final static int MOUSE      = 3;
   public final static int RECTANGLE  = 4;
   public final static int POLYGON    = 5;
   public final static int OVAL       = 6;
   public final static int TEXT       = 7;
   public final static int IMAGE      = 8;

   private Font textFont = new Font("Dialog", Font.PLAIN, 10);

   private Color lineColor = Color.black;
   private Color fillColor = Color.white;
   private Color textColor = Color.black;


   private int line_wid = 1;
   private int dash_wid = 1;
   private int line_type = LineShape.NOARROW_LINE;

   private int shapeType;

   public ShapeProperties() {
     shapeType = -1;
   }

   /**
    * get shape type
    */
   public int getShapeType() {
      return shapeType;
   }

   /**
    * set shape type
    */
   public void setShapeType(int i) {
     shapeType = i;
   }   

   /**
    * get textFont
    */
   public Font getTextFont() {
      return textFont;
   }

   /**
    * set textFont
    */
   public void setTextFont(Font f) {
     textFont = f;
   }   

   /**
    * get lineColor
    */
   public Color getLineColor() {
      return lineColor;
   }

   /**
    * set lineColor
    */
   public void setLineColor(Color c) {
     lineColor = c;
   }   

   /**
    * get fillColor
    */
   public Color getFillColor() {
      return fillColor;
   }

   /**
    * set fillColor
    */
   public void setFillColor(Color c) {
     fillColor = c;
   }   

   /**
    * get textColor
    */
   public Color getTextColor() {
      return textColor;
   }

   /**
    * set textColor
    */
   public void setTextColor(Color c) {
     textColor = c;
   }   


   /**
    * get line width
    */
   public int getLineWidth() {
      return line_wid;
   }

   /**
    * set line width
    */
   public void setLineWidth(int w) {
     line_wid = w;
   }   

   /**
    * get dash size
    */
   public int getDashSize() {
      return dash_wid;
   }

   /**
    * set dash size
    */
   public void setDashSize(int w) {
     dash_wid = w;
   }   


   /**
    * get line type
    */
   public int getLineType() {
      return line_type;
   }

   /**
    * set line type
    */
   public void setLineType(int i) {
     line_type = i;
   }   


   /** 
    * calculate the arrows
    * @param w line width
    * @param p1 start point of the line
    * @param p2 end point of the line
    */
   public static final Point[] calcArrow(int w, Point p1, Point p2) {
      double len = 10.0;
      double wid = (double)(w + 10);

      double dx = p2.x - p1.x;
      double dy = p2.y - p1.y;
      double sz = Math.sqrt(dx * dx + dy * dy);
      if (sz == 0.0)
          sz = 1.0;

      double d1 = dx / sz;
      double d2 = dy / sz;
      double d3 = -dy / sz;
      double d4 = dx / sz;

      Point a1 = new Point(p2.x, p2.y);

      Point a2 = new Point((int)((double)p2.x - len*d1 + 0.5*wid*d3 + 0.5), 
                           (int)((double)p2.y - len*d2 + 0.5*wid*d4 + 0.5));

      Point a3 = new Point((int)((double)p2.x - len*d1 - 0.5*wid*d3 + 0.5), 
                           (int)((double)p2.y - len*d2 - 0.5*wid*d4 + 0.5));

      Point[] pts = { a1, a2, a3 };
      return pts;
   }


} // end of class


