package sv.drawer;

/** 
 * TextShape.java
 * Define Text object
 * @version 0.5     9/15/97
 * @author Byeongseob Ki
 */

import java.awt.*;
import java.util.*;

public class TextShape extends ShapeObject implements Observer {
   HotSpot _hsC;

   int _x, _y;

   Color textc = Color.black;
   Font f = new Font("Serif", Font.PLAIN, 10);
   String label = null;

   private Rectangle rect;

   public TextShape(int type, int x, int y, int w, int h) {
      this.type = type;
      _x = x + w/2;
      _y = y + h/2;

      _hsC = new HotSpot(new Point(_x, _y), new Dimension(15, 15));
      _hsC.addObserver(this);
   }

   public TextShape(int type, int x, int y, int c, String s, Font f) {
      this.type = type;
      textc = new Color(c);
      label = s;
      this.f = f;
      _x = x;
      _y = y;
      _hsC = new HotSpot(new Point(_x, _y), new Dimension(15, 15));
      _hsC.addObserver(this);
   }


   public String makeString() {
      StringBuffer str = new StringBuffer();      // retrun value buffer

      str.append(type + ";");
      str.append(_x + ";");
      str.append(_y + ";");
      str.append(textc.getRGB() + ";");
      str.append(f.getName() + ";");
      str.append(f.getStyle() + ";");
      str.append(f.getSize() + ";");
      str.append(label + ";");
      return(new String(str));
   }

  public void setTextColor(Color c) {
    textc = c;
  }

  public void setFont(Font f) {
    this.f = f;
  }

  public void setLabel(String lbl) {
     label = lbl;
  }

   public Vector getHotSpots() {
      Vector vec = new Vector();
      vec.addElement(_hsC);
      return vec;
   }

   public void paint(Graphics g) {
      if (label == null) return;
      
      Font sf = g.getFont();    // save current font
      g.setFont(f);

      FontMetrics fm = g.getFontMetrics(); 

      int x = _x - fm.stringWidth(label) / 2;
 
      g.setColor(textc);
      g.drawString(label, x, _y);
      g.setFont(sf);           // recover font
      rect = new Rectangle(x-5, _y-fm.getHeight()/2, 
                               fm.stringWidth(label)+10, 2*fm.getHeight());
   }

   public void update(Observable obs, Object obj) {

      Point coordC = _hsC.getPoint();

      if (obs == _hsC) {
         _x = coordC.x;
         _y = coordC.y;
      }

      _hsC.setPoint(new Point(_x, _y));
   }

   public void changeCoordinates(double xfactor, double yfactor) {
      if (!isRelativeDrawing) return;

      Point tmp1 = getChangedValue(xfactor, yfactor, new Point(_x, _y));

      _x = tmp1.x;
      _y = tmp1.y;

      _hsC.setPoint(tmp1);
   }


  public boolean contains(int x, int y) {
     return rect.contains(x, y);
  }

} // end of class


