package sv.kernel;

import java.awt.*;
import sv.ui.SVUIC;

/** 
 * Axis.java (recovered from the lost)
 * Define axis for 1d and vctor
 * @version 1.0
 * @author Byeongseob Ki
 */


public class Axis {

    // boundary area of the axis
    private Rectangle boundRect;

    // min and max values
    private float xmin, xmax, ymin, ymax;

    // ticker mode
    private boolean isTickMode = false;

    // should be draw the grids?
    private boolean isGridX = false, isGridY = false;

    // grid length
    private int gridLength = 5;

    // number of major ticks
    private int xticks_no = 4;
    private int yticks_no = 4;
 
    // number of minor ticks between major ticks
    private int xsmallticks_no = 1;
    private int ysmallticks_no = 1;

    // format mode
    private int xformat_mode = SVUIC.ALGEBRAIC_FORMAT, 
                yformat_mode = SVUIC.ALGEBRAIC_FORMAT;

    // significant and precision
    private int xsig = 3, ysig = 3, xprec = 2, yprec = 2;

    /**
     * constructor
     */
    public Axis() {
        boundRect = new Rectangle();
    }

    /** 
     * set ticker mode
     */
    public void setTickMode(boolean flag) {
        isTickMode = flag;
    }

    /**
     * set boundary rectangle
     */
    public void setAxisBounds(Rectangle rect) {
        boundRect.x = rect.x - 5;
        boundRect.y = rect.y - 5;
        boundRect.width = rect.width + 10;
        boundRect.height = rect.height + 10;
    }

    /**
     * set boundary rectangle
     */
    public void setAxisMinMax(float[] minmaxArr) {
        xmin = minmaxArr[0];
        xmax = minmaxArr[1];
        ymin = minmaxArr[2];
        ymax = minmaxArr[3];
    }

  /**
   * painter
   */
  public void paint(Graphics g) {
    Color savedColor = g.getColor();
    g.setColor(Color.black);

    g.drawRect(boundRect.x, boundRect.y, boundRect.width, boundRect.height);
    FontMetrics fm = g.getFontMetrics(g.getFont());

    int labelWid, labelHgt, dummy=0, tmpx, tmpy;
    int step, step2;
    float labelStep;
    String label;

    /***********************************************************
     * TICK MODE
     ***********************************************************/
    if (isTickMode) {
      // STEP1: draw x ticks and tick labels
      step = boundRect.width / (xticks_no + 1);
      step2 = step / (xsmallticks_no+1);
      tmpy = boundRect.y+boundRect.height;
       
      labelStep = (xmax - xmin) / (float)(xticks_no + 1);
      labelHgt = fm.getHeight();


      label = SVUIC.formatFloat(xmin, xsig, xprec, xformat_mode);
      labelWid = fm.stringWidth(label);
      g.drawString(label, boundRect.x-labelWid/2, tmpy+labelHgt);

      for (int i=0; i< xticks_no; i++) {
         tmpx = boundRect.x+step*(i+1);
         g.drawLine(tmpx, tmpy, tmpx, tmpy-5);

         label = SVUIC.formatFloat(xmin + (float)(i+1)*labelStep, 
                                              xsig, xprec, xformat_mode);
         labelWid = fm.stringWidth(label);
         g.drawString(label, tmpx-labelWid/2, tmpy+labelHgt);

         if (isGridX) 
           SVUIC.drawDottedLine(g, tmpx, boundRect.y, 
              tmpx, boundRect.y+boundRect.height, 1, gridLength, Color.black);


         for (int j=0; j< xsmallticks_no; j++) {
            tmpx = boundRect.x+step*i + step2*(j+1);
            g.drawLine(tmpx, tmpy, tmpx, tmpy-3);
         }
         dummy = i;
      }
      for (int j=0; j< xsmallticks_no; j++) {
         tmpx = boundRect.x+step*(dummy+1) + step2*(j+1);
         g.drawLine(tmpx, tmpy, tmpx, tmpy-3);
      }  
      label = SVUIC.formatFloat(xmax, xsig, xprec, xformat_mode);
      labelWid = fm.stringWidth(label);
      g.drawString(label, boundRect.x+boundRect.width-labelWid/2, 
                                                   tmpy+labelHgt);

      // STEP2: draw y ticks and tick labels
      step = boundRect.height / (yticks_no + 1);
      step2 = step / (ysmallticks_no+1);
      tmpx = boundRect.x;

      labelStep = (ymax - ymin) / (float)(yticks_no + 1);

      label = SVUIC.formatFloat(ymin, ysig, yprec, yformat_mode);
      labelWid = fm.stringWidth(label);
      g.drawString(label, boundRect.x-labelWid-5, 
                      boundRect.y+boundRect.height);

      for (int i=0; i< yticks_no; i++) {
         tmpy = boundRect.y+boundRect.height-step*(i+1);
         g.drawLine(tmpx, tmpy, tmpx+5, tmpy);

         label = SVUIC.formatFloat(ymin + (float)(i+1)*labelStep, 
                                              ysig, yprec, yformat_mode);
         labelWid = fm.stringWidth(label);
         g.drawString(label, boundRect.x-labelWid-5, tmpy+labelHgt/2);

         if (isGridY) 
           SVUIC.drawDottedLine(g, boundRect.x, tmpy, 
              boundRect.x+boundRect.width, tmpy, 1, gridLength, Color.black);

         for (int j=0; j< ysmallticks_no; j++) {
            tmpy = boundRect.y+step*i + step2*(j+1);
            g.drawLine(tmpx, tmpy, tmpx+3, tmpy);
         }
         dummy = i;
      }
      for (int j=0; j< ysmallticks_no; j++) {
         tmpy = boundRect.y+step*(dummy+1) + step2*(j+1);
         g.drawLine(tmpx, tmpy, tmpx+3, tmpy);
      }  
      label = SVUIC.formatFloat(ymax, ysig, yprec, yformat_mode);
      labelWid = fm.stringWidth(label);
      g.drawString(label, boundRect.x-labelWid-5, boundRect.y+labelHgt/2);


    /***********************************************************
     * NO TICK MODE
     ***********************************************************/
    } else {

       String upper = SVUIC.formatFloat(xmax, xsig, xprec, xformat_mode) +
              ", " + SVUIC.formatFloat(ymax, ysig, yprec, yformat_mode);
       String lower = SVUIC.formatFloat(xmin, xsig, xprec, xformat_mode) +
              ", " + SVUIC.formatFloat(ymin, ysig, yprec, yformat_mode);
       labelWid = fm.stringWidth(upper);
       g.drawString(upper, boundRect.x + boundRect.width - labelWid,
                                 boundRect.y - fm.getMaxDescent() - 5);
       g.drawString(lower, boundRect.x - 5, 
                     boundRect.y + boundRect.height + fm.getMaxAscent() + 5);
 
    }

    g.setColor(savedColor);
  }


    /**
     * set settings
     */
    public void setSetting(int intvars[], boolean aflag[]) {
        xticks_no = intvars[0];
        yticks_no = intvars[1];
        xsmallticks_no = intvars[2];
        ysmallticks_no = intvars[3];
        xsig = intvars[4];
        xprec = intvars[5];
        xformat_mode = intvars[6];
        ysig = intvars[7];
        yprec = intvars[8];
        yformat_mode = intvars[9];
        isGridX = aflag[0];
        isGridY = aflag[1];
    }

    /**
     * return all int variables
     */
    public int[] getIntVars() {
        int intvars[] = new int[10];
        intvars[0] = xticks_no;
        intvars[1] = yticks_no;
        intvars[2] = xsmallticks_no;
        intvars[3] = ysmallticks_no;
        intvars[4] = xsig;
        intvars[5] = xprec;
        intvars[6] = xformat_mode;
        intvars[7] = ysig;
        intvars[8] = yprec;
        intvars[9] = yformat_mode;
        return intvars;
    }

    /**
     * return all boolean variables
     */
    public boolean[] getBoolVars() {
        boolean aflag[] = { isGridX, isGridY };
        return aflag;
    }
}
