package sv.kernel;

import java.util.*;

/** 
 * ClipboardHandler.java
 * Merge Filter Handler
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class ClipboardHandler {
  private Vector data1d_vec;     // clipboard contents (1 dimension)
  private Vector data2d_vec;     // clipboard contents (2 dimension)
  private Vector data3d_vec;     // clipboard contents (3 dimension)
  private Vector data4d_vec;     // clipboard contents (2 dimension)

  public ClipboardHandler() {
     data1d_vec = new Vector();
     data2d_vec = new Vector();
     data3d_vec = new Vector();
     data4d_vec = new Vector();
  }

  /**
   * set clipboard with given DataSet
   * @param dataVec clipboard contents
   */
  public void setClipboard(Vector dataVec) {
     data1d_vec = dataVec;
     data2d_vec.removeAllElements();
     data3d_vec.removeAllElements();
     data4d_vec.removeAllElements();
  }

  /**
   * set clipboard with given DataSet2D
   * @param dataVec clipboard contents
   */
  public void set2DClipboard(Vector dataVec) {
     data2d_vec = dataVec;
     data1d_vec.removeAllElements();
     data3d_vec.removeAllElements(); 
     data4d_vec.removeAllElements();
  }

  /**
   * set clipboard with given DataSetVec
   * @param dataVec clipboard contents
   */
  public void set3DClipboard(Vector dataVec) {
     data3d_vec = dataVec;
     data1d_vec.removeAllElements();
     data2d_vec.removeAllElements();
     data4d_vec.removeAllElements();
  }

  /**
   * set clipboard with given DataSetVec
   * @param dataVec clipboard contents
   */
  public void set4DClipboard(Vector dataVec) {
     data4d_vec = dataVec;
     data1d_vec.removeAllElements();
     data2d_vec.removeAllElements();
     data3d_vec.removeAllElements();
  }



  /**
   * get clipboard contents
   * @return clipboard contents
   */
  public Vector getClipboard() {
     return data1d_vec;
  }

  /**
   * get clipboard 2d data contents
   * @return clipboard contents
   */
  public Vector get2DClipboard() {
     return data2d_vec;
  }


  /**
   * get clipboard 3d data contents
   * @return clipboard contents
   */
  public Vector get3DClipboard() {
     return data3d_vec;
  }


  /**
   * get clipboard 4d data contents
   * @return clipboard contents
   */
  public Vector get4DClipboard() {
     return data4d_vec;
  }

} // end of class
