package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * DSPlane.java
 * Define our scientific data
 * @version 1.0  10/15/97 
 * @author Byeongseob Ki
 */

public class DSPlane extends DataSet {

  public static final int NORMAL       = 1;
  public static final int VECTOR       = 2;
  public static final int CONNECTIVITY = 3;

  protected int dataType;

  /**
   * point data in each time
   */
  public TDPlane[] timeData;


  /**
   * constructor to produce dataset
   * @param title title of this data set
   * @param timedataVec time data vector
   */
  public DSPlane(String title, Vector timeDataVec) {
     setRank(1);
     this.title = title;
     noOfTime = timeDataVec.size();

     timeData = new TDPlane[noOfTime];
     timeArr = new float[noOfTime];

     for (int i=0; i<noOfTime; i++) {
        TDPlane data = (TDPlane)timeDataVec.elementAt(i);
        timeArr[i] = data.getTime();
     }

     // first sort with respect to time of TDPlane
     Sorter sorter = new Sorter();
     sorter.QSortFloat(timeArr, 0, noOfTime-1);

     for (int i=0; i<noOfTime; i++) {
        for (Enumeration e=timeDataVec.elements(); e.hasMoreElements();) {
           TDPlane data = (TDPlane)e.nextElement();
           if (data.getTime() == timeArr[i]) {
              timeData[i] = data;
              timeDataVec.removeElement(data);
              break;
           }
        }
     }
  }


  /**
   * constructor from one TDPlane
   * @param title title of this class
   * @param timedata TDPlane to be added
   */
  public DSPlane(String title, TDPlane timedata) {
     setRank(1);
     this.title = title;

     timeData = new TDPlane[1];
     timeArr = new float[1];

     timeData[0] = timedata;
     timeArr[0] = timedata.getTime();
  }


  /** 
   * add TDPlane to this class
   * @param timedata TDPlane to be added
   */
  public void addTimeData(TimeData data) {
     int pos, oldsize, newsize;

     if (noOfTime >= bufferSize) {
        // remove additional data set and then insert
        
         TDPlane[] tmptd = new TDPlane[bufferSize-1];
         float[] tmpta = new float[bufferSize-1];

         pos = noOfTime - bufferSize + 1;

         System.arraycopy(timeData, pos, tmptd, 0, tmptd.length);
         System.arraycopy(timeArr, pos, tmpta, 0, tmptd.length);
         noOfTime = bufferSize - 1;

         timeData = new TDPlane[noOfTime];
         timeArr = new float[noOfTime];

         System.arraycopy(tmptd, 0, timeData, 0, noOfTime);
         System.arraycopy(tmpta, 0, timeArr, 0, noOfTime);
     } 

     TDPlane timedata = (TDPlane)data;
     float newtime = timedata.getTime();

     noOfTime++;     

     TDPlane[] tmpTimeData = timeData;
     float[] tmpTimeArr = timeArr;
     oldsize = tmpTimeArr.length;
     newsize = oldsize + 1;

     pos = -1;

     for (int i=oldsize-1; i>=0; i--) {
        if (tmpTimeData[i].getTime() < newtime) {
           pos = i;
           break;
        }
     }

     timeData = new TDPlane[newsize];
     timeArr = new float[newsize];

     if (pos == -1) {
        timeData[0] = timedata;
        timeArr[0] = timedata.getTime();
        for (int i=0; i<oldsize; i++) {
          timeData[i+1] = tmpTimeData[i];
          timeArr[i+1] = tmpTimeArr[i];
        }
     } else if (pos == oldsize) {
        for (int i=0; i<oldsize; i++) {
          timeData[i] = tmpTimeData[i];
          timeArr[i] = tmpTimeArr[i];
        }
        timeData[oldsize] = timedata;
        timeArr[oldsize] = timedata.getTime();
     } else {
        for (int i=0; i<=pos; i++) {
           timeData[i] = tmpTimeData[i];
           timeArr[i] = tmpTimeArr[i];
        }
        timeData[pos+1] = timedata;
        timeArr[pos+1] = timedata.getTime();
        for (int i=pos+2; i<newsize; i++) {
           timeData[i] = tmpTimeData[i-1];
           timeArr[i] = tmpTimeArr[i-1];
        }
     }
  }


  /**
   * update TDPlane 
   * @param data TDPlane to be update
   * @param pos position to be updated
   */
  public void updateTimeData(TDPlane data, int pos, boolean parallel) {
     if (data.getDataType() == DSPlane.NORMAL)
        ((TimeData1D)timeData[pos]).updateData((TimeData1D)data, parallel);
     else if (data.getDataType() == DSPlane.VECTOR)
        ((TimeDataVec)timeData[pos]).updateData((TimeDataVec)data, parallel);
     else if (data.getDataType() == DSPlane.CONNECTIVITY)
        ((TimeDataCon)timeData[pos]).updateData((TimeDataCon)data, parallel);
  }

  /**
   * return data type
   */
  public int getDataType() {
     return dataType;
  }
  
} // end of class
