package sv.kernel;

/** 
 * DataSet.java
 * Define our scientific data
 * @version 1.0     10/1/97
 * @author Byeongseob Ki
 */


public abstract class DataSet {
 
  /**
   * 1 dimensional data set
   */
  public static final int ONED = 1;

  /**
   * 2 dimensional data set
   */
  public static final int TWOD = 2;

  /**
   * 3 dimensional data set (isosurface data)
   */
  public static final int THREED = 3;


  /**
   * title of this DataSet
   */
  protected String title = "Untitled";

  /**
   * time array for all TimeData attached this DataSet
   */
  protected float[] timeArr;

  /**
   * the size of time data array for all TimeData attached this DataSet
   */
  protected int noOfTime = 1;


  /**
   * buffer size
   */
  protected int bufferSize = Integer.MAX_VALUE;

  // dimension of the data set
  private int dimension_;

  /**
   * add one time data to this data set
   * @param data time data to be added
   */
  public abstract void addTimeData(TimeData data);


  /**
   * return title of this DataSet
   */
  public String getTitle() {
     return title;
  }


  /**
   * rename the title
   */
  public void setTitle(String t) {
     title = t;
  }

  /**
   * return timeArr
   */
  public float[] getTimeArr() {
     return timeArr;
  }

  /**
   * return i-th time value
   */
  public float getTimeValue(int i) {
     if (timeArr.length > i)
        return timeArr[i];
     return -1.0f;
  }
  

  /**
   * update the timeArr
   */
  public void setTimeArr(float[] ta) {
     timeArr = ta;
  }
 
  /**
   * return noOfTime
   */
  public int getnoOfTime() {
     return noOfTime;
  }

  /**
   * update noOfTime
   */
  public void setnoOfTime(int n) {
     noOfTime = n;
  }

  /**
   * return dimension of this frame
   */
  public int getRank() {
     return dimension_;
  }

  /**
   * set dimension of this frame
   * @param d dimension
   */
  public void setRank(int d) {
     dimension_ = d;
  }


  /**
   * return buffer size of this frame
   */
  public int getBufferSize() {
     return bufferSize;
  }

  /**
   * set buffer size of this frame
   * @param sz buffer size
   */
  public void setBufferSize(int sz) {
     bufferSize = sz;
  }

}  // end of class

