package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * DataSet2D.java
 * Define our 2d scientific data
 * @version 1.0     10/1/97
 * @author Byeongseob Ki
 */

public class DataSet2D extends DataSet {

  /**
   * point data in each time
   */
  public TimeData2D[] timeData;


  /**
   * constructor for 2D  data
   * @param title title of this class
   * @param td time data
   */
  public DataSet2D(String title, TimeData2D td) {
      setRank(2);
      this.title = title;
      timeArr = new float[1];
      timeData = new TimeData2D[1];

      timeData[0] = td;
      timeArr[0] = td.getTime();
  }

  /**
   * constructor for 2D  data
   * @param title title of this class
   * @param timeDataVec time data vector
   */
  public DataSet2D(String title, Vector vec) {
     setRank(2);
     this.title = title;
     noOfTime = vec.size();

     timeData = new TimeData2D[noOfTime];
     timeArr = new float[noOfTime];

     vec.copyInto(timeData);
     for (int i=0; i<noOfTime; i++) {
//        timeData[i] = (TimeData2D)vec.elementAt(i);
        timeArr[i] = timeData[i].getTime();
     }
  }


  /** 
   * add TimeData2D to this class
   * @param timedata TimeData2D to be added
   */
  public void addTimeData(TimeData data) {
     int pos, oldsize, newsize;

     if (noOfTime >= bufferSize) {
        // remove additional data set and then insert
        
         TimeData2D[] tmptd = new TimeData2D[bufferSize-1];
         float[] tmpta = new float[bufferSize-1];

         pos = noOfTime - bufferSize + 1;

         System.arraycopy(timeData, pos, tmptd, 0, tmptd.length);
         System.arraycopy(timeArr, pos, tmpta, 0, tmptd.length);
         noOfTime = bufferSize - 1;

         timeData = new TimeData2D[noOfTime];
         timeArr = new float[noOfTime];

         System.arraycopy(tmptd, 0, timeData, 0, noOfTime);
         System.arraycopy(tmpta, 0, timeArr, 0, noOfTime);
     } 


     TimeData2D timedata = (TimeData2D)data;
     float newtime = timedata.getTime();

     noOfTime++;     

     TimeData2D[] tmpTimeData = timeData;
     float[] tmpTimeArr = timeArr;

     oldsize = tmpTimeArr.length;
     newsize = oldsize + 1;

     pos = -1;

     for (int i=oldsize-1; i>=0; i--) {
        if (tmpTimeData[i].getTime() < newtime) {
           pos = i;
           break;
        }
     }


     timeData = new TimeData2D[newsize];
     timeArr = new float[newsize];

     if (pos == -1) {
        timeData[0] = timedata;
        timeArr[0] = timedata.getTime();
        for (int i=0; i<oldsize; i++) {
          timeData[i+1] = tmpTimeData[i];
          timeArr[i+1] = tmpTimeArr[i];
        }
     } else if (pos == oldsize) {
        for (int i=0; i<oldsize; i++) {
          timeData[i] = tmpTimeData[i];
          timeArr[i] = tmpTimeArr[i];
        }
        timeData[oldsize] = timedata;
        timeArr[oldsize] = timedata.getTime();
     } else {
        for (int i=0; i<=pos; i++) {
           timeData[i] = tmpTimeData[i];
           timeArr[i] = tmpTimeArr[i];
        }
        timeData[pos+1] = timedata;
        timeArr[pos+1] = timedata.getTime();
        for (int i=pos+2; i<newsize; i++) {
           timeData[i] = tmpTimeData[i-1];
           timeArr[i] = tmpTimeArr[i-1];
        }
     }
  }


  /**
   * update TimeData 
   * @param data TimeData2D to be update
   * @param pos position to be updated
   */
  public void updateTimeData(TimeData2D data, int pos) {
     timeData[pos].updateData(data);
  }


  /**
   * set TimeDataVec
   * @param vs TimeDataVec to be set
   * @param pos position to be set
   */
  public void setVectorSet(TimeDataVec vs, int pos) {
     timeData[pos].setVectorSet(vs);
  }

}






