package sv.kernel;

import java.util.*;
import java.io.*;

import sv.util.*;

/** 
 * FilterMenuHandler.java
 * Merge Filter Handler
 * @version 1.0   10/1/97 
 * @author Byeongseob Ki
 */

public class FilterMenuHandler {

  public static Vector menuVec;          // MenuInfo vector
  public static Vector menuVec2D ;       // MenuInfo vector for 2D
  public static Vector menuVec4D ;       // MenuInfo vector for 4D

  public FilterMenuHandler(String sv_menu) {
     menuVec = new Vector();
     menuVec2D = new Vector();
     menuVec4D = new Vector();
 
     if (sv_menu != null)
       parseMenuInfo(sv_menu);
  }

  /**
   * retrun menuVec enumeration
   */
  public Enumeration get1dElements() {
     return menuVec.elements();
  }


  /**
   * retrun menuVec2D enumeration
   */
  public Enumeration get2dElements() {
     return menuVec2D.elements();
  }

  /**
   * retrun menuVec4D enumeration
   */
  public Enumeration get4dElements() {
     return menuVec4D.elements();
  }



  // Each line should start with begin, <menu>, <item>, </item>, or end. 
  // It allow only 3 level total.
  private void parseMenuInfo(String sv_menu) {
     
     BufferedReader in = null;

     String line, tmp, menuName, token;
     int level = 0, index = 0, root0 = 0, root1 = 0, root2 = 0;

     String mname = null, mclass = null, mpackage = null;
     int mkey = -1;

     try {
        in = new BufferedReader(new FileReader(sv_menu));
     } catch (Exception e) {
        System.out.println("Can't find menu information file. System exits...");
        System.exit(1);
     }

     try {
        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("1d") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage,
                                       -1, level, index, -1));
             } else if (level == 1) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage,
                                       mkey, level, index, root0));
             } else if (level == 2) {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage, 
                                       mkey, level, index, root1));
             } else {
               menuVec.addElement(new MenuInfo(mname, mclass, mpackage, 
                                       mkey, level, index, root2));
             }
          } 

	}
     } catch (Exception e) {
        System.out.println("Corrupted menu information(1D), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

     ////////////////////////////////////////////////
     // parse 2d menu information
     ////////////////////////////////////////////////
     try {

        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("2d") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec2D.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        -1, level, index, -1));
             } else if (level == 1) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        mkey, level, index, root0));
             } else if (level == 2) {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root1));
             } else {
               menuVec2D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root2));
             }
          } 

	}

     } catch (Exception e) {
        System.out.println("Corrupted menu information(2D), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

     ////////////////////////////////////////////////
     // parse 4d menu information
     ////////////////////////////////////////////////
     try {

        while (true) {
          String readed = in.readLine().toLowerCase();
          if ((readed.indexOf("begin") >= 0) && (readed.indexOf("vector") >= 0))
             break;
        }
 
        while (true) {
          line = in.readLine().trim();
                    
          if (line.equalsIgnoreCase("end")) 
             break;
 
          tmp = line.toLowerCase();
          if (tmp.indexOf("<menu>") >= 0) {
             menuName = line.substring(6);
             menuName = menuName.trim();
             index++;

             if (level == 0) {
                root0 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, -1));
             } else if (level == 1) {
                root1 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, root0));
             } else {
                root2 = index;
                menuVec4D.addElement(new MenuInfo(menuName, level, index, root1));
             }

             level++;
          } else if (tmp.indexOf("</menu>") >= 0) {
             level--;
          } else if (tmp.indexOf("<item>") >= 0) {
             index++;
             menuName = line.substring(6);

             mkey = -1;
             StringTokenizer st = new StringTokenizer(menuName, ",");
             while(st.hasMoreTokens()) {
                tmp = st.nextToken().trim();
                                
                int pos = tmp.indexOf("=");
                if (tmp.indexOf("NAME") >= 0) {
                   mname = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("CLASS") >= 0) {
                   mclass = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("PACKAGE") >= 0) {
                   mpackage = tmp.substring(pos+2, tmp.length()-1);
                } else if (tmp.indexOf("SHORTKEY") >= 0) {
                   mkey = Integer.valueOf(tmp.substring(pos+1, tmp.length())).intValue();
                }
             }

             if (level == 0) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        -1, level, index, -1));
             } else if (level == 1) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage,
                                        mkey, level, index, root0));
             } else if (level == 2) {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root1));
             } else {
               menuVec4D.addElement(new MenuInfo(mname, mclass, mpackage, 
                                        mkey, level, index, root2));
             }
          } 

	}

     } catch (Exception e) {
        System.out.println("Corrupted menu information(Vector), system exits ...");
        e.printStackTrace();
        System.exit(1);
     }

  }

} // end of class
