package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * TDPlane.java
 * Define point data in each time
 * @version 1.0     10/15/97
 * @author Byeongseob Ki
 */

public abstract class TDPlane extends TimeData {

  // x, y min and max
  protected float xmin = Float.MAX_VALUE;
  protected float ymin = Float.MAX_VALUE;
  protected float cmin = Float.MAX_VALUE;
  
  protected float xmax = Float.MIN_VALUE;
  protected float ymax = Float.MIN_VALUE;
  protected float cmax = Float.MIN_VALUE;

  // global x, y min and max
  protected float globalxmin = Float.MAX_VALUE;
  protected float globalymin = Float.MAX_VALUE;
  protected float globalxmax = Float.MIN_VALUE;
  protected float globalymax = Float.MIN_VALUE;


  // x and y arrays
  protected float[] xArr;
  protected float[] yArr;
  protected float[] cArr;

  protected boolean bb = false;
 
  protected int dataType;

  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   */
  public TDPlane(float time, int noOfPoints, float[] xArr, float[] yArr) {
     this.time = time;
     this.noOfPoints = noOfPoints;
     this.xArr = xArr;
     this.yArr = yArr;

     for (int i=0; i< noOfPoints; i++) {
        if (xmin > xArr[i]) xmin = xArr[i];
        if (xmax < xArr[i]) xmax = xArr[i];
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
     }
  }


  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   * @param cArr color value array
   */
  public TDPlane(float time, int noOfPoints, 
                               float[] xArr, float[] yArr, float[] cArr) {
     this.time = time;
     this.noOfPoints = noOfPoints;
     this.xArr = xArr;
     this.yArr = yArr;
     this.cArr = cArr;

     for (int i=0; i< noOfPoints; i++) {
        if (xmin > xArr[i]) xmin = xArr[i];
        if (xmax < xArr[i]) xmax = xArr[i];
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
        if (cmin > cArr[i]) cmin = cArr[i];
        if (cmax < cArr[i]) cmax = cArr[i];
     }
  }



  /**
   * constructor (by java_bbser)
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xmin x min value
   * @param xmin x max value
   * @param yArr y value array
   */
  public TDPlane(float time, int noOfPoints, 
                     float xmin, float xmax, float[] yArr) {
     this.time = time;
     this.noOfPoints = noOfPoints;
     this.yArr = yArr;
     bb = true;
     xArr = new float[yArr.length];

     float xinc = (xmax - xmin) / (float)(noOfPoints-1);
     for (int i=0; i< noOfPoints; i++) {
        xArr[i] = xmin + xinc * (float)i;
     }

     this.xmin = xmin;
     this.xmax = xmax;
     for (int i=0; i< noOfPoints; i++) {
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
     }
  }


  /**
   * constructor (by java_bbser)
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xmin x min value
   * @param xmin x max value
   * @param yArr y value array
   * @param cArr c value array
   */
  public TDPlane(float time, int noOfPoints, 
                     float xmin, float xmax, float[] yArr, float[] cArr) {
     this.time = time;
     this.noOfPoints = noOfPoints;
     this.yArr = yArr;
     this.cArr = cArr;
     bb = true;
     xArr = new float[yArr.length];

     float xinc = (xmax - xmin) / (float)(noOfPoints-1);
     for (int i=0; i< noOfPoints; i++) {
        xArr[i] = xmin + xinc * (float)i;
     }

     this.xmin = xmin;
     this.xmax = xmax;
     for (int i=0; i< noOfPoints; i++) {
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
        if (cmin > cArr[i]) cmin = cArr[i];
        if (cmax < cArr[i]) cmax = cArr[i];
     }
  }

  /** 
   * sending data by using DataOutputStream
   */
  public abstract void sendingData(DataOutputStream out);


  /**
   * set global xmin, xmax, ymin and ymax values
   * @param globalxmin global xmin
   * @param globalxmax global xmax
   * @param globalymin global ymin
   * @param globalymax global ymax
   */
  public void setGlobalValues(float globalxmin, float globalxmax, 
                               float globalymin, float globalymax) {
     this.globalxmin = globalxmin;
     this.globalymin = globalymin;
     this.globalxmax = globalxmax;
     this.globalymax = globalymax;
  }


  /**
   * return xArr
   */
  public float[] getxArr() {
     int sz = xArr.length;
     float[] ret = new float[sz];
     System.arraycopy(xArr, 0, ret, 0, sz);

     return ret;
  }

  /**
   * return yArr
   */
  public float[] getyArr() {
     int sz = yArr.length;
     float[] ret = new float[sz];
     System.arraycopy(yArr, 0, ret, 0, sz);

     return ret;
  }

  /**
   * return cArr
   */
  public float[] getcArr() {
     int sz = cArr.length;
     float[] ret = new float[sz];
     System.arraycopy(cArr, 0, ret, 0, sz);

     return ret;
  }


  /**
   * return xmin
   */
  public float getxmin() {
     return xmin;
  }

  /**
   * return xmax
   */
  public float getxmax() {
     return xmax;
  }

  /**
   * return ymin
   */
  public float getymin() {
     return ymin;
  }

  /**
   * return ymax
   */
  public float getymax() {
     return ymax;
  }

  /**
   * return globalxmin
   */
  public float getglobalxmin() {
     return globalxmin;
  }

  /**
   * return cmin
   */
  public float getcmin() {
     return cmin;
  }


  /**
   * return cmax
   */
  public float getcmax() {
     return cmax;
  }


  /**
   * return globalxmax
   */
  public float getglobalxmax() {
     return globalxmax;
  }

  /**
   * return globalymin
   */
  public float getglobalymin() {
     return globalymin;
  }

  /**
   * return globalymax
   */
  public float getglobalymax() {
     return globalymax;
  }


  /**
   * Return the data point that is closest to the parsed (x,y) position
   * @param x 
   * @param y (x,y) position in data space. 
   * @return array containing the closest data point.
   */
  public float[] getClosestPoint(float x, float y) {
     float point[] = {0.0f, 0.0f, 0.0f};
     int i;
     float xdiff, ydiff, dist2;

     xdiff = xArr[0] - x;
     ydiff = yArr[0] - y;
              
     point[0] = xArr[0];
     point[1] = yArr[0];
     point[2] = xdiff*xdiff + ydiff*ydiff;
            
     for(i=1; i<noOfPoints; i++) {

        xdiff = xArr[i] - x;
        ydiff = yArr[i] - y;

        dist2 = xdiff*xdiff + ydiff*ydiff;

        if(dist2 < point[2]) {
           point[0] = xArr[i];
           point[1] = yArr[i];
           point[2] = dist2;
        }
     }
     return point;
   }


  /**
   * return data type
   */
  public int getDataType() {
     return dataType;
  }

} // end of class




