package sv.kernel;

import java.awt.*;
import java.io.*;
import java.util.*;

import sv.util.*;

/** 
 * TimeData1D.java
 * Define point data in each time
 * @version 1.0     10/15/97
 * @author Byeongseob Ki
 */

public class TimeData1D extends TDPlane {

  /**
   * to hold color values
   */
  public Color[] colorArr;


  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   */
  public TimeData1D(float time, int noOfPoints, float[] xArr, float[] yArr) {
     super(time, noOfPoints, xArr, yArr);
     dataType = DSPlane.NORMAL;
  }


  /**
   * constructor
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xArr x value array
   * @param yArr y value array
   * @param cArr color value array
   */
  public TimeData1D(float time, int noOfPoints, float[] xArr, float[] yArr, float[] cArr) {
     super(time, noOfPoints, xArr, yArr, cArr);
     dataType = DSPlane.NORMAL;
  }


  /**
   * constructor (by java_bbser)
   * @param time time value
   * @param noOfPoints the number of points of this data
   * @param xmin x min value
   * @param xmin x max value
   * @param yArr y value array
   */
  public TimeData1D(float time, int noOfPoints, 
                     float xmin, float xmax, float[] yArr) {
     super(time, noOfPoints, xmin, xmax, yArr);
     dataType = DSPlane.NORMAL;
  }


  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) {

    try {
       // send time
       out.writeFloat(time);

       // send number of points in given time
       out.writeInt(noOfPoints);
     
       // send cArr
       if (cArr == null) {
          out.writeByte(0);
       } else {
          out.writeByte(1);
          for (int i=0; i<noOfPoints; i++) 
             out.writeFloat(cArr[i]);
       }

       // send reserved for future use (2 items e.g, error values)
       out.writeInt(0);
       out.writeInt(0);

       // send x and y arrays
       if (bb) {
         out.writeByte(1);     // bb is true... send boundbox first
         out.writeFloat(xmin);
         out.writeFloat(xmax);

         for (int i=0; i<noOfPoints; i++) 
           out.writeFloat(yArr[i]);

       } else {
         out.writeByte(0);      // bb is false

         for (int i=0; i<noOfPoints; i++) {
            out.writeFloat(xArr[i]);
            out.writeFloat(yArr[i]);
         }
       }

    } catch (Exception e) {
       System.out.println("Data sending error(TimeData1D).");
       //e.printStackTrace();
    } 
  }

  /**
   * update data 
   * @param data TimeData1D to be appended or updated
   */
  public void updateData(TimeData1D data, boolean parallel) {

     float[] newxArr = data.getxArr();
     float[] newyArr = data.getyArr();

     float[] savedxArr = new float[noOfPoints];
     float[] savedyArr = new float[noOfPoints];

     System.arraycopy(xArr, 0, savedxArr, 0, noOfPoints);
     System.arraycopy(yArr, 0, savedyArr, 0, noOfPoints);

     int[] duplicate = new int[data.getnoOfPoints()];

     for (int i=0; i<duplicate.length; i++) 
        duplicate[i] = 0;

     int saved = noOfPoints;
     int index;
     if (parallel) {
        for (int i=0; i<noOfPoints; i++) {
           for (int j=0; j<newxArr.length; j++) {
              if (xArr[i] == newxArr[j]) {
                savedyArr[i] = newyArr[j];
                duplicate[j] = 1;
             }
           }
        }

        index = 0; 
        for (int i=0; i<data.getnoOfPoints(); i++) 
           if (duplicate[i] == 0)
              index++;

        noOfPoints = noOfPoints + index;
     } else {
        noOfPoints = noOfPoints + data.getnoOfPoints();
     }

     xArr = new float[noOfPoints];
     yArr = new float[noOfPoints];

     System.arraycopy(savedxArr, 0, xArr, 0, saved);
     System.arraycopy(savedyArr, 0, yArr, 0, saved);

     index = saved;
     for (int i=0; i<data.getnoOfPoints(); i++) {
        if (duplicate[i] == 0) {
           xArr[index] = newxArr[i];
           yArr[index] = newyArr[i];
           index++;
        }
     }

if ((noOfPoints != yArr.length) || (noOfPoints != xArr.length)) { 
System.out.println("Update TimeData1D failed ......");
}

     xmin = 1e10f;
     ymin = 1e10f;
     xmax = -1e10f;
     ymax = -1e10f;

     for (int i=0; i< noOfPoints; i++) {
        if (xmin > xArr[i]) xmin = xArr[i];
        if (xmax < xArr[i]) xmax = xArr[i];
        if (ymin > yArr[i]) ymin = yArr[i];
        if (ymax < yArr[i]) ymax = yArr[i];
     }
     bb = false;
  }


  /**
   * calc AMR colors
   */
  public void calcAMRColor(int colorModel, 
                   float[] rcolors, float[] gcolors, float[] bcolors,
                   float[] hcolors, float[] scolors, float[] lcolors) {
     float[] dx = new float[noOfPoints-1];
     for (int i=0; i<noOfPoints-1; i++) 
        dx[i] = xArr[i+1] - xArr[i];

     float[] sorteddx = new float[noOfPoints-1];
     colorArr = new Color[noOfPoints];

     for (int i=0; i<noOfPoints-1; i++) 
        sorteddx[i] = dx[i];

     Sorter sorter = new Sorter();
     sorter.QSortFloat(sorteddx, 0, noOfPoints-2);

     int index = 0;
     float[] codes = new float[noOfPoints-1];
     codes[0] = sorteddx[0];

     for (int i=1; i<noOfPoints-1; i++) {
        if (codes[index] != sorteddx[i]) {
           index++;
           codes[index] = sorteddx[i];
        }  
     }
     sorteddx = null;

     Color c;

     float color_factor = codes[index] - codes[0];
     for (int i=0; i<noOfPoints-1; i++) {
        float dummy =  codes[index]-dx[i];
        int val = 
          (int)((codes[index]-dx[i])/color_factor * 100f);

          if (colorModel == GFrame.RGBCOLORMODEL) 
               c = new Color((int)(rcolors[val] * 255f), 
                             (int)(gcolors[val] * 255f), 
                             (int)(bcolors[val] * 255f));
          else 
               c = Color.getHSBColor((float)hcolors[val], 
                                     (float)scolors[val],
                                     (float)lcolors[val]);
 
        colorArr[i] = c;  

     }
     colorArr[noOfPoints-1] = colorArr[0];
  }


 /**
   * calc colors by color info
   */
  public void calcColorByInfo(int colorModel, 
                   float[] rcolors, float[] gcolors, float[] bcolors,
                   float[] hcolors, float[] scolors, float[] lcolors) {

     if (cArr == null) {
//       System.out.println("No color information in this time: " + time);
       return;
     }

     colorArr = new Color[noOfPoints];

     Color c;
     for (int i=0; i<noOfPoints; i++) {

        int val = (int)((cArr[i]-cmin) / (cmax-cmin) * 100f);

        if (colorModel == GFrame.RGBCOLORMODEL) 
             c = new Color((int)(rcolors[val] * 255f), 
                           (int)(gcolors[val] * 255f), 
                           (int)(bcolors[val] * 255f));
        else 
             c = Color.getHSBColor((float)hcolors[val], 
                                   (float)scolors[val],
                                   (float)lcolors[val]);
 
        colorArr[i] = c;  
     }
  }

} // end of class


