package sv.kernel;

import java.io.*;

/** 
 * TimeDataVec.java
 * Define vector set for vector plot
 * @version 1.0   10/1/97
 * @author Byeongseob Ki
 */

public class TimeDataVec extends TDPlane {

  // arrays
  private float[] rArr;     // r vals
  private float[] tArr;     // angle vals

  public TimeDataVec(float time, float[] xArr, float[] yArr,
                                float[] rArr, float[] tArr) {
     super(time, xArr.length, xArr, yArr);
     dataType = DSPlane.VECTOR;

     this.rArr = rArr;
     this.tArr = tArr;
  }

  /** 
   * sending data by using DataOutputStream
   */
  public void sendingData(DataOutputStream out) {

    try {
       // send time
       out.writeFloat(time);

       // send number of points in given time
       out.writeInt(noOfPoints);
     
       // send cArr
       if (cArr == null) {
          out.writeByte(0);
       } else {
          out.writeByte(1);
          for (int i=0; i<noOfPoints; i++) 
             out.writeFloat(cArr[i]);
       }

       // send reserved for future use (2 items)
       out.writeInt(0);
       out.writeInt(0);

       // send x and y arrays
       if (bb) {
         out.writeByte(1);     // bb is true... send boundbox first
         out.writeFloat(xmin);
         out.writeFloat(xmax);

         for (int i=0; i<noOfPoints; i++) {
           out.writeFloat(yArr[i]);
           out.writeFloat(rArr[i]);
           out.writeFloat(tArr[i]);
         }

       } else {
         out.writeByte(0);      // bb is false

         for (int i=0; i<noOfPoints; i++) {
            out.writeFloat(xArr[i]);
            out.writeFloat(yArr[i]);
            out.writeFloat(rArr[i]);
            out.writeFloat(tArr[i]);
         }
       }

    } catch (Exception e) {
       System.out.println("Data sending error(TimeDataVec).");
       //e.printStackTrace();
    } 
  }


  /**
   * update data 
   * @param data TimeDataVec to be appended or updated
   */
  public void updateData(TimeDataVec data, boolean parallel) {
     System.out.println("Update time data function is not implemented.");
  }


  /**
   * return rArr
   */
  public float[] getrArr() {
     int sz = rArr.length;
     float[] ret = new float[sz];
     System.arraycopy(rArr, 0, ret, 0, sz);

     return ret;
  }

  /**
   * return tArr
   */
  public float[] gettArr() {
     int sz = tArr.length;
     float[] ret = new float[sz];
     System.arraycopy(tArr, 0, ret, 0, sz);

     return ret;
  }


} // end of class

