package sv.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.util.*;
import sv.kernel.*;

/** 
 * ColorMapDialog.java
 * Color map input
 * @version 1.0
 * @author Byeongseob Ki
 */
public class ColorMapDialog extends Dialog 
                 implements ActionListener, ItemListener {

  ColorMapCanvas colormap;
  RGBChooser rgbchooser;
  HSBChooser hsbchooser;

  Choice model;
  
  Checkbox rb, gb, bb, hb, sb, lb;
  int selected = 0;

  Panel cards;

  int colorModel  = 1;
  NewtonMethod rspline, gspline, bspline, hspline, sspline, lspline;    

  float[] rcolors, gcolors, bcolors, hcolors, scolors, lcolors;

  private float[] rxctls, ryctls, gxctls, gyctls, bxctls, byctls, 
                  hxctls, hyctls, sxctls, syctls, lxctls, lyctls;

  private GFrame gframe;

  /** 
   * constructor
   * @param gframe parent class
   */
  public ColorMapDialog(Frame gframe) {
     super(gframe, "Color Map Chooser", false);
 
     this.gframe = (GFrame)gframe;

     this.colorModel = ((GFrame)gframe).colorModel;
     this.rspline = ((GFrame)gframe).rspline;
     this.gspline = ((GFrame)gframe).gspline;
     this.bspline = ((GFrame)gframe).bspline;
     this.hspline = ((GFrame)gframe).hspline;
     this.sspline = ((GFrame)gframe).sspline;
     this.lspline = ((GFrame)gframe).lspline;

     this.rcolors = ((GFrame)gframe).rcolors;
     this.gcolors = ((GFrame)gframe).gcolors;
     this.bcolors = ((GFrame)gframe).bcolors;
     this.hcolors = ((GFrame)gframe).hcolors;
     this.scolors = ((GFrame)gframe).scolors;
     this.lcolors = ((GFrame)gframe).lcolors;

     rxctls = new float[rspline.xcontrols.length];
     System.arraycopy(rspline.xcontrols, 0, rxctls, 0, rspline.xcontrols.length);

     ryctls = new float[rspline.ycontrols.length];
     System.arraycopy(rspline.ycontrols, 0, ryctls, 0, rspline.ycontrols.length);
     gxctls = new float[gspline.xcontrols.length];
     System.arraycopy(gspline.xcontrols, 0, gxctls, 0, gspline.xcontrols.length);

     gyctls = new float[gspline.ycontrols.length];
     System.arraycopy(gspline.ycontrols, 0, gyctls, 0, gspline.ycontrols.length);     
   
     bxctls = new float[bspline.xcontrols.length];
     System.arraycopy(bspline.xcontrols, 0, bxctls, 0, bspline.xcontrols.length);

     byctls = new float[bspline.ycontrols.length];
     System.arraycopy(bspline.ycontrols, 0, byctls, 0, bspline.ycontrols.length);     

     hxctls = new float[hspline.xcontrols.length];
     System.arraycopy(hspline.xcontrols, 0, hxctls, 0, hspline.xcontrols.length);

     hyctls = new float[hspline.ycontrols.length];
     System.arraycopy(hspline.ycontrols, 0, hyctls, 0, hspline.ycontrols.length);     

     sxctls = new float[sspline.xcontrols.length];
     System.arraycopy(sspline.xcontrols, 0, sxctls, 0, sspline.xcontrols.length);

     syctls = new float[sspline.ycontrols.length];
     System.arraycopy(sspline.ycontrols, 0, syctls, 0, sspline.ycontrols.length);     
     lxctls = new float[lspline.xcontrols.length];
     System.arraycopy(lspline.xcontrols, 0, lxctls, 0, lspline.xcontrols.length);

     lyctls = new float[lspline.ycontrols.length];
     System.arraycopy(lspline.ycontrols, 0, lyctls, 0, lspline.ycontrols.length);     


     // make GUI components
     makeGUI("Color Map");
     pack();
  }

  /**
   * overrided method
   */
  public void actionPerformed(ActionEvent evt) {
     String command = evt.getActionCommand();

     if (command.equals("Apply ")) {  
        gframe.changeColorMap(getColorInfo());

     } else if (command.equals("Reset ")) {   
         rspline = new NewtonMethod(rxctls, ryctls);
         gspline = new NewtonMethod(gxctls, gyctls);
         bspline = new NewtonMethod(bxctls, byctls);
         hspline = new NewtonMethod(hxctls, hyctls);
         sspline = new NewtonMethod(sxctls, syctls);
         lspline = new NewtonMethod(lxctls, lyctls);

         rgbchooser.rspline = rspline;
         rgbchooser.gspline = gspline;
         rgbchooser.bspline = bspline;
         hsbchooser.hspline = hspline;
         hsbchooser.sspline = sspline;
         hsbchooser.lspline = lspline;
         rgbchooser.repaint();
         hsbchooser.repaint();
         colormap.repaint();

     } else if (command.equals("Quit ")) {       
        this.dispose();
     } else if (command.equals("Add Point")) {    
        if (colorModel == 1) 
           rgbchooser.addPoint();
        else
           hsbchooser.addPoint();
     } else if (command.equals("Remove Point")) {    
        if (colorModel == 1)
           rgbchooser.removePoint();
        else
           hsbchooser.removePoint();
     } else if (command.equals("Save as...")) {    

        FileDialog openFileDialog = new FileDialog(gframe, 
                         "Save color map to file", FileDialog.SAVE);
        openFileDialog.show();

        String fname = openFileDialog.getFile();
        if (fname == null) return;
        fname = openFileDialog.getDirectory() + fname;

        try {
           PrintWriter ow
                = new PrintWriter(new FileWriter(fname.trim()));

           ow.println("r " + rspline.nControls);
           for (int i=0; i<rspline.nControls; i++) 
              ow.println(rspline.xcontrols[i] + " " + rspline.ycontrols[i]);

           ow.println("g " + gspline.nControls);
           for (int i=0; i<gspline.nControls; i++) 
              ow.println(gspline.xcontrols[i] + " " + gspline.ycontrols[i]);

           ow.println("b " + bspline.nControls);
           for (int i=0; i<bspline.nControls; i++) 
              ow.println(bspline.xcontrols[i] + " " + bspline.ycontrols[i]);

           ow.println("h " + hspline.nControls);
           for (int i=0; i<hspline.nControls; i++) 
              ow.println(hspline.xcontrols[i] + " " + hspline.ycontrols[i]);

           ow.println("s " + sspline.nControls);
           for (int i=0; i<sspline.nControls; i++) 
              ow.println(sspline.xcontrols[i] + " " + sspline.ycontrols[i]);

           ow.println("l " + lspline.nControls);
           for (int i=0; i<lspline.nControls; i++) 
              ow.println(lspline.xcontrols[i] + " " + lspline.ycontrols[i]);

           ow.close();
        } catch (Exception e) {
           System.out.println("Color map writing error ...");
        }

     } else if (command.equals("Open ")) {    
        FileDialog openFileDialog = new FileDialog(gframe, 
                         "Load color map file", FileDialog.LOAD);
        openFileDialog.show();

        String fname = openFileDialog.getFile();
        if (fname == null) return;
        fname = openFileDialog.getDirectory() + fname;
        
        doRead(fname);
        rgbchooser.repaint();
        hsbchooser.repaint();
        colormap.repaint();
     }
  }

  private void doRead(String fname) {

     try {
        BufferedReader reader = new BufferedReader(new FileReader(fname));

        int nControls;
        float[] xcontrols, ycontrols;
        String input, type;
        StringTokenizer st;

        for (int tt=0; tt<6; tt++) {
           input = reader.readLine();
           st = new StringTokenizer(input);
           type = st.nextToken().toLowerCase();

           if (type.indexOf("r") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              rspline = new NewtonMethod(xcontrols, ycontrols);
              
           } else if (type.indexOf("g") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              gspline = new NewtonMethod(xcontrols, ycontrols);

           } else if (type.indexOf("b") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              bspline = new NewtonMethod(xcontrols, ycontrols);

           } else if (type.indexOf("h") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              hspline = new NewtonMethod(xcontrols, ycontrols);

           } else if (type.indexOf("s") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              sspline = new NewtonMethod(xcontrols, ycontrols);

           } else if (type.indexOf("l") >= 0) {
              nControls = Integer.valueOf(st.nextToken()).intValue();

              xcontrols = new float[nControls];
              ycontrols = new float[nControls];

              for (int i=0; i<nControls; i++) {
                input = reader.readLine();
		st = new StringTokenizer(input);

                xcontrols[i] = Float.valueOf(st.nextToken()).floatValue();
                ycontrols[i] = Float.valueOf(st.nextToken()).floatValue();
              }
   
              lspline = new NewtonMethod(xcontrols, ycontrols);

           } else {
             System.out.println("Can't read colormap file...");
             reader.close();
             return;
           }
        }
        reader.close();

     } catch (Exception e) {
        e.printStackTrace();
        System.out.println("Error while reading the colormap file ...");
        System.out.println("This may cause undesirable color effects ...");
        return;
     }

     rgbchooser.rspline = rspline;
     rgbchooser.gspline = gspline;
     rgbchooser.bspline = bspline;
     hsbchooser.hspline = hspline;
     hsbchooser.sspline = sspline;
     hsbchooser.lspline = lspline;
  }

  /**
   * overrided method (for list)
   */
  public void itemStateChanged(ItemEvent e) {

     String itemName  = e.getItem().toString();

     if (itemName.equals("RGB model")) {
        colorModel = 1;
        ((CardLayout)cards.getLayout()).show(cards,"RGB model");
        colormap.repaint();
     } else if (itemName.equals("HSB model")) {
        colorModel = 2;
        ((CardLayout)cards.getLayout()).show(cards,"HSB model");
        colormap.repaint();
     } else if (e.getStateChange() == ItemEvent.SELECTED) {
        if (itemName.equals("r")) 
           selected = 1;
        else if (itemName.equals("g")) 
           selected = 2;
        else if (itemName.equals("b")) 
           selected = 3;
        else if (itemName.equals("h")) 
           selected = 1;
        else if (itemName.equals("s")) 
           selected = 2;
        else if (itemName.equals("l")) 
           selected = 3;
     }
  }


  public void rightAction() {
    if (colorModel == 1) 
       rgbchooser.rightAction();
    else 
       hsbchooser.rightAction();
  }

  public void leftAction() {
    if (colorModel == 1) 
       rgbchooser.leftAction();
    else 
       hsbchooser.leftAction();
  }

  public void upAction() {
    if (colorModel == 1) 
       rgbchooser.upAction();
    else 
       hsbchooser.upAction();
  }

  public void downAction() {
    if (colorModel == 1) 
       rgbchooser.downAction();
    else 
       hsbchooser.downAction();
  }


  // return color info
  private String[] getColorInfo() {
     String[] str = new String[2];
     str[0] = String.valueOf(colorModel);
     str[1] = "";

     for (int i=0; i<101; i++) {
        str[1] += Float.toString(rcolors[i]) + ";";
        str[1] += Float.toString(gcolors[i]) + ";";
        str[1] += Float.toString(bcolors[i]) + ";";
        str[1] += Float.toString(hcolors[i]) + ";";
        str[1] += Float.toString(scolors[i]) + ";";
        str[1] += Float.toString(lcolors[i]) + ";";
     }
     return str;
  }


  // make GUI of this class
  private void makeGUI(String title) {

     Button btn;

     setLayout(new BorderLayout());

     Panel centerPanel = new Panel();
     centerPanel.setLayout(new GridLayout(2, 1));

     JPanel p = new JPanel();	
     p.setLayout(new BorderLayout());
   
     Panel p1 = new Panel();
     p1.setLayout(new GridLayout(1, 3));
     p1.add(btn = new Button("Add Point"));
     btn.addActionListener(this);

     p1.add(btn = new Button("Remove Point"));
     btn.addActionListener(this);

     p1.add(model = new Choice());
     model.addItem("RGB model");
     model.addItem("HSB model");
     model.addItemListener(this);
     if (colorModel == 1)
       model.select("RGB model");
     else
       model.select("HSB model");

     JPanel pp = new JPanel();
     pp.setBorder(new TitledBorder("Result: "));	
     pp.setLayout(new BorderLayout());
     pp.add("Center", colormap = new ColorMapCanvas(this));

     p.add("Center", pp);
     p.add("South", p1);
     centerPanel.add(p);

     cards = new Panel();
     cards.setLayout(new CardLayout());

     p = new JPanel();
     p.setBorder(new TitledBorder("RGB Chooser: "));
     p.setLayout(new BorderLayout());

     p.add("Center", rgbchooser = new RGBChooser(this));

     Panel p2 = new Panel();
     p2.setLayout(new GridLayout(3, 1));
     CheckboxGroup group1 = new CheckboxGroup();
     p2.add(rb = new Checkbox("r", group1, false));
     rb.addItemListener(this);
     rb.setBackground(Color.red);
     p2.add(gb = new Checkbox("g", group1, false));
     gb.addItemListener(this);
     gb.setBackground(Color.green);
     p2.add(bb = new Checkbox("b", group1, false));
     bb.addItemListener(this);
     bb.setBackground(Color.blue);
     p.add("East", p2);

     rgbchooser.setSize(280, 120);

     cards.add("RGB model",  p);


     p = new JPanel();
     p.setBorder(new TitledBorder("HSB Chooser: "));	
     p.setLayout(new BorderLayout());

     p.add("Center", hsbchooser = new HSBChooser(this));

     p2 = new Panel();
     p2.setLayout(new GridLayout(3, 1));
     CheckboxGroup group2 = new CheckboxGroup();
     p2.add(hb = new Checkbox("h", group2, false));
     hb.addItemListener(this);
     hb.setBackground(Color.magenta);
     p2.add(sb = new Checkbox("s", group2, false));
     sb.addItemListener(this);
     sb.setBackground(Color.pink);
     p2.add(lb = new Checkbox("l", group2, false));
     lb.addItemListener(this);
     lb.setBackground(Color.cyan);
     p.add("East", p2);

     hsbchooser.setSize(280, 120);
     cards.add("HSB model",  p);

     centerPanel.add(cards);

     Panel dummy = new Panel();      // for buttons
     dummy.setLayout(new FlowLayout(FlowLayout.RIGHT));

     btn = new Button("Open ");
     btn.addActionListener(this);
     dummy.add(btn);

     btn = new Button("Save as...");
     btn.addActionListener(this);
     dummy.add(btn);

     btn = new Button("Apply ");
     btn.addActionListener(this);
     dummy.add(btn);

     btn = new Button("Reset ");
     btn.addActionListener(this);
     dummy.add(btn);

     btn = new Button("Quit ");
     btn.addActionListener(this);
     dummy.add(btn);

     add("Center", centerPanel);
     add("South", dummy);
  }
 
}

///////////////////////////////////////////////////////////////////
class ColorMapCanvas extends Canvas {
  Polygon leftPoly, rightPoly;
  Polygon upPoly, downPoly;

  int[] leftpolyX;
  int[] leftpolyY;
  int[] rightpolyX;
  int[] rightpolyY;
  int[] uppolyX;
  int[] uppolyY;
  int[] downpolyX;
  int[] downpolyY;
  
  Graphics offScrGc;
  Image offScrImage;

  boolean leftBtnPressed = false;
  boolean rightBtnPressed = false;
  boolean upBtnPressed = false;
  boolean downBtnPressed = false;

  int wid = 100;
  int hgt = 100;

  ColorMapDialog parent;

  public ColorMapCanvas(ColorMapDialog parent) {
     this.parent = parent;

     leftpolyX = new int[3];
     leftpolyY = new int[3];
     rightpolyX = new int[3];
     rightpolyY = new int[3];
     uppolyX = new int[3];
     uppolyY = new int[3];
     downpolyX = new int[3];
     downpolyY = new int[3];
     enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }


  void changePoly() {
     Dimension d = getSize();
     
     // i don't need mainPoly, so removed ...

     leftpolyX[0] = d.width-100;
     leftpolyX[1] = d.width-65;
     leftpolyX[2] = d.width-65; 
     leftpolyY[0] = 70;
     leftpolyY[1] = 55;
     leftpolyY[2] = 85; 
     leftPoly = new Polygon(leftpolyX, leftpolyY, 3);
 
     rightpolyX[0] = d.width-10;
     rightpolyX[1] = d.width-45;
     rightpolyX[2] = d.width-45; 
     rightpolyY[0] = 70;
     rightpolyY[1] = 55;
     rightpolyY[2] = 85; 
     rightPoly = new Polygon(rightpolyX, rightpolyY, 3);

     uppolyX[0] = d.width-220;
     uppolyX[1] = d.width-185;
     uppolyX[2] = d.width-185; 
     uppolyY[0] = 70;
     uppolyY[1] = 55;
     uppolyY[2] = 85; 
     upPoly = new Polygon(uppolyX, uppolyY, 3);
 
     downpolyX[0] = d.width-130;
     downpolyX[1] = d.width-165;
     downpolyX[2] = d.width-165; 
     downpolyY[0] = 70;
     downpolyY[1] = 55;
     downpolyY[2] = 85; 
     downPoly = new Polygon(downpolyX, downpolyY, 3);
  }

  public void paint(Graphics g) {
     Dimension d = getSize();
 
     if ((offScrImage == null) || 
          (d.width != wid) || (d.height != hgt)) {

       offScrImage = createImage(d.width, d.height);
       offScrGc = offScrImage.getGraphics();
       wid = d.width;
       hgt = d.height;
       changePoly();
    }

    offScrGc.setColor(Color.lightGray);
    offScrGc.fillRect(0, 0, d.width, d.height);

    offScrGc.setColor(Color.blue);
    offScrGc.fillPolygon(leftPoly);
    offScrGc.fillPolygon(rightPoly);

    offScrGc.setColor(Color.red);
    offScrGc.fillPolygon(upPoly);
    offScrGc.fillPolygon(downPoly);

    // Draw left button(only used internal)
    if (leftBtnPressed)
      offScrGc.setColor(Color.darkGray);
    else
      offScrGc.setColor(Color.white);
    offScrGc.drawLine(leftpolyX[0], leftpolyY[0], leftpolyX[1], leftpolyY[1]);

    if (leftBtnPressed)
      offScrGc.setColor(Color.white);
    else
      offScrGc.setColor(Color.darkGray);
    offScrGc.drawLine(leftpolyX[0], leftpolyY[0], leftpolyX[2], leftpolyY[2]);
    offScrGc.drawLine(leftpolyX[1], leftpolyY[1], leftpolyX[2], leftpolyY[2]); 

    // Draw right button(only used internal)
    if (rightBtnPressed)
      offScrGc.setColor(Color.darkGray);
    else
      offScrGc.setColor(Color.white);
    offScrGc.drawLine(rightpolyX[0], rightpolyY[0], rightpolyX[1], rightpolyY[1]);
    offScrGc.drawLine(rightpolyX[1], rightpolyY[1], rightpolyX[2], rightpolyY[2]);

    if (rightBtnPressed)
      offScrGc.setColor(Color.white);
    else
      offScrGc.setColor(Color.darkGray);
    offScrGc.drawLine(rightpolyX[0], rightpolyY[0], rightpolyX[2], rightpolyY[2]);


    // Draw up button(only used internal)
    if (upBtnPressed)
      offScrGc.setColor(Color.darkGray);
    else
      offScrGc.setColor(Color.white);
    offScrGc.drawLine(uppolyX[0], uppolyY[0], uppolyX[1], uppolyY[1]);

    if (upBtnPressed)
      offScrGc.setColor(Color.white);
    else
      offScrGc.setColor(Color.darkGray);
    offScrGc.drawLine(uppolyX[0], uppolyY[0], uppolyX[2], uppolyY[2]);
    offScrGc.drawLine(uppolyX[1], uppolyY[1], uppolyX[2], uppolyY[2]); 

    // Draw down button(only used internal)
    if (downBtnPressed)
      offScrGc.setColor(Color.darkGray);
    else
      offScrGc.setColor(Color.white);
    offScrGc.drawLine(downpolyX[0], downpolyY[0], downpolyX[1], downpolyY[1]);
    offScrGc.drawLine(downpolyX[1], downpolyY[1], downpolyX[2], downpolyY[2]);


    if (downBtnPressed)
      offScrGc.setColor(Color.white);
    else
      offScrGc.setColor(Color.darkGray);
    offScrGc.drawLine(downpolyX[0], downpolyY[0], downpolyX[2], downpolyY[2]);


    offScrGc.setColor(Color.black);
    offScrGc.drawString("min", 10, 15);
    offScrGc.drawString("max", d.width-35, 15);


    float step = (d.width-20.0f)/100.0f;

    int prev = 10;
    int next;
    for (int i=0; i<=100; i++) {
       if (parent.colorModel == 1) {
          offScrGc.setColor(
                 new Color((int)(parent.rcolors[i] * 255f), 
                           (int)(parent.gcolors[i] * 255f), 
                           (int)(parent.bcolors[i] * 255f)));
       } else {
          offScrGc.setColor(
                Color.getHSBColor((float)parent.hcolors[i], 
                                  (float)parent.scolors[i],
                                  (float)parent.lcolors[i]));
       }

       next =  (int)((i+1)*step+10);
       offScrGc.fillRect(prev, 20, next-prev, 25);
       prev = next;
    }  

    g.drawImage(offScrImage, 0, 0, this);
  }


  // overrided function
  public void processMouseEvent(MouseEvent e) {
     Dimension d = getSize();
     int x = e.getX();
     int y = e.getY();

     if (e.getID() == MouseEvent.MOUSE_PRESSED) { 
       if (leftPoly.contains(x, y)) {
          leftBtnPressed = true;  
          repaint();
          parent.leftAction();          

       } else if (rightPoly.contains(x, y)) {
          rightBtnPressed = true;     
          repaint();
          parent.rightAction();    

       } else if (upPoly.contains(x, y)) {
          upBtnPressed = true;       
          repaint();
          parent.upAction();    

       } else if (downPoly.contains(x, y)) {
          downBtnPressed = true;     
          repaint();
          parent.downAction();    
       }

     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) { 
       if (leftBtnPressed) {
          leftBtnPressed = false;
          repaint();
          return;

       } else if (rightBtnPressed) {
          rightBtnPressed = false;
          repaint();
          return;
       } else if (upBtnPressed) {
          upBtnPressed = false;
          repaint();
          return;

       } else if (downBtnPressed) {
          downBtnPressed = false;
          repaint();
          return;

       }
     }
     super.processMouseEvent(e);
  }




  public void update(Graphics g) {
     paint(g);
  }
}  // end of class

//////////////////////////////////////////////////////////////

class RGBChooser extends Canvas {
  
  Graphics offScrGc;
  Image offScrImage;

  int wid = 100;
  int hgt = 100;

  ColorMapDialog parent;

  NewtonMethod rspline, gspline, bspline;     

  int selected;            // selected control points
  int prevx, prevy;        // mouse click

  boolean addMode = false;
  boolean removeMode = false;

  public RGBChooser(ColorMapDialog parent) {
     this.parent = parent;

     rspline = parent.rspline;
     gspline = parent.gspline;
     bspline = parent.bspline;

     enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }


  public void paint(Graphics g) {
     Dimension d = getSize();
 
     if ((offScrImage == null) || 
          (d.width != wid) || (d.height != hgt)) {

       offScrImage = createImage(d.width, d.height);
       offScrGc = offScrImage.getGraphics();
       wid = d.width;
       hgt = d.height;
    }

    offScrGc.setColor(Color.white);
    offScrGc.fillRect(0, 0, d.width, d.height);

    offScrGc.setColor(Color.black);
    if (addMode || removeMode) {
       if (parent.selected < 1) 
          offScrGc.drawString("Select color first.", 15, 25);
       else
          offScrGc.drawString("Click on the canvas.", 15, 25);
    } 


    Rectangle r = new Rectangle();
    r.x = 10;
    r.y = 10;
    r.width = d.width - 20;
    r.height = d.height - 20;
    offScrGc.setColor(Color.black);
    offScrGc.drawRect(r.x, r.y, r.width, r.height);

    offScrGc.setColor(Color.red);
    int index = 0;
    float value;
    int sx = 0, sy = 0, ex = 0, ey = 0;


    //System.out.println("float rcolors[] = { ");
    for (int i=0; i<101; i++) {
        value = rspline.computePoint((float)i/100f); 
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

        parent.rcolors[i] = value;

	//System.out.println(value + "d, ");
        int pos = (int)(rspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.rcolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);
            
           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.rcolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }

    //System.out.println("float gcolors[] = { ");
    offScrGc.setColor(Color.green);
    index = 0;
    for (int i=0; i<101; i++) {

        value = gspline.computePoint((float)i/100f);    
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

        parent.gcolors[i] = value;
	//System.out.println(value + "d, ");
        int pos = (int)(gspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.gcolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);

           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.gcolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }

    //System.out.println("float bcolors[] = { ");
    offScrGc.setColor(Color.blue);
    index = 0;
    for (int i=0; i<101; i++) {
        value = bspline.computePoint((float)i/100f);    
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

        parent.bcolors[i] = value;
	//System.out.println(value + "d, ");
        int pos = (int)(bspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.bcolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);
           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.bcolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }

     g.drawImage(offScrImage, 0, 0, this);

  }

  // left button action
  public void leftAction() {

     float[] xcontrols = {0.0f, 0.125f, 0.25f, 0.375f, 0.48f, 0.52f,
                           0.625f, 0.75f, 0.875f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.rcolors[0];
     ycontrols[1] = parent.rcolors[25];
     ycontrols[2] = parent.rcolors[50]; 
     ycontrols[3] = parent.rcolors[75];
     ycontrols[4] = parent.rcolors[100]; 
     ycontrols[5] = parent.rcolors[0];
     ycontrols[6] = parent.rcolors[25];
     ycontrols[7] = parent.rcolors[50]; 
     ycontrols[8] = parent.rcolors[75];
     ycontrols[9] = parent.rcolors[100];

     parent.rspline = new NewtonMethod(xcontrols, ycontrols);
     rspline = parent.rspline;

     ycontrols[0] = parent.gcolors[0];
     ycontrols[1] = parent.gcolors[25];
     ycontrols[2] = parent.gcolors[50]; 
     ycontrols[3] = parent.gcolors[75];
     ycontrols[4] = parent.gcolors[100]; 
     ycontrols[5] = parent.gcolors[0];
     ycontrols[6] = parent.gcolors[25];
     ycontrols[7] = parent.gcolors[50]; 
     ycontrols[8] = parent.gcolors[75];
     ycontrols[9] = parent.gcolors[100];

     parent.gspline = new NewtonMethod(xcontrols, ycontrols);
     gspline = parent.gspline;

     ycontrols[0] = parent.bcolors[0];
     ycontrols[1] = parent.bcolors[25];
     ycontrols[2] = parent.bcolors[50]; 
     ycontrols[3] = parent.bcolors[75];
     ycontrols[4] = parent.bcolors[100]; 
     ycontrols[5] = parent.bcolors[0];
     ycontrols[6] = parent.bcolors[25];
     ycontrols[7] = parent.bcolors[50]; 
     ycontrols[8] = parent.bcolors[75];
     ycontrols[9] = parent.bcolors[100];

     parent.bspline = new NewtonMethod(xcontrols, ycontrols);
     bspline = parent.bspline;

     repaint();
     parent.colormap.repaint();
  }

  // right button action
  public void rightAction() {

     float[] xcontrols = {0.0f, 0.25f, 0.5f, 0.75f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.rcolors[0];
     ycontrols[1] = parent.rcolors[13];
     ycontrols[2] = parent.rcolors[25]; 
     ycontrols[3] = parent.rcolors[37];
     ycontrols[4] = parent.rcolors[50]; 

     parent.rspline = new NewtonMethod(xcontrols, ycontrols);
     rspline = parent.rspline;

     ycontrols[0] = parent.gcolors[0];
     ycontrols[1] = parent.gcolors[13];
     ycontrols[2] = parent.gcolors[25]; 
     ycontrols[3] = parent.gcolors[37];
     ycontrols[4] = parent.gcolors[50]; 

     parent.gspline = new NewtonMethod(xcontrols, ycontrols);
     gspline = parent.gspline;

     ycontrols[0] = parent.bcolors[0];
     ycontrols[1] = parent.bcolors[13];
     ycontrols[2] = parent.bcolors[25]; 
     ycontrols[3] = parent.bcolors[37];
     ycontrols[4] = parent.bcolors[50]; 

     parent.bspline = new NewtonMethod(xcontrols, ycontrols);
     bspline = parent.bspline;

     repaint();
     parent.colormap.repaint();

  }

  // up button action
  public void upAction() {

     float[] xcontrols = {0.0f, 0.125f, 0.5f, 0.875f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.rcolors[0];
     ycontrols[1] = parent.rcolors[15];
     ycontrols[2] = parent.rcolors[50]; 
     ycontrols[3] = parent.rcolors[85];
     ycontrols[4] = parent.rcolors[100]; 

     parent.rspline = new NewtonMethod(xcontrols, ycontrols);
     rspline = parent.rspline;

     ycontrols[0] = parent.gcolors[0];
     ycontrols[1] = parent.gcolors[15];
     ycontrols[2] = parent.gcolors[50]; 
     ycontrols[3] = parent.gcolors[85];
     ycontrols[4] = parent.gcolors[100]; 

     parent.gspline = new NewtonMethod(xcontrols, ycontrols);
     gspline = parent.gspline;

     ycontrols[0] = parent.bcolors[0];
     ycontrols[1] = parent.bcolors[15];
     ycontrols[2] = parent.bcolors[50]; 
     ycontrols[3] = parent.bcolors[85];
     ycontrols[4] = parent.bcolors[100]; 

     parent.bspline = new NewtonMethod(xcontrols, ycontrols);
     bspline = parent.bspline;

     repaint();
     parent.colormap.repaint();
  }

  // down button action
  public void downAction() {

     float[] xcontrols = {0.0f, 0.375f, 0.5f, 0.625f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.rcolors[0];
     ycontrols[1] = parent.rcolors[35];
     ycontrols[2] = parent.rcolors[50]; 
     ycontrols[3] = parent.rcolors[65];
     ycontrols[4] = parent.rcolors[100]; 

     parent.rspline = new NewtonMethod(xcontrols, ycontrols);
     rspline = parent.rspline;

     ycontrols[0] = parent.gcolors[0];
     ycontrols[1] = parent.gcolors[35];
     ycontrols[2] = parent.gcolors[50]; 
     ycontrols[3] = parent.gcolors[65];
     ycontrols[4] = parent.gcolors[100]; 

     parent.gspline = new NewtonMethod(xcontrols, ycontrols);
     gspline = parent.gspline;

     ycontrols[0] = parent.bcolors[0];
     ycontrols[1] = parent.bcolors[35];
     ycontrols[2] = parent.bcolors[50]; 
     ycontrols[3] = parent.bcolors[85];
     ycontrols[4] = parent.bcolors[100]; 

     parent.bspline = new NewtonMethod(xcontrols, ycontrols);
     bspline = parent.bspline;

     repaint();
     parent.colormap.repaint();
  }


  public void addPoint() {
     if (parent.selected < 1) return;
     
     addMode = true;
     repaint();
  } 

  public void removePoint() {
     if (parent.selected < 1) return;
     
     removeMode = true;
     repaint();
  }


  // overrided function
  public void processMouseEvent(MouseEvent e) {
     Dimension d = getSize();
     int x = e.getX();
     int y = e.getY();

     if (e.getID() == MouseEvent.MOUSE_PRESSED) { 
        if (parent.selected == 1) {
           if (addMode) {
              rspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              rspline.removePoint((float)(x-20)/(float)d.width);

           } else {
              for (int i=0; i<rspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - rspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
              }
           }
        } else if (parent.selected == 2) {
           if (addMode) {
              gspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              gspline.removePoint((float)(x-20)/(float)d.width);

           } else {
             for (int i=0; i<gspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - gspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
             }
           }
        } else if (parent.selected == 3) {
           if (addMode) {
              bspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              bspline.removePoint((float)(x-20)/(float)d.width);

           } else {
              for (int i=0; i<bspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - bspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
              }
           }
        } else 
           return;


        prevx = x;
        prevy = y;
          
     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) { 
        if (addMode || removeMode)  {
           addMode = false;
           removeMode = false;
           repaint();
           return;
        }

        int amount = prevy - y;
        float val = (float)amount / (float)d.height;

        if (parent.selected == 1) {
           rspline.ycontrols[selected] += val;
           rspline.computeCoef();

        } else if (parent.selected == 2) {
           gspline.ycontrols[selected] += val;
           gspline.computeCoef();

        } else if (parent.selected == 3) {
           bspline.ycontrols[selected] += val;
           bspline.computeCoef();

        } else
           return;

        repaint();
        parent.colormap.repaint();
     }
     super.processMouseEvent(e);
  }

  public void update(Graphics g) {
     paint(g);
  }
}

//////////////////////////////////////////////////////////////////

class HSBChooser extends Canvas {
  
  Graphics offScrGc;
  Image offScrImage;

  int wid = 100;
  int hgt = 100;

  ColorMapDialog parent;

  NewtonMethod hspline, sspline, lspline;     

  int selected;            // selected control points
  int prevx, prevy;        // mouse click

  boolean addMode = false;
  boolean removeMode = false;

  public HSBChooser(ColorMapDialog parent) {
     this.parent = parent;

     hspline = parent.hspline;
     sspline = parent.sspline;
     lspline = parent.lspline;

     enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }


  public void paint(Graphics g) {

     Dimension d = getSize();
     if (d.width < 10) return;
 
     if ((offScrImage == null) || 
          (d.width != wid) || (d.height != hgt)) {

       offScrImage = createImage(d.width, d.height);
       offScrGc = offScrImage.getGraphics();
       wid = d.width;
       hgt = d.height;
    }

    offScrGc.setColor(Color.white);
    offScrGc.fillRect(0, 0, d.width, d.height);

    offScrGc.setColor(Color.black);
    if (addMode || removeMode) {
       if (parent.selected < 1) 
          offScrGc.drawString("Select color first.", 15, 25);
       else
          offScrGc.drawString("Click on the canvas.", 15, 25);
    } 


    Rectangle r = new Rectangle();
    r.x = 10;
    r.y = 10;
    r.width = d.width - 20;
    r.height = d.height - 20;
    offScrGc.setColor(Color.black);
    offScrGc.drawRect(r.x, r.y, r.width, r.height);

    offScrGc.setColor(Color.magenta);
    int index = 0;
    float value;
    int sx = 0, sy = 0, ex = 0, ey = 0;

    for (int i=0; i<101; i++) {
        value = hspline.computePoint((float)i/100f);    
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

	parent.hcolors[i] = value;

        int pos = (int)(hspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.hcolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);
            
           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.hcolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }


    offScrGc.setColor(Color.pink);
    index = 0;
    for (int i=0; i<101; i++) {
        value = sspline.computePoint((float)i/100f);    
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

        parent.scolors[i] = value;

        int pos = (int)(sspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.scolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);

           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.scolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }


    offScrGc.setColor(Color.cyan);
    index = 0;
    for (int i=0; i<101; i++) {
        value = lspline.computePoint((float)i/100f);    
        if (value > 1.0f) value = 1.0f;
        else if (value < 0.0f) value = 0.0f;

        parent.lcolors[i] = value;

        int pos = (int)(lspline.xcontrols[index] * 100);         
        if (i == pos) { 
           sx = (int)((float)i/100f*r.width)-3+r.x;
           sy = (int)(r.height-parent.lcolors[i]*r.height)-3+r.y;
           offScrGc.fillRect(sx, sy, 6, 6);

           index++;           
        } else {
           ex = (int)((float)i/100f*r.width)-1+r.x;
           ey = (int)(r.height-parent.lcolors[i]*r.height)-1+r.y;
           offScrGc.drawLine(sx, sy, ex, ey);
           sx = ex;
           sy = ey;
        }
     }

     g.drawImage(offScrImage, 0, 0, this);

  }

  public void addPoint() {
     if (parent.selected < 1) return;
     
     addMode = true;
     repaint();
  } 

  public void removePoint() {
     if (parent.selected < 1) return;
     
     removeMode = true;
     repaint();
  }


  // overrided function
  public void processMouseEvent(MouseEvent e) {
     Dimension d = getSize();
     int x = e.getX();
     int y = e.getY();

     if (e.getID() == MouseEvent.MOUSE_PRESSED) { 
        if (parent.selected == 1) {
           if (addMode) {
              hspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              hspline.removePoint((float)(x-20)/(float)d.width);

           } else {
              for (int i=0; i<hspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - hspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
              }
           }
        } else if (parent.selected == 2) {
           if (addMode) {
              sspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              sspline.removePoint((float)(x-20)/(float)d.width);

           } else {
             for (int i=0; i<sspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - sspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
             }
           }
        } else if (parent.selected == 3) {
           if (addMode) {
              lspline.addPoint((float)x/(float)d.width,
                                   1.0f - (float)y/(float)d.height);
           } else if (removeMode) {
              lspline.removePoint((float)(x-20)/(float)d.width);

           } else {
              for (int i=0; i<lspline.nControls; i++) {
	        if (((float)(x-20)/(float)d.width - lspline.xcontrols[i]) 
                                                                  <= 0.0f) {
                    selected = i;
                    break;
                }
              }
           }
        } else 
           return;


        prevx = x;
        prevy = y;
          
     } else if (e.getID() == MouseEvent.MOUSE_RELEASED) { 
        if (addMode || removeMode)  {
           addMode = false;
           removeMode = false;
           repaint();
           return;
        }

        int amount = prevy - y;
        float val = (float)amount / (float)d.height;

        if (parent.selected == 1) {
           hspline.ycontrols[selected] += val;
           hspline.computeCoef();
        } else if (parent.selected == 2) {
           sspline.ycontrols[selected] += val;
           sspline.computeCoef();
        } else if (parent.selected == 3) {
           lspline.ycontrols[selected] += val;
           lspline.computeCoef();
        } else
           return;

        repaint();
        parent.colormap.repaint();
     }
     super.processMouseEvent(e);
  }

  public void update(Graphics g) {
     paint(g);
  }



  // left button action
  public void leftAction() {

     float[] xcontrols = {0.0f, 0.125f, 0.25f, 0.375f, 0.48f, 0.52f,
                           0.625f, 0.75f, 0.875f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.hcolors[0];
     ycontrols[1] = parent.hcolors[25];
     ycontrols[2] = parent.hcolors[50]; 
     ycontrols[3] = parent.hcolors[75];
     ycontrols[4] = parent.hcolors[100]; 
     ycontrols[5] = parent.hcolors[0];
     ycontrols[6] = parent.hcolors[25];
     ycontrols[7] = parent.hcolors[50]; 
     ycontrols[8] = parent.hcolors[75];
     ycontrols[9] = parent.hcolors[100];

     parent.hspline = new NewtonMethod(xcontrols, ycontrols);
     hspline = parent.hspline;

     ycontrols[0] = parent.scolors[0];
     ycontrols[1] = parent.scolors[25];
     ycontrols[2] = parent.scolors[50]; 
     ycontrols[3] = parent.scolors[75];
     ycontrols[4] = parent.scolors[100]; 
     ycontrols[5] = parent.scolors[0];
     ycontrols[6] = parent.scolors[25];
     ycontrols[7] = parent.scolors[50]; 
     ycontrols[8] = parent.scolors[75];
     ycontrols[9] = parent.scolors[100];

     parent.sspline = new NewtonMethod(xcontrols, ycontrols);
     sspline = parent.sspline;

     ycontrols[0] = parent.lcolors[0];
     ycontrols[1] = parent.lcolors[25];
     ycontrols[2] = parent.lcolors[50]; 
     ycontrols[3] = parent.lcolors[75];
     ycontrols[4] = parent.lcolors[100]; 
     ycontrols[5] = parent.lcolors[0];
     ycontrols[6] = parent.lcolors[25];
     ycontrols[7] = parent.lcolors[50]; 
     ycontrols[8] = parent.lcolors[75];
     ycontrols[9] = parent.lcolors[100];

     parent.lspline = new NewtonMethod(xcontrols, ycontrols);
     lspline = parent.lspline;

     repaint();
     parent.colormap.repaint();
  }

  // right button action
  public void rightAction() {

     float[] xcontrols = {0.0f, 0.25f, 0.5f, 0.75f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.hcolors[0];
     ycontrols[1] = parent.hcolors[13];
     ycontrols[2] = parent.hcolors[25]; 
     ycontrols[3] = parent.hcolors[37];
     ycontrols[4] = parent.hcolors[50]; 

     parent.hspline = new NewtonMethod(xcontrols, ycontrols);
     hspline = parent.hspline;

     ycontrols[0] = parent.scolors[0];
     ycontrols[1] = parent.scolors[13];
     ycontrols[2] = parent.scolors[25]; 
     ycontrols[3] = parent.scolors[37];
     ycontrols[4] = parent.scolors[50]; 

     parent.sspline = new NewtonMethod(xcontrols, ycontrols);
     sspline = parent.sspline;

     ycontrols[0] = parent.lcolors[0];
     ycontrols[1] = parent.lcolors[13];
     ycontrols[2] = parent.lcolors[25]; 
     ycontrols[3] = parent.lcolors[37];
     ycontrols[4] = parent.lcolors[50]; 

     parent.lspline = new NewtonMethod(xcontrols, ycontrols);
     lspline = parent.lspline;

     repaint();
     parent.colormap.repaint();

  }

  // up button action
  public void upAction() {

     float[] xcontrols = {0.0f, 0.125f, 0.5f, 0.875f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.hcolors[0];
     ycontrols[1] = parent.hcolors[15];
     ycontrols[2] = parent.hcolors[50]; 
     ycontrols[3] = parent.hcolors[85];
     ycontrols[4] = parent.hcolors[100]; 

     parent.hspline = new NewtonMethod(xcontrols, ycontrols);
     hspline = parent.hspline;

     ycontrols[0] = parent.scolors[0];
     ycontrols[1] = parent.scolors[15];
     ycontrols[2] = parent.scolors[50]; 
     ycontrols[3] = parent.scolors[85];
     ycontrols[4] = parent.scolors[100]; 

     parent.sspline = new NewtonMethod(xcontrols, ycontrols);
     sspline = parent.sspline;

     ycontrols[0] = parent.lcolors[0];
     ycontrols[1] = parent.lcolors[15];
     ycontrols[2] = parent.lcolors[50]; 
     ycontrols[3] = parent.lcolors[85];
     ycontrols[4] = parent.lcolors[100]; 

     parent.lspline = new NewtonMethod(xcontrols, ycontrols);
     lspline = parent.lspline;

     repaint();
     parent.colormap.repaint();
  }

  // down button action
  public void downAction() {

     float[] xcontrols = {0.0f, 0.375f, 0.5f, 0.625f, 1.0f};
     float[] ycontrols = new float[10];

     ycontrols[0] = parent.hcolors[0];
     ycontrols[1] = parent.hcolors[35];
     ycontrols[2] = parent.hcolors[50]; 
     ycontrols[3] = parent.hcolors[65];
     ycontrols[4] = parent.hcolors[100]; 

     parent.hspline = new NewtonMethod(xcontrols, ycontrols);
     hspline = parent.hspline;

     ycontrols[0] = parent.scolors[0];
     ycontrols[1] = parent.scolors[35];
     ycontrols[2] = parent.scolors[50]; 
     ycontrols[3] = parent.scolors[65];
     ycontrols[4] = parent.scolors[100]; 

     parent.sspline = new NewtonMethod(xcontrols, ycontrols);
     sspline = parent.sspline;

     ycontrols[0] = parent.lcolors[0];
     ycontrols[1] = parent.lcolors[35];
     ycontrols[2] = parent.lcolors[50]; 
     ycontrols[3] = parent.lcolors[85];
     ycontrols[4] = parent.lcolors[100]; 

     parent.lspline = new NewtonMethod(xcontrols, ycontrols);
     lspline = parent.lspline;

     repaint();
     parent.colormap.repaint();
  }

}







