package sv.ui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;


/** 
 * LabelInputDialog2.java
 * To get the input of the 1d and vector labels
 * @version 1.0
 * @author Byeongseob Ki
 */

public class LabelInputDialog2 extends Dialog implements ActionListener {
    // labels to be changed
    String labels[];
    Font[] xyfont;
    boolean[] isRotate;


    TextField tfs[];
    Checkbox xcb, ycb;
    JComboBox xnamecb, xsizecb, xstylecb;  
    JComboBox ynamecb, ysizecb, ystylecb;    

    public LabelInputDialog2(Frame frame, 
              String[] lbls, Font[] xyfont, boolean[] isRotate) {

        super(frame, "Label Change", true);   // make it modal
        labels = lbls;
        this.xyfont = xyfont;
        this.isRotate = isRotate;

        tfs = new TextField[2];

        makeGUI();
        pack();
    }

    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (command.equals("Apply")) {
           
            String name;
            int style, size;

            int[] sizearr = {8, 10, 12, 14, 18, 24, 36};
            int[] stylearr = {Font.PLAIN, Font.BOLD, Font.ITALIC };

            for (int i = 0; i < tfs.length; i++)
                labels[i] = tfs[i].getText().trim();

            if (xcb.getState()) isRotate[0] = true;
            else                isRotate[0] = false;
            if (ycb.getState()) isRotate[1] = true;
            else                isRotate[1] = false;

            name = ((String)xnamecb.getSelectedItem()).trim();
            int stylep = xstylecb.getSelectedIndex();
            int sizep = xsizecb.getSelectedIndex();

            size = sizearr[sizep];
            style = stylearr[stylep];

            xyfont[0] = new Font(name, style, size);
         
            name = ((String)ynamecb.getSelectedItem()).trim();
            stylep = ystylecb.getSelectedIndex();
            sizep = ysizecb.getSelectedIndex();

            size = sizearr[sizep];
            style = stylearr[stylep];

            xyfont[1] = new Font(name, style, size);

            dispose();

        } else if (command.equals("Clear")) {
            for (int j = 0; j < tfs.length; j++)
                tfs[j].setText("");
            return;
        } else if (command.equals("Cancel")) {
            labels = null;
            dispose();
        }
    }

    public String[] getArgs() {
        if (labels == null || labels[0].length() < 1 ||
                             labels[1].length() < 1)
           return null;

        String[] args = new String[10];

        args[0] = labels[0];
        args[1] = labels[1];
        args[2] = xyfont[0].getName();
        args[3] = String.valueOf(xyfont[0].getStyle());
        args[4] = String.valueOf(xyfont[0].getSize());
        args[5] = xyfont[1].getName();
        args[6] = String.valueOf(xyfont[1].getStyle());
        args[7] = String.valueOf(xyfont[1].getSize());

        if (isRotate[0])
           args[8] = "true";
        else
           args[8] = "false";

        if (isRotate[1])
           args[9] = "true";
        else
           args[9] = "false";

        return args;
    }


    private void makeGUI() {
        JPanel p;

        setLayout(new BorderLayout());

        JPanel centerp = new JPanel();
        centerp.setLayout(new GridLayout(2, 1));


        JPanel p1 = new JPanel();
        p1.setLayout(new GridLayout(3, 1));
        p1.setBorder(new TitledBorder("X Axis Label"));

        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
        p.add(new Label("X Axis Label: "));
        p.add(tfs[0] = new TextField(15));
        tfs[0].setText(labels[0]);
        p1.add(p);

        p1.add(xcb = new Checkbox("Rotate 90 degree", isRotate[0]));
       
        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
        
        xnamecb = new JComboBox();
        String[] fonts = getToolkit().getFontList();

        for(int i=0; i<fonts.length; i++)
           xnamecb.addItem(fonts[i]);
        xnamecb.setSelectedItem(xyfont[0].getName()); 

        xsizecb = new JComboBox();
        xsizecb.addItem(" 8");
        xsizecb.addItem("10");
        xsizecb.addItem("12");
        xsizecb.addItem("14");
        xsizecb.addItem("18");
        xsizecb.addItem("24");
        xsizecb.addItem("36");
        xsizecb.setSelectedItem(String.valueOf(xyfont[0].getSize())); 
   
        xstylecb = new JComboBox();
        xstylecb.addItem("Plain");
        xstylecb.addItem("Bold");
        xstylecb.addItem("Italic");

        if (xyfont[0].isPlain())
           xstylecb.setSelectedItem("Plain");
        else if (xyfont[0].isBold())
           xstylecb.setSelectedItem("Bold");
        else if (xyfont[0].isItalic())
           xstylecb.setSelectedItem("Italic");
    


        p.add(xnamecb);
        p.add(xsizecb);
        p.add(xstylecb);

        p1.add(p);


        JPanel p2 = new JPanel();
        p2.setLayout(new GridLayout(3, 1));
        p2.setBorder(new TitledBorder("Y Axis Label"));

        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
        p.add(new Label("Y Axis Label: "));
        p.add(tfs[1] = new TextField(15));
        tfs[1].setText(labels[1]);
        p2.add(p);

        p2.add(ycb = new Checkbox("Rotate 90 degree", isRotate[1]));
       
        p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
        
        ynamecb = new JComboBox();
        fonts = getToolkit().getFontList();

        for(int i=0; i<fonts.length; i++)
           ynamecb.addItem(fonts[i]);
        ynamecb.setSelectedItem(xyfont[1].getName()); 

        ysizecb = new JComboBox();
        ysizecb.addItem(" 8");
        ysizecb.addItem("10");
        ysizecb.addItem("12");
        ysizecb.addItem("14");
        ysizecb.addItem("18");
        ysizecb.addItem("24");
        ysizecb.addItem("36");
        ysizecb.setSelectedItem(String.valueOf(xyfont[1].getSize())); 

        ystylecb = new JComboBox();
        ystylecb.addItem("Plain");
        ystylecb.addItem("Bold");
        ystylecb.addItem("Italic");

        if (xyfont[1].isPlain())
           ystylecb.setSelectedItem("Plain");
        else if (xyfont[1].isBold())
           ystylecb.setSelectedItem("Bold");
        else if (xyfont[1].isItalic())
           ystylecb.setSelectedItem("Italic");

        p.add(ynamecb);
        p.add(ysizecb);
        p.add(ystylecb);

        p2.add(p);


        centerp.add(p1);
        centerp.add(p2);

        // button panel
        Panel btnp = new Panel();
        btnp.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));

        Button btn = new Button("Apply");
        btn.addActionListener(this);
        btnp.add(btn);

        btn = new Button("Clear");
        btn.addActionListener(this);
        btnp.add(btn);

        btn = new Button("Cancel");
        btn.addActionListener(this);
        btnp.add(btn);

        add("Center", centerp);
        add("South", btnp);
    }
}
