package sv.ui;

import java.awt.*;
import java.awt.event.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.drawer.LineShape;

/** 
 * LineChooser.java
 * To get the user input
 * @version 1.0
 * @author Byeongseob Ki
 */

public class LineChooser extends Dialog 
                         implements ActionListener, ItemListener {

  int line_type;
  int line_wid;
  int dash_size;
  Color line_color;

  LineCanvas canvas;
  JComboBox lwcb, dscb;
  Checkbox fillcb;  
  JRadioButton normalbtn, dashbtn, leftbtn, rightbtn, nobtn, lrbtn;

  /** 
   * constructor
   */
  public LineChooser(Frame fr, int type, int wid, int dash, Color c) {
     super(fr, "Line Chooser", true);    // make it modal

     line_type = type;
     line_color = c;
     line_wid = wid;
     dash_size = dash;

     // make GUI components
     makeGUI();
     pack();
  }


  /**
   * overrided method
   */
  public void actionPerformed(ActionEvent evt) {
     String command = evt.getActionCommand();

     if (command.equals("Ok")) { 
         
     } else if (command.equals("Cancel"))     
        line_type = -1;

     dispose();
  }


  /**
   * overrided method 
   */
  public void itemStateChanged(ItemEvent e) {
    if (lwcb == null || dscb == null || fillcb == null ||
         normalbtn == null || dashbtn == null || leftbtn == null || 
         rightbtn == null || nobtn == null || lrbtn == null || canvas == null)
        return;

    if (e.getStateChange() == ItemEvent.SELECTED ||
                                       e.getSource() instanceof Checkbox) {

       if (e.getSource() instanceof JComboBox) {

          line_wid  = 
        Integer.valueOf(((String)lwcb.getSelectedItem()).trim()).intValue();

          dash_size =
        Integer.valueOf(((String)dscb.getSelectedItem()).trim()).intValue();

       } else if (e.getSource() instanceof JRadioButton || 
                               e.getSource() instanceof Checkbox) {

          boolean fillS = fillcb.getState();

          if (normalbtn.isSelected()) {
             dscb.setEnabled(false);

             if (nobtn.isSelected()) {
                line_type = LineShape.NOARROW_LINE;
                fillcb.setState(false);
             } else if (rightbtn.isSelected()) {
                if (fillS) line_type = LineShape.RIGHTFILLARROW_LINE;
                else       line_type = LineShape.RIGHTARROW_LINE;
 
             } else if (leftbtn.isSelected()) {
                if (fillS) line_type = LineShape.LEFTFILLARROW_LINE;
                else       line_type = LineShape.LEFTARROW_LINE;
 
             } else if (lrbtn.isSelected()) {
                if (fillS) line_type = LineShape.LRFILLARROW_LINE;
                else       line_type = LineShape.LRARROW_LINE;
             } 
          } else if (dashbtn.isSelected()) {
             dscb.setEnabled(true);

             if (nobtn.isSelected()) {
                line_type = LineShape.NOARROW_DLINE;
                fillcb.setState(false);
             } else if (rightbtn.isSelected()) {
                if (fillS) line_type = LineShape.RIGHTFILLARROW_DLINE;
                else       line_type = LineShape.RIGHTARROW_DLINE;
 
             } else if (leftbtn.isSelected()) {
                if (fillS) line_type = LineShape.LEFTFILLARROW_DLINE;
                else       line_type = LineShape.LEFTARROW_DLINE;
 
             } else if (lrbtn.isSelected()) {
                if (fillS) line_type = LineShape.LRFILLARROW_DLINE;
                else       line_type = LineShape.LRARROW_DLINE;
             } 
          }

       }
    }

    canvas.repaint();
  }


  public int getLineType() {
      return line_type;
  }
  
  public int getLineWidth() {
      return line_wid;
  }

  public int getDashSize() {
      return dash_size;
  }

  // make GUI of this class
  private void makeGUI() {
     setLayout(new BorderLayout());
     Panel p;
     ButtonGroup group;

     Panel cp = new Panel();
     cp.setLayout(new BoxLayout(cp, BoxLayout.Y_AXIS));


     ////////////////////////////////////////
     // line panel
     JPanel lp = new JPanel();
     lp.setLayout(new GridLayout(3, 1));
     lp.setBorder(new TitledBorder("Line"));

     p = new Panel();
     p.setLayout(new FlowLayout(FlowLayout.CENTER));
     group = new ButtonGroup();

     normalbtn = new JRadioButton("Normal");
     normalbtn.setToolTipText("Normal line");
     group.add(normalbtn);
     normalbtn.addItemListener(this);
     p.add(normalbtn);

     dashbtn = new JRadioButton("Dashed");
     dashbtn.setToolTipText("Dashed line");
     group.add(dashbtn);
     dashbtn.addItemListener(this);
     p.add(dashbtn);

     lp.add(p);

     p = new Panel();
     p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
     p.add(new Label("Line Width: "));
     lwcb = new JComboBox();

     for(int i=1; i<7; i++)
        lwcb.addItem(String.valueOf(i));

     lwcb.setSelectedItem(String.valueOf(line_wid));
     lwcb.addItemListener(this);
     p.add(lwcb);

     lp.add(p);

     p = new Panel();
     p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
     p.add(new Label("Dash Size:  "));
     dscb = new JComboBox();

     for(int i=1; i<7; i++)
        dscb.addItem(String.valueOf(i));

     dscb.setSelectedItem(String.valueOf(dash_size));
     dscb.addItemListener(this);
     p.add(dscb);

     if (line_type < LineShape.NOARROW_DLINE) {
       normalbtn.setSelected(true);
       dscb.setEnabled(false);
     } else
       dashbtn.setSelected(true);

     lp.add(p);

     cp.add(lp);

     ////////////////////////////////////////
     // arrow panel
     JPanel ap = new JPanel();
     ap.setLayout(new BoxLayout(ap, BoxLayout.Y_AXIS));
     ap.setBorder(new TitledBorder("Arrow"));

     p = new Panel();
     p.setLayout(new GridLayout(2, 2));
     group = new ButtonGroup();

     leftbtn = new JRadioButton("Left");
     leftbtn.setToolTipText("Left Arrow");
     group.add(leftbtn);
     leftbtn.addItemListener(this);
     p.add(leftbtn);

     rightbtn = new JRadioButton("Right");
     rightbtn.setToolTipText("Right Arrow");
     group.add(rightbtn);
     rightbtn.addItemListener(this);
     p.add(rightbtn);

     lrbtn = new JRadioButton("Left-Right");
     lrbtn.setToolTipText("Left-Right Arrow");
     group.add(lrbtn);
     lrbtn.setSelected(true);
     lrbtn.addItemListener(this);
     p.add(lrbtn);

     nobtn = new JRadioButton("None");
     nobtn.setToolTipText("No Arrow");
     group.add(nobtn);
     nobtn.addItemListener(this);
     p.add(nobtn);

     if (line_type == LineShape.NOARROW_LINE ||
             line_type == LineShape.NOARROW_DLINE)
       nobtn.setSelected(true);
     else if (line_type == LineShape.RIGHTFILLARROW_LINE ||
            line_type == LineShape.RIGHTARROW_LINE ||
            line_type == LineShape.RIGHTFILLARROW_DLINE ||
            line_type == LineShape.RIGHTARROW_DLINE)
       rightbtn.setSelected(true);
     else if (line_type == LineShape.LEFTFILLARROW_LINE ||
            line_type == LineShape.LEFTARROW_LINE ||
            line_type == LineShape.LEFTFILLARROW_DLINE ||
            line_type == LineShape.LEFTARROW_DLINE)
       leftbtn.setSelected(true);
     else
       lrbtn.setSelected(true);

     ap.add(p);
     ap.add(fillcb = new Checkbox("Fill Arrow"));
     fillcb.addItemListener(this);

     if (line_type == LineShape.RIGHTFILLARROW_LINE ||
           line_type == LineShape.RIGHTFILLARROW_DLINE ||
           line_type == LineShape.LEFTFILLARROW_LINE ||
           line_type == LineShape.LEFTFILLARROW_DLINE ||
           line_type == LineShape.LRFILLARROW_LINE ||
           line_type == LineShape.LRFILLARROW_DLINE)
         fillcb.setState(true);
     else
         fillcb.setState(false);

     cp.add(ap);

     ////////////////////////////////////////
     // line canvas
     cp.add(canvas = new LineCanvas(this));
     canvas.setSize(ap.getSize().width, 40);

     ////////////////////////////////////////
     // button panel
     Panel btnp = new Panel();      // for buttons
     btnp.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
    
     Button btn = new Button("Ok");
     btn.addActionListener(this);
     btnp.add(btn);

     btn = new Button("Cancel");
     btn.addActionListener(this);
     btnp.add(btn);

     add("Center", cp);
     add("South", btnp);
  }


  class LineCanvas extends Canvas {
    LineChooser chooser;

    public LineCanvas(LineChooser chooser) {
       this.chooser = chooser;
    }
 
    public void paint(Graphics g) {
       g.setColor(Color.white);
       g.fillRect(0, 0, getSize().width, getSize().height);

       int cx = getSize().width / 2;
       int cy = getSize().height / 2;

       int sx = cx - 40;
       int ex = cx + 40;

       int[] xpts = new int[3];
       int[] ypts = new int[3];
       int dummy1, dummy2;

       switch (chooser.line_type) {
        case LineShape.NOARROW_LINE:
          SVUIC.drawLine(g, sx, cy, ex, cy, 
                                 chooser.line_wid, chooser.line_color);
          break;

        case LineShape.RIGHTFILLARROW_LINE:
          dummy1 = ex-chooser.line_wid-10;

          SVUIC.drawLine(g, sx, cy, dummy1, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.RIGHTARROW_LINE:
          dummy1 = ex-chooser.line_wid-10;

          SVUIC.drawLine(g, sx, cy, dummy1, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTFILLARROW_LINE:
          dummy1 = sx+chooser.line_wid+10;

          SVUIC.drawLine(g, dummy1, cy, ex, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTARROW_LINE:
          dummy1 = sx+chooser.line_wid+10;

          SVUIC.drawLine(g, dummy1, cy, ex, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LRFILLARROW_LINE:
          dummy1 = sx+chooser.line_wid+10;
          dummy2 = ex-chooser.line_wid-10;

          SVUIC.drawLine(g, dummy1, cy, dummy2, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);


          break;

        case LineShape.LRARROW_LINE:
          dummy1 = sx+chooser.line_wid+10;
          dummy2 = ex-chooser.line_wid-10;

          SVUIC.drawLine(g, dummy1, cy, dummy2, cy, 
                                 chooser.line_wid, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;


        case LineShape.NOARROW_DLINE:
          SVUIC.drawDottedLine(g, sx, cy, ex, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          break;

        case LineShape.RIGHTFILLARROW_DLINE:
          dummy1 = ex-chooser.line_wid-10;

          SVUIC.drawDottedLine(g, sx, cy, dummy1, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.RIGHTARROW_DLINE:
          dummy1 = ex-chooser.line_wid-10;

          SVUIC.drawDottedLine(g, sx, cy, dummy1, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = ex;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTFILLARROW_DLINE:
          dummy1 = sx+chooser.line_wid+10;

          SVUIC.drawDottedLine(g, dummy1, cy, ex, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          break;

        case LineShape.LEFTARROW_DLINE:
          dummy1 = sx+chooser.line_wid+10;

          SVUIC.drawDottedLine(g, dummy1, cy, ex, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;

        case LineShape.LRFILLARROW_DLINE:
          dummy1 = sx+chooser.line_wid+10;
          dummy2 = ex-chooser.line_wid-10;

          SVUIC.drawDottedLine(g, dummy1, cy, dummy2, cy, 
                     chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.fillPolygon(xpts, ypts, 3);


          break;

        case LineShape.LRARROW_DLINE:
          dummy1 = sx+chooser.line_wid+10;
          dummy2 = ex-chooser.line_wid-10;

          SVUIC.drawDottedLine(g, dummy1, cy, dummy2, cy, 
                    chooser.line_wid, chooser.dash_size, chooser.line_color);
          xpts[0] = dummy1;
          xpts[1] = sx;
          xpts[2] = dummy1;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          xpts[0] = dummy2;
          xpts[1] = ex;
          xpts[2] = dummy2;
          ypts[0] = cy - chooser.line_wid - 5;
          ypts[1] = cy;
          ypts[2] = cy + chooser.line_wid + 5;
          g.setColor(chooser.line_color);
          g.drawPolygon(xpts, ypts, 3);

          break;

       }
    }


    public void update(Graphics g) {
       paint(g);
    }
  }

}



