package sv.ui;

import java.awt.*;
import java.awt.event.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;


/** 
 * MarkerChooser.java
 * To get the user input
 * @version 1.0
 * @author Byeongseob Ki
 */


public class MarkerChooser extends Dialog implements ActionListener {
    public static final int CIRCLEMARKER = 1;
    public static final int SQUAREMARKER = 2;
    public static final int DIAMONDMARKER = 3;
    public static final int CIRCLEFILLMARKER = 4;
    public static final int SQUAREFILLMARKER = 5;
    public static final int DIAMONDFILLMARKER = 6;
    public static final int CROSSMARKER = 7;
    public static final int XMARKER = 8;
    public static final int NOMARKER = 9;

    Markers[] markerArr;


    // marker type
    int type;

    /**
     * constructor
     */
    public MarkerChooser(Frame frame, int type) {
        super(frame, "Marker Chooser", true);  // make it modal
        this.type = type;
        markerArr = new Markers[9];

        makeGUI();
        pack();
    }

    /**
     * overrided method
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (command.equals("Done"))
            dispose();
    }

    public int getType() {
        return type;
    }

  public void notifySelection(int selected) {
     type = selected;

     for (int i=0; i<9; i++) {
       if (selected == i+1) 
          markerArr[i].setSelected(true);
       else
          markerArr[i].setSelected(false);
       markerArr[i].repaint();
     }
  }


    public Dimension getPreferredSize() {
        return new Dimension(300, 200);
    }

    private void makeGUI() {
        setLayout(new BorderLayout());

        JPanel p = new JPanel();
        p.setLayout(new GridLayout(3, 3, 5, 2));
        p.setBorder(new TitledBorder("Markers"));

        p.add(markerArr[0] = new Markers(this, CIRCLEMARKER));
        p.add(markerArr[1] = new Markers(this, SQUAREMARKER));
        p.add(markerArr[2] = new Markers(this, DIAMONDMARKER));
        p.add(markerArr[3] = new Markers(this, CIRCLEFILLMARKER));
        p.add(markerArr[4] = new Markers(this, SQUAREFILLMARKER));
        p.add(markerArr[5] = new Markers(this, DIAMONDFILLMARKER));
        p.add(markerArr[6] = new Markers(this, CROSSMARKER));
        p.add(markerArr[7] = new Markers(this, XMARKER));
        p.add(markerArr[8] = new Markers(this, NOMARKER));

        switch (type) {
        case CIRCLEMARKER:
            markerArr[0].setSelected(true);
            break;

        case SQUAREMARKER:
            markerArr[1].setSelected(true);
            break;

        case DIAMONDMARKER:
            markerArr[2].setSelected(true);
            break;

        case CIRCLEFILLMARKER:
            markerArr[3].setSelected(true);
            break;

        case SQUAREFILLMARKER:
            markerArr[4].setSelected(true);
            break;

        case DIAMONDFILLMARKER:
            markerArr[5].setSelected(true);
            break;
        case CROSSMARKER:
            markerArr[6].setSelected(true);
            break;

        case XMARKER:
            markerArr[7].setSelected(true);
            break;

        case NOMARKER:
            markerArr[8].setSelected(true);
            break;
        }

        // button panel
        Panel p2 = new Panel();
        p2.setLayout(new FlowLayout(FlowLayout.RIGHT));

        Button btn = new Button("Done");
        btn.addActionListener(this);
        p2.add(btn);

        add("Center", p);
        add("South", p2);
    }


   class Markers extends Canvas {


     int type;
     private boolean selected = false;

     private Rectangle marker;
     private Rectangle choosed;

     Image checkImg ;
     MarkerChooser parent;

     Markers(MarkerChooser parent, int type) {
        this.parent = parent;
        this.type = type;

        checkImg = 
           getToolkit().getImage(getClass().getResource("checkmark.gif"));

        marker = new Rectangle();
        choosed = new Rectangle();

        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
     }


     public void setSelected(boolean b) {
        selected = b;
     }

     public boolean isSelected() {
        return selected;
     }

     public void paint(Graphics g) {
        Dimension d = getSize();

        choosed.x = d.width - 30;
        choosed.y = d.height - 30;
        choosed.width = 24;
        choosed.height = 20;

        marker.x = d.width - 60;
        marker.y = d.height - 30;
        marker.width = 24;
        marker.height = 20;
     
        g.setColor(Color.white);
        g.fillRect(choosed.x, choosed.y, choosed.width, choosed.height);
        g.setColor(Color.black);
        g.drawRect(choosed.x+1, choosed.y+1, choosed.width-2, choosed.height-2);

        g.setColor(Color.white);
        g.fillRect(marker.x, marker.y, marker.width, marker.height);
        g.setColor(Color.black);
        g.drawRect(marker.x+1, marker.y+1, marker.width-2, marker.height-2);

        if (selected)
          g.drawImage(checkImg, choosed.x+2, choosed.y+2, this);

        int[] xpts = {marker.x+12,  marker.x+9, marker.x+12, marker.x+15 };
        int[] ypts = {marker.y+7, marker.y+10, marker.y+13, marker.y+10 };

        g.setColor(Color.red);
        switch(type) {
        case MarkerChooser.CIRCLEMARKER:
           g.drawOval(marker.x+9, marker.y+7, 6, 6);
           break;
        case MarkerChooser.SQUAREMARKER:
           g.drawRect(marker.x+9, marker.y+7, 6, 6);
           break;
        case MarkerChooser.DIAMONDMARKER:
           g.drawPolygon(xpts, ypts, 4);
           break;

        case MarkerChooser.CIRCLEFILLMARKER:
           g.fillOval(marker.x+9, marker.y+7, 6, 6);
           break;
        case MarkerChooser.SQUAREFILLMARKER:
           g.fillRect(marker.x+9, marker.y+7, 6, 6);
           break;
        case MarkerChooser.DIAMONDFILLMARKER:
           g.drawPolygon(xpts, ypts, 4);
           g.fillPolygon(xpts, ypts, 4);
           break;

        case MarkerChooser.CROSSMARKER:
           g.drawLine(marker.x+9, marker.y+10, marker.x+15, marker.y+10);
           g.drawLine(marker.x+12, marker.y+7, marker.x+12, marker.y+13);
           break;
        case MarkerChooser.XMARKER:
           g.drawLine(marker.x+9, marker.y+7, marker.x+15, marker.y+13);
           g.drawLine(marker.x+9, marker.y+13, marker.x+15, marker.y+7);
           break;
        }
     }

     // overrided function
     public void processMouseEvent(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();

        if (e.getID() == MouseEvent.MOUSE_PRESSED) { 
          if (choosed.contains(x, y)) 
             parent.notifySelection(type);
        }
        super.processMouseEvent(e);
     }
   }

}

