package sv.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.kernel.*;


public class RenameDialog extends Dialog 
                   implements ActionListener,  ItemListener {

  // title TextField
  private TextField titleField;

  // current avaliable DataSet list
  private List dataList;

  private String[] titleArgs;

  // to access GFrame class as parent
  private GFrame gframe;

  public RenameDialog(Frame gframe) {
     super(gframe, "Rename the data set(s)", true);   // make it modal
     this.gframe = (GFrame)gframe;

     // make GUI components
     makeGUI();
     refleshItems();
     pack();
  }


  public void refleshItems() {

     // dataList and TextField initializing
     Enumeration e;
     boolean first = true;

     if (dataList.getItemCount() >  0) 
        dataList.removeAll();

     for (e=gframe.dataSetVector.elements(); e.hasMoreElements();) {
        DataSet data = (DataSet)e.nextElement();
        dataList.addItem(data.getTitle());
        if (first) {
           titleField.setText(data.getTitle());
           first = false;
        }
     }

     dataList.select(0);
  }


  /**
   * overrided method (for buttons)
   */
  public void actionPerformed(ActionEvent e) {
     String command = e.getActionCommand();
     int selected = dataList.getSelectedIndex();
     String str = "";

     if (command.equals("Apply")) {
        titleArgs = new String[2];

        DataSet data = 
          (DataSet)gframe.dataSetVector.elementAt(selected);

        titleArgs[0] = data.getTitle();
        titleArgs[1] = titleField.getText().trim();

     } else if (command.equals("Cancel")) 
        titleArgs = null;

     dispose();
  }


  // return setted values
  public String[] getArguments() {
     return titleArgs;
  }


  /**
   * overrided method (for list)
   */
  public void itemStateChanged(ItemEvent e) {
     if (e.getStateChange() == ItemEvent.SELECTED) {
        DataSet data = 
          (DataSet)gframe.dataSetVector.elementAt(dataList.getSelectedIndex());
        titleField.setText(data.getTitle());
     }
  }


  // make GUI of this class
  private void makeGUI() {
     Component c;
     setLayout(new BorderLayout());

     GridBagLayout gbl = new GridBagLayout();
     GridBagConstraints gbc = new GridBagConstraints();
     
     // define upper, center and lower panel
     Panel upperPanel = new Panel();
     JPanel centerPanel = new JPanel();
     Panel lowerPanel = new Panel();

     // make upper part(label and list)
     upperPanel.setLayout(new BorderLayout());
     upperPanel.add("North", new Label("Available DataSet for this frame:"));
     upperPanel.add("Center", dataList = new List(5, false));
     dataList.addActionListener(this);
     dataList.addItemListener(this); 

     // make center part(current setting panel)
     centerPanel = new JPanel();
     centerPanel.setLayout(gbl);
     centerPanel.setBorder(new TitledBorder("Current Setting"));

     gbc.gridx = 1;
     gbc.gridy = 1;
     c = new Label("Title:");
     gbl.setConstraints(c, gbc);
     centerPanel.add(c);

     gbc.gridx = 2;
     gbc.gridy = 1;
     gbc.gridwidth = 4;
     titleField = new TextField(25);
     gbl.setConstraints(titleField, gbc);
     centerPanel.add(titleField);


     // make lower part(two buttons)
     lowerPanel.setLayout(new BorderLayout());
     lowerPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
     Button btn = new Button("Apply");
     btn.addActionListener(this);
     lowerPanel.add(btn);
     btn = new Button("Cancel");
     btn.addActionListener(this);
     lowerPanel.add(btn);

     // add all parts
     add("North", upperPanel);
     add("Center", centerPanel);
     add("South", lowerPanel);
  }

 
} // end of class

