package sv.ui;

import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

import sv.util.*;

/** 
 * SVUIC.java
 * Define Scivis user interface components
 * @version 1.0     10/15/97
 * @author Byeongseob Ki
 */

public class SVUIC {
    public final static Dimension vpad5 = new Dimension(1,5);
    public final static Dimension vpad10 = new Dimension(1,10);

    public final static Border emptyBorder0 = new EmptyBorder(0,0,0,0);
    public final static Border emptyBorder5 = new EmptyBorder(5,5,5,5);
    public final static Border emptyBorder10 = new EmptyBorder(10,10,10,10);
    public final static Border emptyBorder15 = new EmptyBorder(15,15,15,15);
    public final static Border emptyBorder20 = new EmptyBorder(20,20,20,20);

    public final static Border etchedBorder5 
               = new CompoundBorder(new EtchedBorder(), emptyBorder5);
    public final static Border etchedBorder10 
               = new CompoundBorder(new EtchedBorder(), emptyBorder10);
    public final static Border raisedBorder 
               = new BevelBorder(BevelBorder.RAISED);
    public final static Border lightLoweredBorder 
               = new BevelBorder(BevelBorder.LOWERED, Color.white, Color.gray);
    public final static Border loweredBorder 
               = new SoftBevelBorder(BevelBorder.LOWERED);
    public final static Border systemBorder 
               = new CompoundBorder(emptyBorder5, etchedBorder5);


    public static Image iconImg;


     // format mode
    public static final int ALGEBRAIC_FORMAT  = 0;
    public static final int SCIENTIFIC_FORMAT = 1;
   

  /** 
   * format float number
   * @param d formatted number
   * @param n  number of significant figures
   * @param p  precision of the number
   * @param sformat scientific format or not
   */
  public static final String formatFloat(float d, int n, int p, int sformat) {

     if (sformat == 1) {                 // scientific format
        float x = d;
        int left = n - p;
        float right = 0;
        int power;
        int exponent;
        int i;
        StringBuffer s = new StringBuffer(n+4);
        
        if(left < 0 ) {
           System.out.println("Warning: Significant < Precision");          
           String val = String.valueOf(d);
           return val.toLowerCase();
        }

        if(d < 0.0f) {
           x = -d;
           s.append("-");
        }

       if(d == 0.0f) exponent = 0;
       else         exponent = (int)(Math.floor(SpecialFunction.log10(x)));

       power = exponent - (left - 1);

       if( power < 0 ) {
          for(i=power; i<0; i++) { x *= 10.0f; }
       } else {
          for(i=0; i<power; i++) { x /= 10.0f; }
       }

       left = (int)x;
       s.append(left);

       if (p > 0) {
          s.append('.');
          right = x-left;
          for(i=0; i<p; i++) {
             right *= 10;
             if(i==p-1) right += 0.5;
             s.append((int)(right)); 
             right -= (int)right;
          }
        }

        if(power != 0) {
           s.append('e');
           if(power < 0) s.append('-');
           else          s.append('+');
           power = Math.abs(power);
           if(power > 9) {
              s.append(power);
           } else {
              s.append('0');
              s.append(power);
           }
        } else 
          s.append("e+00");


        return s.toString();


     } else {                       // algebraic format
        int ival = (int)d;

//System.out.println(xPrec + " " + yPrec + " " + timePrec + " " + p);

        if (p < 1) {
//           System.out.println("Warning: decimal < 0");
           return String.valueOf(ival);
        }

        float mult = 1.0f;
        for (int i=0; i<p; i++)
           mult *= 10.0f;
      
        int intd = (int)(d * mult);
        String val = new String(String.valueOf(intd));
        int len = val.length();

//System.out.println(d + " " + mult + " " + val + " " + len + " " + p);

        if (val.indexOf("e") > 0 || val.indexOf("E") > 0)
           return String.valueOf(ival);
      
        if (d > 0) {
      
           if (len == p) 
              val = "0." + val;
           else if (len < p) {
              int gap = p - len - 1;
              val = "0.0" + val;
              for (int i=0; i<gap; i++)
                 val = val + "0";
           } else
              val = val.substring(0, len-p) + "." + val.substring(len-p);  

        } else if (d <= 0) {
           
           if (len == p) 
              val = "-0.0" + val.substring(1);
           else if (len < p) {
              int gap = p - len + 1;
              val = val + ".";
              for (int i=0; i<gap; i++)
                 val = val + "0";
           } else if (len == p+1)
             val = val.substring(0, len-p) + "0." + val.substring(len-p); 
           else
              val = val.substring(0, len-p) + "." + val.substring(len-p);  
        }

        return val;
     } 
  }  // end of formatFloat()


   /**
    * draw dotted line
    */
    public static final void drawDottedLine(Graphics g,
               int startx, int starty, int endx, int endy, 
               int wid, int dot_size, Color color) {

        int pad = 3;
        if (dot_size < pad)
            pad = dot_size;

        int xwid = endx - startx;
        int ywid = endy - starty;
        int step = (Math.abs(xwid) > Math.abs(ywid)) ? Math.abs(xwid) : Math.abs(ywid);

        float stepx = (float)xwid / (float)step;
        float stepy = (float)ywid / (float)step;

        g.setColor(color);

        float incx = startx;
        float incy = starty;
        int tmpx = Math.round(incx);
        int tmpy = Math.round(incy);

        int prevx = tmpx;
        int prevy = tmpy;

        int lines = 0;
        int spaces = 0;

        for (int i=0; i<step; i++) {
            incx += stepx;
            incy += stepy;
            tmpx = Math.round(incx);
            tmpy = Math.round(incy);

            if (lines < dot_size) {
                drawLine(g, prevx, prevy, tmpx, tmpy, wid, color);
                lines++;
            } else {
               if (spaces < pad)
                 spaces++;
               else {
                 lines = 0;
                 spaces = 0;
               }
            }
            prevx = tmpx;
            prevy = tmpy;
        }
    }

   /**
    * We draw a thick line by computing the four corners of an equivalent
    * diagonal rectangle, and filling it.
    */
   public static final void drawLine(Graphics g,
      int startX, int startY, int endX, int endY, int wid, Color col) {
       // Make this reflect the current width and color...
       g.setColor(col);

       if (wid == 1) {
            g.drawLine(startX, startY, endX, endY);
            return;
        }

       //
       // Compute the corner points of the rectangle to fill
       //

       // The x and y deltas from the start to the end of the line...
       int dX = endX - startX;
       int dY = endY - startY;

       // The length of the line...
       double D = Math.sqrt(dX * dX + dY * dY);

       // The ratio of half the line thickness to the length of the line...
       double scale = (double)(wid) / (2 * D);

       // The x and y increments from an endpoint needed to create a rectangle...
       double ddx = -scale * (double)dY;
       double ddy = scale * (double)dX;
       if (ddx > 0) ddx += 0.5; else ddx -= 0.5;               // round off
       if (ddy > 0) ddy += 0.5; else ddy -= 0.5;               //  "
       int dx = (int)ddx;
       int dy = (int)ddy;

       // Now we can compute the corner points...

       int xPoints[] = new int[4];
       int yPoints[] = new int[4];

       xPoints[0] = startX + dx;
       yPoints[0] = startY + dy;

       xPoints[1] = startX - dx;
       yPoints[1] = startY - dy;

       xPoints[2] = endX - dx;
       yPoints[2] = endY - dy;

       xPoints[3] = endX + dx;
       yPoints[3] = endY + dy;

       // Fill the rectangle, effectively drawing a thick line
       g.fillPolygon(xPoints, yPoints, 4);
    }



} // end of class
