package sv.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;

/** 
 * TickerSetting.java
 * To get the user input
 * @version 1.0
 * @author Byeongseob Ki
 */


public class TickerSetting extends Dialog 
                     implements ActionListener, ItemListener {
    int xticks, yticks;
    int xsmallticks, ysmallticks;

    int xsig, xprec, xformat;
    int ysig, yprec, yformat;

    boolean xgrid, ygrid;

    TextField xtickField, ytickField, xsmalltickField, ysmalltickField;
    TextField xsigField, ysigField, xprecField, yprecField;
    JRadioButton xalgbtn, xscibtn, yalgbtn, yscibtn;
    JCheckBox xgridcb, ygridcb;

    String settingArr[];


    /**
     * constructor
     */
    public TickerSetting(Frame frame, int ints[], boolean bools[]) {
        super(frame, "Tick Mark Change", true);   //  make it modal

        xsigField = new TextField(3);
        ysigField = new TextField(3);

        xticks = ints[0];
        yticks = ints[1];
        xsmallticks = ints[2];
        ysmallticks = ints[3];
        xsig = ints[4];
        xprec = ints[5];
        xformat = ints[6];
        ysig = ints[7];
        yprec = ints[8];
        yformat = ints[9];

        xgrid = bools[0];
        ygrid = bools[1];

        makeGUI();
        pack();
    }

    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (command.equals("Apply")) {
            settingArr = new String[12];
            settingArr[0] = xtickField.getText().trim();
            settingArr[1] = ytickField.getText().trim();
            settingArr[2] = xsmalltickField.getText().trim();
            settingArr[3] = ysmalltickField.getText().trim();
            settingArr[4] = xsigField.getText().trim();
            settingArr[5] = xprecField.getText().trim();
            settingArr[6] = String.valueOf(xformat);
            settingArr[7] = ysigField.getText().trim();
            settingArr[8] = yprecField.getText().trim();
            settingArr[9] = String.valueOf(yformat);

            if (xgridcb.isSelected())
                settingArr[10] = "true";
            else
                settingArr[10] = "false";

            if (ygridcb.isSelected())
                settingArr[11] = "true";
            else
                settingArr[11] = "false";

        } else if (command.equals("Cancel"))
            settingArr = null;

        dispose();
    }

    public String[] getSettings() {
        return settingArr;
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getSource() instanceof JRadioButton && 
                             e.getStateChange() == ItemEvent.SELECTED) {

            JRadioButton jRadioButton = (JRadioButton)e.getSource();
            String str = jRadioButton.getText();

            if (str.equals("Algebraic(x)")) {
                xsigField.setEnabled(false);
                xformat = SVUIC.ALGEBRAIC_FORMAT;
            } else if (str.equals("Scientific(x)")) {
                xsigField.setEnabled(true);
                xformat = SVUIC.SCIENTIFIC_FORMAT;
            } if (str.equals("Algebraic(y)")) {
                ysigField.setEnabled(false);
                yformat = SVUIC.ALGEBRAIC_FORMAT;
            } if (str.equals("Scientific(y)")) {
                ysigField.setEnabled(true);
                yformat = SVUIC.SCIENTIFIC_FORMAT;
            }
        }
    }

    private void makeGUI() {
        BorderLayout bl = new BorderLayout();
        bl.setVgap(3);
        setLayout(bl);

        JTabbedPane tabbedPane = new JTabbedPane();

        JPanel xp = new JPanel();
        xp.setLayout(new BoxLayout(xp, BoxLayout.Y_AXIS));
        xp.setBorder(SVUIC.systemBorder);

        JPanel yp = new JPanel();
        yp.setLayout(new BoxLayout(yp, BoxLayout.Y_AXIS));
        yp.setBorder(SVUIC.systemBorder);

        Panel p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("Number of major ticks"));
        p.add(xtickField = new TextField(3));
        xtickField.setText(String.valueOf(xticks));
        xp.add(p);

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("No of minor ticks between major ticks"));
        p.add(xsmalltickField = new TextField(3));
        xsmalltickField.setText(String.valueOf(xsmallticks));
        xp.add(p);

        ButtonGroup group = new ButtonGroup();

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));

        xalgbtn = new JRadioButton("Algebraic(x)");
        group.add(xalgbtn);
        xalgbtn.addItemListener(this);
        p.add(xalgbtn);

        xscibtn = new JRadioButton("Scientific(x)");
        group.add(xscibtn);
        xscibtn.addItemListener(this);
        p.add(xscibtn);

        if (xformat == SVUIC.SCIENTIFIC_FORMAT)
            xscibtn.setSelected(true);
        else
            xalgbtn.setSelected(true);
        xp.add(p);

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("Significant"));
        p.add(xsigField);
        xsigField.setText(String.valueOf(xsig));

        p.add(new Label("Precision"));
        p.add(xprecField = new TextField(3));
        xprecField.setText(String.valueOf(xprec));
        xp.add(p);

        xp.add(xgridcb = new JCheckBox("Draw grid", xgrid));

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("Number of major ticks"));
        p.add(ytickField = new TextField(3));
        ytickField.setText(String.valueOf(yticks));
        yp.add(p);

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("No of minor ticks between major ticks"));
        p.add(ysmalltickField = new TextField(3));
        ysmalltickField.setText(String.valueOf(ysmallticks));
        yp.add(p);

        group = new ButtonGroup();

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        yalgbtn = new JRadioButton("Algebraic(y)");
        group.add(yalgbtn);
        yalgbtn.addItemListener(this);
        p.add(yalgbtn);

        yscibtn = new JRadioButton("Scientific(y)");
        group.add(yscibtn);
        yscibtn.addItemListener(this);
        p.add(yscibtn);

        if (yformat == SVUIC.SCIENTIFIC_FORMAT)
            yscibtn.setSelected(true);
        else
            yalgbtn.setSelected(true);
        yp.add(p);

        p = new Panel();
        p.setLayout(new FlowLayout(FlowLayout.LEFT));
        p.add(new Label("Significant"));
        p.add(ysigField);
        ysigField.setText(String.valueOf(ysig));
        p.add(new Label("Precision"));
        p.add(yprecField = new TextField(3));
        yprecField.setText(String.valueOf(yprec));
        yp.add(p);

        yp.add(ygridcb = new JCheckBox("Draw grid", ygrid));

        tabbedPane.addTab("X Axis", xp);
        tabbedPane.setSelectedIndex(0);
        tabbedPane.addTab("Y Axis", yp);

        Panel btnp = new Panel();
        btnp.setLayout(new FlowLayout(FlowLayout.RIGHT));

        Button btn = new Button("Apply");
        btn.addActionListener(this);
        btnp.add(btn);

        btn = new Button("Cancel");
        btn.addActionListener(this);
        btnp.add(btn);

        add("Center", tabbedPane);
        add("South", btnp);
    }
}

