package sv.ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

public class Wheel extends Canvas 
                  implements MouseListener, MouseMotionListener{

    private Image       wheelimage;
    private int         size;
    private int         motionsize;
    private IconMap []  wheelmap;
    private int         orientation;
    private int         state;
    private boolean     buttonup;
    private Image       offScrImg;
    private Graphics    ofScrGc;
    private WheelClient client;

    public static final int VERTICAL    = 1;
    public static final int HORIZONTAL  = 2;

    private static int startoffset [] = {0, 2, 2, 5};
    private static int endoffset   [] = {6, 4, 2, 1};

    private static final IconMap verticalmap [] = {
        new IconMap (0,  0,  20, 30),
        new IconMap (0,  30, 20, 29),
        new IconMap (20, 0,  20, 30),
        new IconMap (20, 30, 20, 29),
        new IconMap (40, 0,  20, 30),
        new IconMap (40, 30, 20, 29),
        new IconMap (60, 0,  20, 30),
        new IconMap (60, 30, 20, 29)};


    private static final IconMap horizontalmap [] = {
        new IconMap (0, 139, 30, 20),
        new IconMap (30,139, 29, 20),
        new IconMap (0, 119, 30, 20),
        new IconMap (30,119, 29, 20),
        new IconMap (0,  99, 30, 20),
        new IconMap (30, 99, 29, 20),
        new IconMap (0,  79, 30, 20),
        new IconMap (30, 79, 29, 20)};


    private static final IconMap upmap   = new IconMap( 0, 59, 20, 20);
    private static final IconMap downmap = new IconMap(20, 59, 20, 20);


    public Wheel(int orient, int wheelsize, WheelClient wclient, Image img) {
       super ();

       client = wclient;
       orientation = orient;
       size = wheelsize;
       motionsize = size - 79;
       state = 0;
       buttonup = true;
       offScrImg = null;
       wheelimage = img;

       if (orientation == VERTICAL) {
           wheelmap = verticalmap;
           setSize(20, size);
       } else {
           wheelmap = horizontalmap;
           setSize(size, 20);
       }

       addMouseMotionListener(this);
       addMouseListener(this);
    }


    public int getOrientation () {
        return orientation;
    }


    public void paint(Graphics g) {

       if (offScrImg == null) {
          offScrImg  = createImage(getSize().width, getSize().height);
          ofScrGc = offScrImg.getGraphics();
       }

       if (orientation == VERTICAL) {
            wheelmap[state*2].draw(ofScrGc, wheelimage, 0, 0, this);
            wheelmap[state*2+1].draw(ofScrGc, wheelimage, 0, 30+motionsize, this);
            if (buttonup)
                upmap.draw(ofScrGc, wheelimage, 0, size-20, this);
            else
                downmap.draw(ofScrGc, wheelimage, 0, size-20, this);


            ofScrGc.setColor(Color.lightGray);
            ofScrGc.fillRect(2, 30, 17, motionsize);
            ofScrGc.setColor(Color.darkGray);
            ofScrGc.drawLine(19, 30, 19, 29+motionsize);
            ofScrGc.drawLine(4,  30,  4, 29+motionsize);
            ofScrGc.setColor(Color.gray);
            ofScrGc.drawLine(18, 30, 18, 29+motionsize);
            ofScrGc.drawLine(0,  30,  0, 29+motionsize);
            ofScrGc.setColor(Color.white);
            ofScrGc.drawLine(1,  30,  1, 29+motionsize);
            ofScrGc.drawLine(5,  29+motionsize/6,  5, 29+5*motionsize/6);
            ofScrGc.drawLine(14, 29+motionsize/5, 14, 29+5*motionsize/6);
            ofScrGc.setColor(Color.black);
            ofScrGc.drawLine(15, 30, 15, 29+motionsize);
        } else  {
            wheelmap[state*2].draw(ofScrGc, wheelimage, 0, 0, this);
            wheelmap[state*2+1].draw(ofScrGc, wheelimage, 30+motionsize, 0, this);
            if (buttonup)
                upmap.draw(ofScrGc, wheelimage, size-20, 0, this);
            else
                downmap.draw(ofScrGc, wheelimage, size-20, 0, this);

            ofScrGc.setColor(Color.lightGray);
            ofScrGc.fillRect(30, 2, motionsize, 17);
            ofScrGc.setColor(Color.darkGray);
            ofScrGc.drawLine(30, 19, 29+motionsize, 19);
            ofScrGc.drawLine(30, 4,  29+motionsize, 4);
            ofScrGc.setColor(Color.gray);
            ofScrGc.drawLine(30, 18, 29+motionsize, 18);
            ofScrGc.drawLine(30,  0, 29+motionsize, 0);
            ofScrGc.setColor(Color.white);
            ofScrGc.drawLine(30,  1, 29+motionsize, 1);
            ofScrGc.drawLine(29+motionsize/6,  5, 29+5*motionsize/6, 5);
            ofScrGc.drawLine(29+motionsize/5, 14, 29+5*motionsize/6, 14);
            ofScrGc.setColor(Color.black);
            ofScrGc.drawLine(30, 15, 29+motionsize, 15);
        }


        int openpos  = startoffset [state];
        int closepos = motionsize - endoffset [state];
        int stepsize = 7;
        int span = motionsize;
        int spanacc = 0;
        int i;

        while (closepos > openpos && closepos-openpos >= stepsize) {
            if (orientation == VERTICAL) {
                ofScrGc.setColor(Color.black);
                ofScrGc.drawLine(6, openpos+30, 13, openpos+30);
                ofScrGc.drawLine(6, closepos+30,   13, closepos+30);
                ofScrGc.setColor(Color.white);
                for (i=0; i<stepsize-6; i++) {
                    ofScrGc.drawLine(6, openpos+29-i, 13, openpos+29-i);
                    ofScrGc.drawLine(6, closepos+29-i, 13, closepos+29-i);
                }
            } else {
                ofScrGc.setColor(Color.black);
                ofScrGc.drawLine(openpos+30, 6, openpos+30, 13);
                ofScrGc.drawLine(closepos+30, 6, closepos+30, 13);
                ofScrGc.setColor(Color.white);
                for (i=0; i<stepsize-6; i++) {
                    ofScrGc.drawLine(openpos+29-i, 6, openpos+29-i, 13);
                    ofScrGc.drawLine(closepos+29-i, 6, closepos+29-i, 13);
                }
            }

            spanacc += stepsize * 2;
            if (spanacc > span/2) {
                spanacc = 0;
                span = span/2;
                stepsize ++;
            }

            openpos  += stepsize;
            closepos -= stepsize;
        }
        g.drawImage (offScrImg, 0, 0, this);  

   }

    public void update (Graphics g) {
       paint(g);
    }


    int      prevx;
    int      prevy;
    boolean  mousedown = false;

  public void mouseClicked(MouseEvent e) {
    // NOTHING TO DO
  }

  // if mouse pressed
  public void mousePressed(MouseEvent e) {
     int x = e.getX();
     int y = e.getY();

     boolean inbutton;
     if (orientation == VERTICAL)
         inbutton = (y > size-20);
     else
         inbutton = (x > size-20);

     if (inbutton) {
         buttonup = false;
         repaint ();
      } else {
         prevx = x;
         prevy = y;
         mousedown = true;
      }
  }

  // if mouse released
  public void mouseReleased(MouseEvent e) {
     int x = e.getX();
     int y = e.getY();

     mousedown = false;
     if (! buttonup) {
         buttonup = true;
         repaint ();

         boolean inbutton;
         if (orientation == VERTICAL)
             inbutton = (y > size-20);
         else
             inbutton = (x > size-20);
         if (inbutton)
             client.onWheelReset (this);
      }
  }


  public void mouseEntered(MouseEvent e) {
    // NOTHING TO DO
  }

  public void mouseExited(MouseEvent e) {
    // NOTHING TO DO
  }


  public void mouseDragged(MouseEvent e) {
     int x = e.getX();
     int y = e.getY();

    if (mousedown == false)
        return;

    int delta;
    if (orientation == VERTICAL) {
       delta = y - prevy;
       if (y > prevy)
           state = (state + 1) % 4;
       else
           state = (state + 3) % 4;
     } else {
       delta = x - prevx;
       if (x > prevx)
           state = (state + 1) % 4;
       else
           state = (state + 3) % 4;
     }

     client.onWheelMotion (this, delta);
     prevx = x;
     prevy = y;
     repaint ();
  }

  public void mouseMoved(MouseEvent e) {
    // NOTHING TO DO
  }


} // end of class



class IconMap {
  private int x;
  private int y;
  private int width;
  private int height;


  IconMap (int posx, int posy, int w, int h) {
     x  = posx;
     y  = posy;
     width  = w;
     height = h;
  }

  void draw(Graphics g, Image image, int posx, int posy, ImageObserver iobj) {

     Graphics clipg = g.create(posx, posy, width, height);
     clipg.drawImage(image, -x, -y, iobj);
  }
}



