package sv.util;

import java.util.Hashtable;

/**
 * FilterClassLoader.java
 * Filter class loader.
 * @version 1.0   10/1/97
 * @author Byeongseob Ki
 */

public class FilterClassLoader extends ClassLoader {
   private Hashtable classes = new Hashtable();
   
   // class file btye stream
   byte[] classBytes;      

   /**
    * constructor
    */
   public FilterClassLoader(byte[] classBytes) {
      this.classBytes = classBytes;
   }

   /**
    * loading class, always resolve the loaded class
    */
   public Class loadClass(String className) throws ClassNotFoundException {
       return (loadClass(className, true));
   }

   /**
    * yes now we are going to load the class over the network
    */
   public synchronized Class loadClass(String className, boolean resolveIt)
                                               throws ClassNotFoundException {

     Class result;

     result = (Class)classes.get(className);
     if (result != null) {
       System.out.println("(ClassLoader)returning cached class: " + className);
       return result;
     }

     try {
       result = super.findSystemClass(className);
       System.out.println("(ClassLoader)returning system class in CLASSPATH: " + result.getName());
       return result;
     } catch (ClassNotFoundException e) {
       System.out.println("(ClassLoader)Not a system class: " + className);
     }

     // Define it (parse the class file)
     result = defineClass(null, classBytes, 0, classBytes.length);
     if (result == null) {
        System.out.println("(ClassLoader)class define error ...");
        throw new ClassFormatError();
     }

     // Resolve if necessary
     if (resolveIt) resolveClass(result);

     // Done
     classes.put(className, result);
     System.out.println("(ClassLoader)returning loaded class over network: " + className);
     return result;
   }

} // end of class

