package sv.util;


public class NewtonMethod {
  public int nControls;
  public float[] xcontrols;
  public float[] ycontrols;
  private float[] coef;

  public NewtonMethod(float[] x, float[] y) {
     nControls = x.length;
     xcontrols = new float[nControls];
     ycontrols = new float[nControls];

     System.arraycopy(x, 0, xcontrols, 0, nControls);
     System.arraycopy(y, 0, ycontrols, 0, nControls);

     computeCoef();
  }


  public float computePoint(float u) {
     float temp;
     
     temp = coef[nControls-1];

     for (int i=nControls-2; i>=0; i--) 
        temp = temp * (u-xcontrols[i]) + coef[i];

     return temp;
  }

  public void computeCoef() {
    coef = new float[nControls];

    for (int i=0; i<nControls; i++) 
      coef[i] = ycontrols[i];
   
    for (int j=1; j<nControls; j++) {
      for (int i=nControls-1; i>=j; i--) 
         coef[i] = (coef[i] - coef[i-1]) / (xcontrols[i] - xcontrols[i-j]);
    }
  }


  public void addPoint(float xval, float yval) {
      nControls++;
      float[] xtmp = new float[nControls];
      float[] ytmp = new float[nControls];

      int index = 0;
      xtmp[0] = xcontrols[0];
      ytmp[0] = ycontrols[0];
      for (int i=1; i<nControls; i++) {
	if (xval <= xcontrols[i]) {
           xtmp[i] = (xcontrols[i-1] + xcontrols[i]) / 2.0f;
           ytmp[i] = yval;
           index = index+2;
           break;
        } else {
           xtmp[i] = xcontrols[i];
           ytmp[i] = ycontrols[i];
           index++;
        }
      }
      for (int i=index; i<nControls; i++) {
         xtmp[i] = xcontrols[i-1];
         ytmp[i] = ycontrols[i-1];
      }
      
      xcontrols = new float[nControls];
      ycontrols = new float[nControls];
      System.arraycopy(xtmp, 0, xcontrols, 0, nControls);
      System.arraycopy(ytmp, 0, ycontrols, 0, nControls);

      computeCoef();
  }



  public void removePoint(float xval) {
      if (xval <= 0.0f || xval >= 1.0f || (nControls < 3)) {
         System.out.println("Can't remove this point.");
         return;
      }

      nControls--;
      float[] xtmp = new float[nControls];
      float[] ytmp = new float[nControls];

      int index = 0;
      xtmp[0] = xcontrols[0];
      ytmp[0] = ycontrols[0];
      for (int i=1; i<nControls; i++) {
	if (xval <= xcontrols[i]) {
           xtmp[i] = xcontrols[i+1];
           ytmp[i] = ycontrols[i+1];
           index = index+2;
           break;
        } else {
           xtmp[i] = xcontrols[i];
           ytmp[i] = ycontrols[i];
           index++;
        }
      }
      for (int i=index; i<nControls; i++) {
         xtmp[i] = xcontrols[i+1];
         ytmp[i] = ycontrols[i+1];
      }
      
      xcontrols = new float[nControls];
      ycontrols = new float[nControls];
      System.arraycopy(xtmp, 0, xcontrols, 0, nControls);
      System.arraycopy(ytmp, 0, ycontrols, 0, nControls);

      computeCoef();
  }

}
