package sv.util;

/** 
 * Sorter.java
 * Sort by using various sort algorithm
 * @version 1.0
 * @author Byeongseob Ki
 */

public class Sorter {

  /**
   * constructor(nothing to do)
   */
  public Sorter() {

  }  

  /**
   * Sort a float array. The first time you call
   * this function it will be with the parameters 0, arr.length - 1.
   * @param arr a float array
   * @param lo0 left boundary of array partition
   * @param hi0 right boundary of array partition
   */
  public void QSortFloat(float[] arr, int lo0, int hi0) {
     int lo = lo0;
     int hi = hi0;
     float mid;

     if (hi0 > lo0) {

        // Arbitrarily establishing partition element 
        // as the midpoint of the array.
        mid = arr[(lo0+hi0)/2];

        // loop through the array until indices cross
        while (lo <= hi) {
           // find the first element that is greater than or equal to 
           // the partition element starting from the left Index.
           while ((lo < hi0) && (arr[lo] < mid))
	       ++lo;

           // find an element that is smaller than or equal to 
           // the partition element starting from the right Index.
           while (( hi > lo0) && (arr[hi] > mid))
	      --hi;

           // if the indexes have not crossed, swap
           if (lo <= hi)  {
              float T = arr[lo]; 
              arr[lo] = arr[hi];
              arr[hi] = T;
              ++lo;
              --hi;
           }
        }

        // If the right index has not reached the left side of array
        // must now sort the left partition.
        if (lo0 < hi)
           QSortFloat(arr, lo0, hi);

        // If the left index has not reached the right side of array
        // must now sort the right partition.
        if (lo < hi0)
           QSortFloat(arr, lo, hi0);

      }
   }

  /**
   * Sort a float array but it also swap attached array
   * @param arr a float array
   * @param attach  an atached array
   * @param lo0 left boundary of array partition
   * @param hi0 right boundary of array partition
   */
  public void QSortFloat2(float[] arr, float[] attach, int lo0, int hi0) {
     int lo = lo0;
     int hi = hi0;
     float mid;

     if (hi0 > lo0) {

        // Arbitrarily establishing partition element 
        // as the midpoint of the array.
        mid = arr[(lo0+hi0)/2];

        // loop through the array until indices cross
        while (lo <= hi) {
           // find the first element that is greater than or equal to 
           // the partition element starting from the left Index.
           while ((lo < hi0) && (arr[lo] < mid))
	       ++lo;

           // find an element that is smaller than or equal to 
           // the partition element starting from the right Index.
           while (( hi > lo0) && (arr[hi] > mid))
	      --hi;

           // if the indexes have not crossed, swap
           if (lo <= hi)  {
              float S = arr[lo]; 
              float T = attach[lo];
              arr[lo] = arr[hi];
              attach[lo] = attach[hi];
              arr[hi] = S;
              attach[hi] = T;
              ++lo;
              --hi;
           }
        }

        // If the right index has not reached the left side of array
        // must now sort the left partition.
        if (lo0 < hi)
           QSortFloat2(arr, attach, lo0, hi);

        // If the left index has not reached the right side of array
        // must now sort the right partition.
        if (lo < hi0)
           QSortFloat2(arr, attach, lo, hi0);

      }
   }


}

